* This file is a part of NMSSMCALC-nuSS 
* It contains:
* subroutine issSTU_check: compute STU parameters of the NMSSM-ISS
* double precision function issDZself: compute derivative of Z-selfenergy of the NMSSM-ISS
* double precision function issYself: compute gamma-selfenergy  of the NMSSM-ISS
* double precision function issDYself: compute derivative of Z-selfenergy  of the NMSSM-ISS
* double precision function issZself: compute Z-selfenergy  of the NMSSM-ISS
* double precision function issWself: compute W-selfenergy  of the NMSSM-ISS
* double precision function issDWself: compute derivative of W-selfenergy  of the NMSSM-ISS
* subroutine issSTU_UVcheck: check UV finite of STU parameters of the NMSSM-ISS
c last modified by ND 7/2021
c*********************************************************************c
#include "def.h"
      subroutine issSTU_check(icheck)
      implicit none
      integer i,j,gt1,gt2,gt3,gt4,icheck

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
           

      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3


      double precision issZself,issyself,issWself,issyzself,
     &                issdzself,issdyself,issdwself,issdyzself,
     &                S,T,U
      double precision ismZself,ismyself,ismWself,ismyzself,
     &                ismdzself,ismdyself,ismdwself,ismdyzself

      double precision selfz0,selfw0,selfzmz,selfwmw,nZself
#include "issnmssm.h"
#include "HdecayTCLs.inc"
       alfa= e**2/4D0/pi
       MH2=125D0**2

       selfz0=(issZself(0D0)-ismZself(0D0))/mz**2 
       selfw0=(isswself(0D0)-ismwself(0D0))/mw**2
       
       selfzmz=(issZself(mz**2)-ismZself(mz**2))/mz**2
       selfwmw=(isswself(mw**2)-ismwself(mw**2))/mw**2


! using definition of S,T,U in 10.72a,10.72b,10.72c in 10. Electroweak Model and Constraints on New Physics
! Revised March 2020 by J. Erler (IF-UNAM; U. of Mainz) and A. Freitas (Pittsburg U.) in PDG
! web https://pdg.lbl.gov/2020/reviews/contents_sports.html

       T= (selfw0-selfz0)/alfa
       S=((selfzmz-selfz0) -(cw**2-sw**2)/cw/sw*(issyZself(mz**2)+ismyZself(mz**2))/mz**2
     &     -(issyself(mz**2)-ismyself(mz**2))/mz**2 )*4*sw**2*cw**2 /alfa  
       U =( (selfwmw-selfw0) - cw/sw*(issyZself(mz**2)+ismyZself(mz**2))/mz**2-
     &      (issyself(mz**2)-ismyself(mz**2))/mz**2 )*4*sw**2/alfa-S
       para_S=S;para_T=T;para_U=U
c       print*, ",issZself(0D0),ismZself(0D0)",issZself(0D0),ismZself(0D0)
c       print*, "selfz0,selfw0",selfz0,selfw0, (selfw0-selfz0)
c       print*, "issyZself(mz**2)-ismyZself(mz**2)",issyZself(mz**2),ismyZself(mz**2)
c       print*, "alfa",alpha,mz,mw
c       print*, "S,T,U=[-0.11,0.09],[-0.09,0.15],[-0.11,0.13] ",S,T,U
c       stop
        icheck=0
        if (S .gt. 0.09 .or. S.lt.-0.11) icheck=3
        if (T .gt. 0.15 .or. T.lt.-0.09) icheck=3
        if (U .gt. 0.13 .or. U.lt.-0.11) icheck=3
      end
!######################################################################
      subroutine issSTU_check_full()
      implicit none
      integer i,j,gt1,gt2,gt3,gt4

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
           

      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3


      double precision issZself,issyself,issWself,issyzself,
     &                issdzself,issdyself,issdwself,issdyzself,
     &                S,T,U
      double precision selfz,selfy,selfw,selfyz,
     &                 dselfz,dselfy,dselfw,dselfyz,selfQQ,self3Q,
     &                 self33,self11,dselfQQ,dself3Q,
     &                 dself33,dself11 
#include "issnmssm.h"
#include "HdecayTCLs.inc"

       alfa= e**2/4D0/pi
       selfz=issZself(0D0)
       selfw=isswself(0D0)
       selfy=issyself(0D0)
       selfyz=-issyZself(0D0)

       dselfz=(issZself(MZ2)-issZself(0D0))/MZ2
       dselfw=(isswself(MW2)-isswself(0D0))/MW2
       dselfy=issdyself(0D0)
       dselfyz=-issdyZself(0D0)

c       selfQQ= selfy/e**2
c       self3Q=(cw*selfyz*sw)/e**2 + (selfy*sw**2)/e**2
       self11=(selfw*sw**2)/e**2
       self33=(cw**2*selfz*sw**2)/e**2 + 
     -    (2*cw*selfyz*sw**3)/e**2 + (selfy*sw**4)/e**2

c       dselfQQ= dselfy/e**2
       dself3Q=(cw*dselfyz*sw)/e**2 + (dselfy*sw**2)/e**2
       dself11=(dselfw*sw**2)/e**2
       dself33=(cw**2*dselfz*sw**2)/e**2 + 
     -    (2*cw*dselfyz*sw**3)/e**2 + (dselfy*sw**4)/e**2


       S= 16*pi*(dself33 -dself3Q)
       T= 4*pi/sw**2/cw**2/MZ2*(self11 -self33)
       U = 16*pi*(dself11 -dself33)
       para_S=S;para_T=T;para_U=U
c#define details
#ifdef details
       print*, "=======================NMSSM==========================="
       print*, "Follow definitions in  M. E. Peskin and T. Takeuchi 1992, "
       print*, "T", (selfw/MW2-selfz/MZ2 ) /alfa

       print*, "S",4*sw**2*cw**2*4*pi/e**2 *(dselfz - (cw**2-sw**2)/cw/sw*dselfyz -dselfy)
       print*, "U", 4*sw**2*4*pi/e**2*(dselfw - cw**2*dselfz - sw**2*dselfy- 2*cw*sw*dselfyz)
       print*, "selfz,selfw,selfy,selfyz",selfz,selfw,selfy,selfyz
       print*, "dselfz,dselfw,dselfy,dselfyz",dselfz,dselfw,dselfy,dselfyz
       print*, "S,T,U= ",S,T,U
       stop
#endif
      end

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function issZself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      self=0D0
       self=self+CWWZ**2*(4*myA0(MW2)-(2*MW2+5*P2)*
     &  myB0i(bb0,P2,MW2,MW2)-2*(4*myB0i(bb00,P2,MW2,MW2)+P2*
     &  myB0i(bb1,P2,MW2,MW2)))

           do I183 = 1,18
       self=self+(CSvSvVZVZ(I183,I183)*myA0(MSv182(I183)))/2d0
           enddo


           do I183 = 1,18
           do I184 = 1,18
       self=self+2*CSvSvVZ(I183,I184)**2*myB0i(bb00,P2,MSv182(I183),MSv182(I184))
           enddo
           enddo


           do I2G3 = 1,2
       self=self+(CHpHmZZ(I2G3,I2G3)*myA0(MassHpm2(I2G3))+2*
     &  CHmWpZ(I2G3)**2*myB0i(bb0,P2,MW2,MassHpm2(I2G3))-4*CHpHmZ**2*
     &  myB0i(bb00,P2,MassHpm2(I2G3),MassHpm2(I2G3)))
           enddo


           do I2G3 = 1,2
           do I2G4 = 1,2
       self=self+2*((CF12ZL(I2G4,I2G3)*(CF12ZR(I2G3,I2G4)*MCha(I2G3)*
     &  MCha(I2G4)-CF12ZL(I2G3,I2G4)*MCha2(I2G3))+CF12ZR(I2G4,I2G3)*
     &  (CF12ZL(I2G3,I2G4)*MCha(I2G3)*MCha(I2G4)-CF12ZR(I2G3,I2G4)*
     &  MCha2(I2G3)))*myB0i(bb0,P2,MCha2(I2G3),MCha2(I2G4))-
     &  (CF12ZL(I2G3,I2G4)*CF12ZL(I2G4,I2G3)+CF12ZR(I2G3,I2G4)*
     &  CF12ZR(I2G4,I2G3))*(myA0(MCha2(I2G4))-2*
     &  myB0i(bb00,P2,MCha2(I2G3),MCha2(I2G4))+P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G4))))
           enddo
           enddo

           do I3G3 = 1,3
       self=self+2*(-((ctautauZL-ctautauZR)**2*MFe2(I3G3)*
     &  myB0i(bb0,P2,MFe2(I3G3),MFe2(I3G3)))-3*((CddZL**2+CddZR**2)*
     &  myA0(MFd2(I3G3))+(CuuZL**2+CuuZR**2)*myA0(MFu2(I3G3))+(CddZL-
     &  CddZR)**2*MFd2(I3G3)*myB0i(bb0,P2,MFd2(I3G3),MFd2(I3G3))+
     &  (CuuZL-CuuZR)**2*MFu2(I3G3)*myB0i(bb0,P2,MFu2(I3G3),MFu2(I3G3))
     &  )-(ctautauZL**2+ctautauZR**2)*(myA0(MFe2(I3G3))-2*
     &  myB0i(bb00,P2,MFe2(I3G3),MFe2(I3G3)))+6*((CddZL**2+CddZR**2)*
     &  myB0i(bb00,P2,MFd2(I3G3),MFd2(I3G3))+(CuuZL**2+CuuZR**2)*
     &  myB0i(bb00,P2,MFu2(I3G3),MFu2(I3G3)))+8*P2*(-((ctautauZL**2+
     &  ctautauZR**2)*myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3)))/8d0-(3*
     &  ((CddZL**2+CddZR**2)*myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+
     &  (CuuZL**2+CuuZR**2)*myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3))))/8d0))
           enddo

           do I5G3 = 1,5
           do I5G4 = 1,5
       self=self+((CF11ZL(I5G4,I5G3)*(CF11ZR(I5G3,I5G4)*MNeu(I5G3)*
     &  MNeu(I5G4)-CF11ZL(I5G3,I5G4)*MNeu2(I5G3))+CF11ZR(I5G4,I5G3)*
     &  (CF11ZL(I5G3,I5G4)*MNeu(I5G3)*MNeu(I5G4)-CF11ZR(I5G3,I5G4)*
     &  MNeu2(I5G3)))*myB0i(bb0,P2,MNeu2(I5G3),MNeu2(I5G4))-
     &  (CF11ZL(I5G3,I5G4)*CF11ZL(I5G4,I5G3)+CF11ZR(I5G3,I5G4)*
     &  CF11ZR(I5G4,I5G3))*(myA0(MNeu2(I5G4))-2*
     &  myB0i(bb00,P2,MNeu2(I5G3),MNeu2(I5G4))+P2*
     &  myB0i(bb1,P2,MNeu2(I5G3),MNeu2(I5G4))))
           enddo
           enddo


           do I6G3 = 1,6
       self=self+(-(IC*ChhZZ(I6G3,I6G3)*myA0(Masshh2(I6G3)))/2d0+3*
     &  CSdSdZZ(I6G3,I6G3)*myA0(MSd2(I6G3))+CSeSeZZ(I6G3,I6G3)*
     &  myA0(MSe2(I6G3))+3*CSuSuZZ(I6G3,I6G3)*myA0(MSu2(I6G3))-
     &  ChZZ(I6G3)**2*myB0i(bb0,P2,MZ2,Masshh2(I6G3)))
           enddo


           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self-2*(ChhZ(I6G3,I6G4)**2*myB0i(bb00,P2,Masshh2(I6G3)
     &  ,Masshh2(I6G4))+6*CsdmsdpZ(I6G3,I6G4)*CsdmsdpZ(I6G4,I6G3)*
     &  myB0i(bb00,P2,MSd2(I6G3),MSd2(I6G4))+2*CsemsepZ(I6G3,I6G4)*
     &  CsemsepZ(I6G4,I6G3)*myB0i(bb00,P2,MSe2(I6G3),MSe2(I6G4))+6*
     &  CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3)*
     &  myB0i(bb00,P2,MSu2(I6G3),MSu2(I6G4)))
           enddo
           enddo


           do I9G3 = 1,9
           do I9G4 = 1,9
       self=self+((CLFvFvVZ(I9G4,I9G3)*(CRFvFvVZ(I9G3,I9G4)*mv9(I9G3)*
     &  mv9(I9G4)-CLFvFvVZ(I9G3,I9G4)*mv92(I9G3))+CRFvFvVZ(I9G4,I9G3)*
     &  (CLFvFvVZ(I9G3,I9G4)*mv9(I9G3)*mv9(I9G4)-CRFvFvVZ(I9G3,I9G4)*
     &  mv92(I9G3)))*myB0i(bb0,P2,mv92(I9G3),mv92(I9G4))-
     &  (CLFvFvVZ(I9G3,I9G4)*CLFvFvVZ(I9G4,I9G3)+CRFvFvVZ(I9G3,I9G4)*
     &  CRFvFvVZ(I9G4,I9G3))*(myA0(mv92(I9G4))-2*
     &  myB0i(bb00,P2,mv92(I9G3),mv92(I9G4))+P2*myB0i(bb1,P2,mv92(I9G3)
     &  ,mv92(I9G4))))
           enddo
           enddo


       issZself=self*fac
      end

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function issDZself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself-(CWWZ**2*(5*myB0i(bb0,P2,MW2,MW2)+2*
     &  myB0i(bb1,P2,MW2,MW2)+2*MW2*myB0i(dbb0,P2,MW2,MW2)+5*P2*
     &  myB0i(dbb0,P2,MW2,MW2)+8*myB0i(dbb00,P2,MW2,MW2)+2*P2*
     &  myB0i(dbb1,P2,MW2,MW2)))
           do I183 = 1,18
           do I184 = 1,18
       Dself=Dself+2*CSvSvVZ(I183,I184)**2*myB0i(dbb00,P2,MSv182(I183)
     &  ,MSv182(I184))
           enddo
           enddo
           do I2G3 = 1,2
       Dself=Dself+(2*CHmWpZ(I2G3)**2*myB0i(dbb0,P2,MW2,MassHpm2(I2G3))
     &  -4*CHpHmZ**2*myB0i(dbb00,P2,MassHpm2(I2G3),MassHpm2(I2G3)))
           enddo
           do I2G3 = 1,2
           do I2G4 = 1,2
       Dself=Dself+2*((CF12ZL(I2G4,I2G3)*(CF12ZR(I2G3,I2G4)*MCha(I2G3)*
     &  MCha(I2G4)-CF12ZL(I2G3,I2G4)*MCha2(I2G3))+CF12ZR(I2G4,I2G3)*
     &  (CF12ZL(I2G3,I2G4)*MCha(I2G3)*MCha(I2G4)-CF12ZR(I2G3,I2G4)*
     &  MCha2(I2G3)))*myB0i(dbb0,P2,MCha2(I2G3),MCha2(I2G4))-
     &  (CF12ZL(I2G3,I2G4)*CF12ZL(I2G4,I2G3)+CF12ZR(I2G3,I2G4)*
     &  CF12ZR(I2G4,I2G3))*(myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G4))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G4))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G4))))
           enddo
           enddo
           do I3G3 = 1,3
       Dself=Dself+2*(-((ctautauZL**2+ctautauZR**2)*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3)))-3*((CddZL**2+CddZR**2)*
     &  myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+(CuuZL**2+CuuZR**2)*
     &  myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3)))-(ctautauZL-ctautauZR)**2*
     &  MFe2(I3G3)*myB0i(dbb0,P2,MFe2(I3G3),MFe2(I3G3))-3*((CddZL-
     &  CddZR)**2*MFd2(I3G3)*myB0i(dbb0,P2,MFd2(I3G3),MFd2(I3G3))+
     &  (CuuZL-CuuZR)**2*MFu2(I3G3)*myB0i(dbb0,P2,MFu2(I3G3),MFu2(I3G3)
     &  ))+2*(ctautauZL**2+ctautauZR**2)*myB0i(dbb00,P2,MFe2(I3G3)
     &  ,MFe2(I3G3))+6*((CddZL**2+CddZR**2)*myB0i(dbb00,P2,MFd2(I3G3)
     &  ,MFd2(I3G3))+(CuuZL**2+CuuZR**2)*myB0i(dbb00,P2,MFu2(I3G3)
     &  ,MFu2(I3G3)))+8*P2*(-((ctautauZL**2+ctautauZR**2)*
     &  myB0i(dbb1,P2,MFe2(I3G3),MFe2(I3G3)))/8d0-(3*((CddZL**2+CddZR**
     &  2)*myB0i(dbb1,P2,MFd2(I3G3),MFd2(I3G3))+(CuuZL**2+CuuZR**2)*
     &  myB0i(dbb1,P2,MFu2(I3G3),MFu2(I3G3))))/8d0))
           enddo
           do I5G3 = 1,5
           do I5G4 = 1,5
       Dself=Dself+((CF11ZL(I5G4,I5G3)*(CF11ZR(I5G3,I5G4)*MNeu(I5G3)*
     &  MNeu(I5G4)-CF11ZL(I5G3,I5G4)*MNeu2(I5G3))+CF11ZR(I5G4,I5G3)*
     &  (CF11ZL(I5G3,I5G4)*MNeu(I5G3)*MNeu(I5G4)-CF11ZR(I5G3,I5G4)*
     &  MNeu2(I5G3)))*myB0i(dbb0,P2,MNeu2(I5G3),MNeu2(I5G4))-
     &  (CF11ZL(I5G3,I5G4)*CF11ZL(I5G4,I5G3)+CF11ZR(I5G3,I5G4)*
     &  CF11ZR(I5G4,I5G3))*(myB0i(bb1,P2,MNeu2(I5G3),MNeu2(I5G4))-2*
     &  myB0i(dbb00,P2,MNeu2(I5G3),MNeu2(I5G4))+P2*
     &  myB0i(dbb1,P2,MNeu2(I5G3),MNeu2(I5G4))))
           enddo
           enddo
           do I6G3 = 1,6
       Dself=Dself-(ChZZ(I6G3)**2*myB0i(dbb0,P2,MZ2,Masshh2(I6G3)))
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself-2*(ChhZ(I6G3,I6G4)**2*myB0i(dbb00,P2,Masshh2(I6G3)
     &  ,Masshh2(I6G4))+6*CsdmsdpZ(I6G3,I6G4)*CsdmsdpZ(I6G4,I6G3)*
     &  myB0i(dbb00,P2,MSd2(I6G3),MSd2(I6G4))+2*CsemsepZ(I6G3,I6G4)*
     &  CsemsepZ(I6G4,I6G3)*myB0i(dbb00,P2,MSe2(I6G3),MSe2(I6G4))+6*
     &  CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3)*
     &  myB0i(dbb00,P2,MSu2(I6G3),MSu2(I6G4)))
           enddo
           enddo
           do I9G3 = 1,9
           do I9G4 = 1,9
       Dself=Dself+((CLFvFvVZ(I9G4,I9G3)*(CRFvFvVZ(I9G3,I9G4)*mv9(I9G3)
     &  *mv9(I9G4)-CLFvFvVZ(I9G3,I9G4)*mv92(I9G3))+CRFvFvVZ(I9G4,I9G3)*
     &  (CLFvFvVZ(I9G3,I9G4)*mv9(I9G3)*mv9(I9G4)-CRFvFvVZ(I9G3,I9G4)*
     &  mv92(I9G3)))*myB0i(dbb0,P2,mv92(I9G3),mv92(I9G4))-
     &  (CLFvFvVZ(I9G3,I9G4)*CLFvFvVZ(I9G4,I9G3)+CRFvFvVZ(I9G3,I9G4)*
     &  CRFvFvVZ(I9G4,I9G3))*(myB0i(bb1,P2,mv92(I9G3),mv92(I9G4))-2*
     &  myB0i(dbb00,P2,mv92(I9G3),mv92(I9G4))+P2*
     &  myB0i(dbb1,P2,mv92(I9G3),mv92(I9G4))))
           enddo
           enddo
      issDZself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function issYself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      self=0D0
       self=self+e**2*(4*myA0(MW2)-(2*MW2+5*P2)*myB0i(bb0,P2,MW2,MW2)-
     &  2*(4*myB0i(bb00,P2,MW2,MW2)+P2*myB0i(bb1,P2,MW2,MW2)))
           do I2G3 = 1,2
       self=self+2*(e**2*myA0(MassHpm2(I2G3))-2*e**2*myA0(MCha2(I2G3))+
     &  CHmWpA(I2G3)**2*myB0i(bb0,P2,MW2,MassHpm2(I2G3))-2*e**2*
     &  myB0i(bb00,P2,MassHpm2(I2G3),MassHpm2(I2G3))+4*e**2*
     &  myB0i(bb00,P2,MCha2(I2G3),MCha2(I2G3))-2*e**2*P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3)))
           enddo
           do I3G3 = 1,3
       self=self+(-4*e**2*(myA0(MFd2(I3G3))+3*myA0(MFe2(I3G3))+4*
     &  myA0(MFu2(I3G3))-2*myB0i(bb00,P2,MFd2(I3G3),MFd2(I3G3))-6*
     &  myB0i(bb00,P2,MFe2(I3G3),MFe2(I3G3))-8*myB0i(bb00,P2,MFu2(I3G3)
     &  ,MFu2(I3G3))+P2*myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+3*P2*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))+4*P2*
     &  myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3))))/3d0
           enddo
           do I6G3 = 1,6
       self=self+(2*e**2*(myA0(MSd2(I6G3))+3*myA0(MSe2(I6G3))+4*
     &  myA0(MSu2(I6G3))-2*myB0i(bb00,P2,MSd2(I6G3),MSd2(I6G3))-6*
     &  myB0i(bb00,P2,MSe2(I6G3),MSe2(I6G3))-8*myB0i(bb00,P2,MSu2(I6G3)
     &  ,MSu2(I6G3))))/3d0
           enddo
      issYself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function issDYself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself-(e**2*(5*myB0i(bb0,P2,MW2,MW2)+2*
     &  myB0i(bb1,P2,MW2,MW2)+2*MW2*myB0i(dbb0,P2,MW2,MW2)+5*P2*
     &  myB0i(dbb0,P2,MW2,MW2)+8*myB0i(dbb00,P2,MW2,MW2)+2*P2*
     &  myB0i(dbb1,P2,MW2,MW2)))
           do I2G3 = 1,2
       Dself=Dself+(-4*e**2*myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3))+2*
     &  CHmWpA(I2G3)**2*myB0i(dbb0,P2,MW2,MassHpm2(I2G3))-4*e**2*
     &  (myB0i(dbb00,P2,MassHpm2(I2G3),MassHpm2(I2G3))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G3))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G3))))
           enddo
           do I3G3 = 1,3
       Dself=Dself+(-4*e**2*(myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+3*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))+4*myB0i(bb1,P2,MFu2(I3G3)
     &  ,MFu2(I3G3))-2*myB0i(dbb00,P2,MFd2(I3G3),MFd2(I3G3))-6*
     &  myB0i(dbb00,P2,MFe2(I3G3),MFe2(I3G3))-8*
     &  myB0i(dbb00,P2,MFu2(I3G3),MFu2(I3G3))+P2*
     &  myB0i(dbb1,P2,MFd2(I3G3),MFd2(I3G3))+3*P2*
     &  myB0i(dbb1,P2,MFe2(I3G3),MFe2(I3G3))+4*P2*
     &  myB0i(dbb1,P2,MFu2(I3G3),MFu2(I3G3))))/3d0
           enddo
           do I6G3 = 1,6
       Dself=Dself+(-4*e**2*(myB0i(dbb00,P2,MSd2(I6G3),MSd2(I6G3))+3*
     &  myB0i(dbb00,P2,MSe2(I6G3),MSe2(I6G3))+4*
     &  myB0i(dbb00,P2,MSu2(I6G3),MSu2(I6G3))))/3d0
           enddo
      issDYself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function issYZself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision ZH5(5,5),ZH(6,6)
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      self=0D0
       self=self+((-2*CWWZ*e+3*g2**2*S2W)*myA0(MW2)-CWWZ*e*((2*MW2+5*
     &  P2)*myB0i(bb0,P2,MW2,MW2)+8*myB0i(bb00,P2,MW2,MW2)+2*P2*
     &  myB0i(bb1,P2,MW2,MW2)))
           do I2G3 = 1,2
       self=self+16*(((-1+2*CW2)*e**2*myA0(MassHpm2(I2G3)))/(16d0*CW*
     &  SW)-(CHmWpA(I2G3)*CHmWpZ(I2G3)*myB0i(bb0,P2,MW2,MassHpm2(I2G3))
     &  )/8d0+e*(-(CHpHmZ*myB0i(bb00,P2,MassHpm2(I2G3),MassHpm2(I2G3)))
     &  /4d0-((CF12ZL(I2G3,I2G3)+CF12ZR(I2G3,I2G3))*(myA0(MCha2(I2G3))-
     &  2*myB0i(bb00,P2,MCha2(I2G3),MCha2(I2G3))+P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3))))/8d0))
           enddo

           do I3G3 = 1,3
       self=self+2*e*(-((CddZL+CddZR)*myA0(MFd2(I3G3)))-(ctautauZL+
     &  ctautauZR)*myA0(MFe2(I3G3))+2*((CuuZL+CuuZR)*myA0(MFu2(I3G3))+
     &  (CddZL+CddZR)*myB0i(bb00,P2,MFd2(I3G3),MFd2(I3G3))+(ctautauZL+
     &  ctautauZR)*myB0i(bb00,P2,MFe2(I3G3),MFe2(I3G3)))-4*(CuuZL+
     &  CuuZR)*myB0i(bb00,P2,MFu2(I3G3),MFu2(I3G3))+P2*(-((CddZL+CddZR)
     &  *myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3)))-(ctautauZL+ctautauZR)*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))+2*(CuuZL+CuuZR)*
     &  myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3))))
           enddo

           do I6G3 = 1,6
       self=self-(e*(-(e*myA0(MSd2(I6G3))*(2*CW2+DCONJG(ZD(I6G3,1))*
     &  ZD(I6G3,1)-2*DCONJG(ZD(I6G3,2))*ZD(I6G3,2)+
     &  DCONJG(ZD(I6G3,3))*ZD(I6G3,3)-2*DCONJG(ZD(I6G3,4))*
     &  ZD(I6G3,4)+DCONJG(ZD(I6G3,5))*ZD(I6G3,5)-2*
     &  DCONJG(ZD(I6G3,6))*ZD(I6G3,6)))+3*e*myA0(MSe2(I6G3))*(-2*CW2+
     &  DCONJG(ZE(I6G3,1))*ZE(I6G3,1)+2*DCONJG(ZE(I6G3,2))*
     &  ZE(I6G3,2)+DCONJG(ZE(I6G3,3))*ZE(I6G3,3)+2*
     &  DCONJG(ZE(I6G3,4))*ZE(I6G3,4)+DCONJG(ZE(I6G3,5))*ZE(I6G3,5)
     &  +2*DCONJG(ZE(I6G3,6))*ZE(I6G3,6))+2*(6*CW*SW*
     &  (CsdmsdpZ(I6G3,I6G3)*myB0i(bb00,P2,MSd2(I6G3),MSd2(I6G3))+
     &  CsemsepZ(I6G3,I6G3)*myB0i(bb00,P2,MSe2(I6G3),MSe2(I6G3))-2*
     &  CsupsumZ(I6G3,I6G3)*myB0i(bb00,P2,MSu2(I6G3),MSu2(I6G3)))+e*
     &  myA0(MSu2(I6G3))*(-4*CW2+DCONJG(ZU(I6G3,1))*ZU(I6G3,1)+4*
     &  DCONJG(ZU(I6G3,2))*ZU(I6G3,2)+DCONJG(ZU(I6G3,3))*ZU(I6G3,3)
     &  +4*DCONJG(ZU(I6G3,4))*ZU(I6G3,4)+DCONJG(ZU(I6G3,5))*
     &  ZU(I6G3,5)+4*DCONJG(ZU(I6G3,6))*ZU(I6G3,6)))))/(3d0*CW*SW)
           enddo

      issYZself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function issDYZself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself

      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself-(CWWZ*e*(5*myB0i(bb0,P2,MW2,MW2)+2*
     &  myB0i(bb1,P2,MW2,MW2)+2*MW2*myB0i(dbb0,P2,MW2,MW2)+5*P2*
     &  myB0i(dbb0,P2,MW2,MW2)+8*myB0i(dbb00,P2,MW2,MW2)+2*P2*
     &  myB0i(dbb1,P2,MW2,MW2)))

           do I2G3 = 1,2
       Dself=Dself+16*(-(CHmWpA(I2G3)*CHmWpZ(I2G3)*
     &  myB0i(dbb0,P2,MW2,MassHpm2(I2G3)))/8d0+e*(-(CHpHmZ*
     &  myB0i(dbb00,P2,MassHpm2(I2G3),MassHpm2(I2G3)))/4d0-
     &  ((CF12ZL(I2G3,I2G3)+CF12ZR(I2G3,I2G3))*
     &  (myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G3))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G3))))/8d0))
           enddo
           do I3G3 = 1,3
       Dself=Dself+2*e*(-((CddZL+CddZR)*myB0i(bb1,P2,MFd2(I3G3)
     &  ,MFd2(I3G3)))-(ctautauZL+ctautauZR)*myB0i(bb1,P2,MFe2(I3G3)
     &  ,MFe2(I3G3))+2*(CuuZL+CuuZR)*myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3)
     &  )+2*((CddZL+CddZR)*myB0i(dbb00,P2,MFd2(I3G3),MFd2(I3G3))+
     &  (ctautauZL+ctautauZR)*myB0i(dbb00,P2,MFe2(I3G3),MFe2(I3G3)))-4*
     &  (CuuZL+CuuZR)*myB0i(dbb00,P2,MFu2(I3G3),MFu2(I3G3))+P2*(-
     &  ((CddZL+CddZR)*myB0i(dbb1,P2,MFd2(I3G3),MFd2(I3G3)))-
     &  (ctautauZL+ctautauZR)*myB0i(dbb1,P2,MFe2(I3G3),MFe2(I3G3))+2*
     &  (CuuZL+CuuZR)*myB0i(dbb1,P2,MFu2(I3G3),MFu2(I3G3))))
           enddo
           do I6G3 = 1,6
       Dself=Dself-4*e*(CsdmsdpZ(I6G3,I6G3)*myB0i(dbb00,P2,MSd2(I6G3)
     &  ,MSd2(I6G3))+CsemsepZ(I6G3,I6G3)*myB0i(dbb00,P2,MSe2(I6G3)
     &  ,MSe2(I6G3))-2*CsupsumZ(I6G3,I6G3)*myB0i(dbb00,P2,MSu2(I6G3)
     &  ,MSu2(I6G3)))
           enddo

      issDYZself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function issWself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
       self=0D0

       self=self+((-2*CWWZ**2-2*e**2+3*g2**2)*myA0(MW2)+3*CWWZ**2*
     &  myA0(MZ2)-5*e**2*P2*myB0i(bb0,P2,0D0,MW2)-2*CWWZ**2*MZ2*
     &  myB0i(bb0,P2,MW2,MZ2)-3*CWWZ**2*P2*myB0i(bb0,P2,MW2,MZ2)-8*e**
     &  2*myB0i(bb00,P2,0D0,MW2)-8*CWWZ**2*myB0i(bb00,P2,MW2,MZ2)-2*e**
     &  2*P2*myB0i(bb1,P2,0D0,MW2)+2*CWWZ**2*P2*myB0i(bb1,P2,MW2,MZ2))


           do I183 = 1,18
       self=self+(CSvSvVWpVWm(I183,I183)*myA0(MSv182(I183)))/2d0
           enddo
           do I2G3 = 1,2
       self=self+(CHmHpWW(I2G3,I2G3)*myA0(MassHpm2(I2G3))+CHmWpA(I2G3)*
     &  *2*myB0i(bb0,P2,0D0,MassHpm2(I2G3))+CHmWpZ(I2G3)**2*
     &  myB0i(bb0,P2,MZ2,MassHpm2(I2G3)))
           enddo
           do I3G3 = 1,3
           do I3G4 = 1,3
       self=self-6*CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)*(myA0(MFd2(I3G4))
     &  +(-P2+MFu2(I3G3))*myB0i(bb0,P2,MFd2(I3G4),MFu2(I3G3))-2*
     &  myB0i(bb00,P2,MFd2(I3G4),MFu2(I3G3))-P2*myB0i(bb1,P2,MFd2(I3G4)
     &  ,MFu2(I3G3)))
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       self=self+2*((CLF12F11W(I2G3,I5G3)*(DCONJG(CRF12F11W(I2G3,I5G3))
     &  *MCha(I2G3)*MNeu(I5G3)+DCONJG(CLF12F11W(I2G3,I5G3))*(P2-
     &  MNeu2(I5G3)))+CRF12F11W(I2G3,I5G3)*(DCONJG(CLF12F11W(I2G3,I5G3)
     &  )*MCha(I2G3)*MNeu(I5G3)+DCONJG(CRF12F11W(I2G3,I5G3))*(P2-
     &  MNeu2(I5G3))))*myB0i(bb0,P2,MCha2(I2G3),MNeu2(I5G3))-
     &  (CLF12F11W(I2G3,I5G3)*DCONJG(CLF12F11W(I2G3,I5G3))+
     &  CRF12F11W(I2G3,I5G3)*DCONJG(CRF12F11W(I2G3,I5G3)))*
     &  (myA0(MCha2(I2G3))-2*myB0i(bb00,P2,MCha2(I2G3),MNeu2(I5G3))-P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MNeu2(I5G3))))
           enddo
           enddo
           do I6G3 = 1,6
       self=self+(-(IC*ChhWmWp(I6G3,I6G3)*myA0(Masshh2(I6G3)))/2d0+3*
     &  CSdSdWW(I6G3,I6G3)*myA0(MSd2(I6G3))+CSeSeWW(I6G3,I6G3)*
     &  myA0(MSe2(I6G3))+3*CSuSuWW(I6G3,I6G3)*myA0(MSu2(I6G3))-
     &  ChWmWp(I6G3)**2*myB0i(bb0,P2,MW2,Masshh2(I6G3)))
           enddo
           do I183 = 1,18
           do I6G3 = 1,6
       self=self-4*CSemSvVWp(I6G3,I183)*CSvSepVWm(I183,I6G3)*
     &  myB0i(bb00,P2,MSe2(I6G3),MSv182(I183))
           enddo
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       self=self+4*ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3)*
     &  myB0i(bb00,P2,Masshh2(I6G3),MassHpm2(I2G3))
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self-12*CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))*
     &  myB0i(bb00,P2,MSd2(I6G4),MSu2(I6G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I9G3 = 1,9
       self=self+2*((CLFepFvVWm(I3G3,I9G3)*(CRFvFemVWp(I9G3,I3G3)*
     &  MassFe(I3G3)*mv9(I9G3)+CLFvFemVWp(I9G3,I3G3)*(P2-mv92(I9G3)))+
     &  CRFepFvVWm(I3G3,I9G3)*(CLFvFemVWp(I9G3,I3G3)*MassFe(I3G3)*
     &  mv9(I9G3)+CRFvFemVWp(I9G3,I3G3)*(P2-mv92(I9G3))))*
     &  myB0i(bb0,P2,MFe2(I3G3),mv92(I9G3))-(CLFepFvVWm(I3G3,I9G3)*
     &  CLFvFemVWp(I9G3,I3G3)+CRFepFvVWm(I3G3,I9G3)*
     &  CRFvFemVWp(I9G3,I3G3))*(myA0(MFe2(I3G3))-2*
     &  myB0i(bb00,P2,MFe2(I3G3),mv92(I9G3))-P2*myB0i(bb1,P2,MFe2(I3G3)
     &  ,mv92(I9G3))))
           enddo
           enddo
      issWself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function issDWself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself+(-5*e**2*myB0i(bb0,P2,0D0,MW2)-3*CWWZ**2*
     &  myB0i(bb0,P2,MW2,MZ2)-2*e**2*myB0i(bb1,P2,0D0,MW2)+2*CWWZ**2*
     &  myB0i(bb1,P2,MW2,MZ2)-5*e**2*P2*myB0i(dbb0,P2,0D0,MW2)-2*CWWZ**
     &  2*MZ2*myB0i(dbb0,P2,MW2,MZ2)-3*CWWZ**2*P2*
     &  myB0i(dbb0,P2,MW2,MZ2)-8*e**2*myB0i(dbb00,P2,0D0,MW2)-8*CWWZ**
     &  2*myB0i(dbb00,P2,MW2,MZ2)-2*e**2*P2*myB0i(dbb1,P2,0D0,MW2)+2*
     &  CWWZ**2*P2*myB0i(dbb1,P2,MW2,MZ2))
           do I2G3 = 1,2
       Dself=Dself+(CHmWpA(I2G3)**2*myB0i(dbb0,P2,0D0,MassHpm2(I2G3))+
     &  CHmWpZ(I2G3)**2*myB0i(dbb0,P2,MZ2,MassHpm2(I2G3)))
           enddo
           do I3G3 = 1,3
           do I3G4 = 1,3
       Dself=Dself+6*CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)*
     &  (myB0i(bb0,P2,MFd2(I3G4),MFu2(I3G3))+myB0i(bb1,P2,MFd2(I3G4)
     &  ,MFu2(I3G3))+P2*myB0i(dbb0,P2,MFd2(I3G4),MFu2(I3G3))-MFu2(I3G3)
     &  *myB0i(dbb0,P2,MFd2(I3G4),MFu2(I3G3))+2*
     &  myB0i(dbb00,P2,MFd2(I3G4),MFu2(I3G3))+P2*
     &  myB0i(dbb1,P2,MFd2(I3G4),MFu2(I3G3)))
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       Dself=Dself+2*((CLF12F11W(I2G3,I5G3)*DCONJG(CLF12F11W(I2G3,I5G3)
     &  )+CRF12F11W(I2G3,I5G3)*DCONJG(CRF12F11W(I2G3,I5G3)))*
     &  myB0i(bb0,P2,MCha2(I2G3),MNeu2(I5G3))+(CLF12F11W(I2G3,I5G3)*
     &  (DCONJG(CRF12F11W(I2G3,I5G3))*MCha(I2G3)*MNeu(I5G3)+
     &  DCONJG(CLF12F11W(I2G3,I5G3))*(P2-MNeu2(I5G3)))+
     &  CRF12F11W(I2G3,I5G3)*(DCONJG(CLF12F11W(I2G3,I5G3))*MCha(I2G3)*
     &  MNeu(I5G3)+DCONJG(CRF12F11W(I2G3,I5G3))*(P2-MNeu2(I5G3))))*
     &  myB0i(dbb0,P2,MCha2(I2G3),MNeu2(I5G3))+(CLF12F11W(I2G3,I5G3)*
     &  DCONJG(CLF12F11W(I2G3,I5G3))+CRF12F11W(I2G3,I5G3)*
     &  DCONJG(CRF12F11W(I2G3,I5G3)))*(myB0i(bb1,P2,MCha2(I2G3)
     &  ,MNeu2(I5G3))+2*myB0i(dbb00,P2,MCha2(I2G3),MNeu2(I5G3))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MNeu2(I5G3))))
           enddo
           enddo
           do I6G3 = 1,6
       Dself=Dself-(ChWmWp(I6G3)**2*myB0i(dbb0,P2,MW2,Masshh2(I6G3)))
           enddo
           do I183 = 1,18
           do I6G3 = 1,6
       Dself=Dself-4*CSemSvVWp(I6G3,I183)*CSvSepVWm(I183,I6G3)*
     &  myB0i(dbb00,P2,MSe2(I6G3),MSv182(I183))
           enddo
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       Dself=Dself+4*ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3)*
     &  myB0i(dbb00,P2,Masshh2(I6G3),MassHpm2(I2G3))
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself-12*CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))*
     &  myB0i(dbb00,P2,MSd2(I6G4),MSu2(I6G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I9G3 = 1,9
       Dself=Dself+2*((CLFepFvVWm(I3G3,I9G3)*CLFvFemVWp(I9G3,I3G3)+
     &  CRFepFvVWm(I3G3,I9G3)*CRFvFemVWp(I9G3,I3G3))*
     &  myB0i(bb0,P2,MFe2(I3G3),mv92(I9G3))+(CLFepFvVWm(I3G3,I9G3)*
     &  (CRFvFemVWp(I9G3,I3G3)*MassFe(I3G3)*mv9(I9G3)+
     &  CLFvFemVWp(I9G3,I3G3)*(P2-mv92(I9G3)))+CRFepFvVWm(I3G3,I9G3)*
     &  (CLFvFemVWp(I9G3,I3G3)*MassFe(I3G3)*mv9(I9G3)+
     &  CRFvFemVWp(I9G3,I3G3)*(P2-mv92(I9G3))))*
     &  myB0i(dbb0,P2,MFe2(I3G3),mv92(I9G3))+(CLFepFvVWm(I3G3,I9G3)*
     &  CLFvFemVWp(I9G3,I3G3)+CRFepFvVWm(I3G3,I9G3)*
     &  CRFvFemVWp(I9G3,I3G3))*(myB0i(bb1,P2,MFe2(I3G3),mv92(I9G3))+2*
     &  myB0i(dbb00,P2,MFe2(I3G3),mv92(I9G3))+P2*
     &  myB0i(dbb1,P2,MFe2(I3G3),mv92(I9G3))))
           enddo
           enddo
      issDWself=Dself*fac
      end



c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine issSTU_UVcheck()
        implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision ZH5(5,5),ZH(6,6)
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv

      double precision p2,self,dself,selfz,selfy,selfw,selfyz,
     &                 dselfz,dselfy,dselfw,dselfyz,selfQQ,self3Q,
     &                 self33,self11,dselfQQ,dself3Q,
     &                 dself33,dself11,S,T,U

      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"

      
      Dself=0D0
      self=0D0

       Dself=(-10*CWWZ**2)/3d0+Dself
           do I183 = 1,18
           do I184 = 1,18
       Dself=Dself-CSvSvVZ(I183,I184)**2/6d0
           enddo
           enddo
           do I2G3 = 1,2
       Dself=CHpHmZ**2/3d0+Dself
           enddo
           do I2G3 = 1,2
           do I2G4 = 1,2
       Dself=Dself+(2*(CF12ZL(I2G3,I2G4)*CF12ZL(I2G4,I2G3)+
     &  CF12ZR(I2G3,I2G4)*CF12ZR(I2G4,I2G3)))/3d0
           enddo
           enddo
           do I3G3 = 1,3
       Dself=(2*(3*CddZL**2+3*CddZR**2+ctautauZL**2+ctautauZR**2+3*
     &  CuuZL**2+3*CuuZR**2))/3d0+Dself
           enddo
           do I5G3 = 1,5
           do I5G4 = 1,5
       Dself=Dself+(CF11ZL(I5G3,I5G4)*CF11ZL(I5G4,I5G3)+
     &  CF11ZR(I5G3,I5G4)*CF11ZR(I5G4,I5G3))/3d0
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself+(ChhZ(I6G3,I6G4)**2+6*CsdmsdpZ(I6G3,I6G4)*
     &  CsdmsdpZ(I6G4,I6G3)+2*CsemsepZ(I6G3,I6G4)*CsemsepZ(I6G4,I6G3)+
     &  6*CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3))/6d0
           enddo
           enddo
           do I9G3 = 1,9
           do I9G4 = 1,9
       Dself=Dself+(CLFvFvVZ(I9G3,I9G4)*CLFvFvVZ(I9G4,I9G3)+
     &  CRFvFvVZ(I9G3,I9G4)*CRFvFvVZ(I9G4,I9G3))/3d0
           enddo
           enddo
       Dselfz=Dself*fac

      Dself=0D0
       Dself=Dself+(-10*e**2)/3d0
           do I2G3 = 1,2
       Dself=Dself+(5*e**2)/3d0
           enddo
           do I3G3 = 1,3
       Dself=Dself+(32*e**2)/9d0
           enddo
           do I6G3 = 1,6
       Dself=Dself+(8*e**2)/9d0
           enddo
       Dselfy=Dself*fac

      Dself=0D0
       Dself=Dself+(-10*CWWZ*e)/3d0
           do I2G3 = 1,2
       Dself=Dself+(e*(CHpHmZ+2*CF12ZL(I2G3,I2G3)+2*CF12ZR(I2G3,I2G3)))/3d0
           enddo
           do I3G3 = 1,3
       Dself=Dself+(2*(CddZL+CddZR+ctautauZL+ctautauZR-2*CuuZL-2*CuuZR)*e)/3d0
           enddo
           do I6G3 = 1,6
       Dself=Dself+(e*(CsdmsdpZ(I6G3,I6G3)+CsemsepZ(I6G3,I6G3)-2*
     &  CsupsumZ(I6G3,I6G3)))/3d0
           enddo
       Dselfyz=Dself*fac


      Dself=0D0

       Dself=Dself+(-10*(CWWZ**2+e**2))/3d0
           do I3G3 = 1,3
           do I3G4 = 1,3
       Dself=Dself+2*CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       Dself=Dself+(2*(CLF12F11W(I2G3,I5G3)*DCONJG(CLF12F11W(I2G3,I5G3)
     &  )+CRF12F11W(I2G3,I5G3)*DCONJG(CRF12F11W(I2G3,I5G3))))/3d0
           enddo
           enddo
           do I183 = 1,18
           do I6G3 = 1,6
       Dself=Dself+(CSemSvVWp(I6G3,I183)*CSvSepVWm(I183,I6G3))/3d0
           enddo
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       Dself=Dself-(ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3))/3d0
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself+CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I9G3 = 1,9
       Dself=Dself+(2*(CLFepFvVWm(I3G3,I9G3)*CLFvFemVWp(I9G3,I3G3)+
     &  CRFepFvVWm(I3G3,I9G3)*CRFvFemVWp(I9G3,I3G3)))/3d0
           enddo
           enddo
       Dselfw=Dself*fac


       self=0D0
       self=(-2*CWWZ**2*(3*MW2+5*P2))/3d0+self
           do I183 = 1,18
       self=self+(CSvSvVZVZ(I183,I183)*MSv182(I183))/2d0
           enddo
           do I183 = 1,18
           do I184 = 1,18
       self=self-(CSvSvVZ(I183,I184)**2*(P2-3*MSv182(I183)-3*MSv182(I184)))/6d0
           enddo
           enddo
           do I2G3 = 1,2
       self=self+((CHpHmZ**2*P2)/3d0+2*CHmWpZ(I2G3)**2+(-2*CHpHmZ**2+
     &  CHpHmZZ(I2G3,I2G3))*MassHpm2(I2G3))
           enddo
           do I2G3 = 1,2
           do I2G4 = 1,2
       self=self+(CF12ZL(I2G3,I2G4)*(6*CF12ZR(I2G4,I2G3)*MCha(I2G3)*
     &  MCha(I2G4)+CF12ZL(I2G4,I2G3)*(2*P2-3*MCha2(I2G3)-3*MCha2(I2G4))
     &  )+CF12ZR(I2G3,I2G4)*(6*CF12ZL(I2G4,I2G3)*MCha(I2G3)*MCha(I2G4)+
     &  CF12ZR(I2G4,I2G3)*(2*P2-3*MCha2(I2G3)-3*MCha2(I2G4))))/3d0
           enddo
           enddo
           do I3G3 = 1,3
       self=self+(2*(3*CddZL**2*P2+3*CddZR**2*P2+ctautauZL**2*P2+
     &  ctautauZR**2*P2+3*CuuZL**2*P2+3*CuuZR**2*P2-9*(CddZL-CddZR)**2*
     &  MFd2(I3G3)-3*(ctautauZL-ctautauZR)**2*MFe2(I3G3)-9*CuuZL**2*
     &  MFu2(I3G3)+18*CuuZL*CuuZR*MFu2(I3G3)-9*CuuZR**2*MFu2(I3G3)))
     &  /3d0
           enddo
           do I5G3 = 1,5
           do I5G4 = 1,5
       self=self+(CF11ZL(I5G3,I5G4)*(6*CF11ZR(I5G4,I5G3)*MNeu(I5G3)*
     &  MNeu(I5G4)+CF11ZL(I5G4,I5G3)*(2*P2-3*MNeu2(I5G3)-3*MNeu2(I5G4))
     &  )+CF11ZR(I5G3,I5G4)*(6*CF11ZL(I5G4,I5G3)*MNeu(I5G3)*MNeu(I5G4)+
     &  CF11ZR(I5G4,I5G3)*(2*P2-3*MNeu2(I5G3)-3*MNeu2(I5G4))))/6d0
           enddo
           enddo
           do I6G3 = 1,6
       self=self+(-ChZZ(I6G3)**2-(IC*ChhZZ(I6G3,I6G3)*Masshh2(I6G3))
     &  /2d0+3*CSdSdZZ(I6G3,I6G3)*MSd2(I6G3)+CSeSeZZ(I6G3,I6G3)*
     &  MSe2(I6G3)+3*CSuSuZZ(I6G3,I6G3)*MSu2(I6G3))
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self+(ChhZ(I6G3,I6G4)**2*(P2-3*Masshh2(I6G3)-3*
     &  Masshh2(I6G4))+2*(3*CsdmsdpZ(I6G3,I6G4)*CsdmsdpZ(I6G4,I6G3)*
     &  (P2-3*MSd2(I6G3)-3*MSd2(I6G4))+CsemsepZ(I6G3,I6G4)*
     &  CsemsepZ(I6G4,I6G3)*(P2-3*MSe2(I6G3)-3*MSe2(I6G4))+3*
     &  CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3)*(P2-3*MSu2(I6G3)-3*
     &  MSu2(I6G4))))/6d0
           enddo
           enddo
           do I9G3 = 1,9
           do I9G4 = 1,9
       self=self+(CLFvFvVZ(I9G3,I9G4)*(6*CRFvFvVZ(I9G4,I9G3)*mv9(I9G3)*
     &  mv9(I9G4)+CLFvFvVZ(I9G4,I9G3)*(2*P2-3*mv92(I9G3)-3*mv92(I9G4)))
     &  +CRFvFvVZ(I9G3,I9G4)*(6*CLFvFvVZ(I9G4,I9G3)*mv9(I9G3)*mv9(I9G4)
     &  +CRFvFvVZ(I9G4,I9G3)*(2*P2-3*mv92(I9G3)-3*mv92(I9G4))))/6d0
           enddo
           enddo
       selfz=self*fac 

       self=0D0
       self=(-2*e**2*(3*MW2+5*P2))/3d0+self
           do I2G3 = 1,2
       self=self+((5*e**2*P2)/3d0+2*CHmWpA(I2G3)**2)
           enddo
           do I3G3 = 1,3
       self=(32*e**2*P2)/9d0+self
           enddo
           do I6G3 = 1,6
       self=(8*e**2*P2)/9d0+self
           enddo
       selfy=self*fac 



       self=0D0
       self=((-2*CWWZ*e*(12*MW2+5*P2))/3d0+3*g2**2*MW2*S2W)+self
           do I2G3 = 1,2
       self=self+(CHpHmZ*CW*e*P2*SW+2*CW*e*P2*SW*CF12ZL(I2G3,I2G3)+2*
     &  CW*e*P2*SW*CF12ZR(I2G3,I2G3)-6*CW*SW*CHmWpA(I2G3)*CHmWpZ(I2G3)-
     &  3*e**2*MassHpm2(I2G3)+6*CW2*e**2*MassHpm2(I2G3)-6*CHpHmZ*CW*e*
     &  SW*MassHpm2(I2G3))/(3d0*CW*SW)
           enddo
           do I3G3 = 1,3
       self=(2*(CddZL+CddZR+ctautauZL+ctautauZR-2*CuuZL-2*CuuZR)*e*P2)/3d0+self
           enddo
           do I6G3 = 1,6
       self=self+(e*(CsdmsdpZ(I6G3,I6G3)*(P2-6*MSd2(I6G3))+
     &  CsemsepZ(I6G3,I6G3)*(P2-6*MSe2(I6G3))-2*CsupsumZ(I6G3,I6G3)*
     &  (P2-6*MSu2(I6G3))+(e*(MSd2(I6G3)*(2*CW2+DCONJG(ZD(I6G3,1))*
     &  ZD(I6G3,1)-2*DCONJG(ZD(I6G3,2))*ZD(I6G3,2)+
     &  DCONJG(ZD(I6G3,3))*ZD(I6G3,3)-2*DCONJG(ZD(I6G3,4))*
     &  ZD(I6G3,4)+DCONJG(ZD(I6G3,5))*ZD(I6G3,5)-2*
     &  DCONJG(ZD(I6G3,6))*ZD(I6G3,6))+MSe2(I6G3)*(6*CW2-3*
     &  DCONJG(ZE(I6G3,1))*ZE(I6G3,1)-6*DCONJG(ZE(I6G3,2))*
     &  ZE(I6G3,2)-3*DCONJG(ZE(I6G3,3))*ZE(I6G3,3)-6*
     &  DCONJG(ZE(I6G3,4))*ZE(I6G3,4)-3*DCONJG(ZE(I6G3,5))*
     &  ZE(I6G3,5)-6*DCONJG(ZE(I6G3,6))*ZE(I6G3,6))+2*MSu2(I6G3)*(4*
     &  CW2-DCONJG(ZU(I6G3,1))*ZU(I6G3,1)-4*DCONJG(ZU(I6G3,2))*
     &  ZU(I6G3,2)-DCONJG(ZU(I6G3,3))*ZU(I6G3,3)-4*
     &  DCONJG(ZU(I6G3,4))*ZU(I6G3,4)-DCONJG(ZU(I6G3,5))*ZU(I6G3,5)
     &  -4*DCONJG(ZU(I6G3,6))*ZU(I6G3,6))))/(CW*SW)))/3d0
           enddo
       selfyz=self*fac

       self=0D0

       self=(3*g2**2*MW2-(2*e**2*(6*MW2+5*P2))/3d0-(CWWZ**2*(12*MW2+3*
     &  MZ2+10*P2))/3d0)+self
           do I183 = 1,18
       self=self+(CSvSvVWpVWm(I183,I183)*MSv182(I183))/2d0
           enddo
           do I2G3 = 1,2
       self=self+(CHmWpA(I2G3)**2+CHmWpZ(I2G3)**2+CHmHpWW(I2G3,I2G3)*
     &  MassHpm2(I2G3))
           enddo
           do I3G3 = 1,3
           do I3G4 = 1,3
       self=self+CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)*(2*P2-3*MFd2(I3G4)-
     &  3*MFu2(I3G3))
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       self=self+(CLF12F11W(I2G3,I5G3)*(6*DCONJG(CRF12F11W(I2G3,I5G3))*
     &  MCha(I2G3)*MNeu(I5G3)+DCONJG(CLF12F11W(I2G3,I5G3))*(2*P2-3*
     &  MCha2(I2G3)-3*MNeu2(I5G3)))+CRF12F11W(I2G3,I5G3)*(6*
     &  DCONJG(CLF12F11W(I2G3,I5G3))*MCha(I2G3)*MNeu(I5G3)+
     &  DCONJG(CRF12F11W(I2G3,I5G3))*(2*P2-3*MCha2(I2G3)-3*MNeu2(I5G3))
     &  ))/3d0
           enddo
           enddo
           do I6G3 = 1,6
       self=self+(-ChWmWp(I6G3)**2-(IC*ChhWmWp(I6G3,I6G3)*Masshh2(I6G3)
     &  )/2d0+3*CSdSdWW(I6G3,I6G3)*MSd2(I6G3)+CSeSeWW(I6G3,I6G3)*
     &  MSe2(I6G3)+3*CSuSuWW(I6G3,I6G3)*MSu2(I6G3))
           enddo
           do I183 = 1,18
           do I6G3 = 1,6
       self=self+(CSemSvVWp(I6G3,I183)*CSvSepVWm(I183,I6G3)*(P2-3*
     &  MSe2(I6G3)-3*MSv182(I183)))/3d0
           enddo
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       self=self-(ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3)*(P2-3*
     &  Masshh2(I6G3)-3*MassHpm2(I2G3)))/3d0
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self+CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))*(P2-3*
     &  MSd2(I6G4)-3*MSu2(I6G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I9G3 = 1,9
       self=self+(CLFepFvVWm(I3G3,I9G3)*(6*CRFvFemVWp(I9G3,I3G3)*
     &  MassFe(I3G3)*mv9(I9G3)+CLFvFemVWp(I9G3,I3G3)*(2*P2-3*MFe2(I3G3)
     &  -3*mv92(I9G3)))+CRFepFvVWm(I3G3,I9G3)*(6*CLFvFemVWp(I9G3,I3G3)*
     &  MassFe(I3G3)*mv9(I9G3)+CRFvFemVWp(I9G3,I3G3)*(2*P2-3*MFe2(I3G3)
     &  -3*mv92(I9G3))))/3d0
           enddo
           enddo
       selfw=self*fac

       selfQQ= selfy/e**2
       self3Q=(cw*selfyz*sw)/e**2 + (selfy*sw**2)/e**2
       self11=(selfw*sw**2)/e**2
       self33=(cw**2*selfz*sw**2)/e**2 + 
     -    (2*cw*selfyz*sw**3)/e**2 + (selfy*sw**4)/e**2

       dselfQQ= dselfy/e**2
       dself3Q=(cw*dselfyz*sw)/e**2 + (dselfy*sw**2)/e**2
       dself11=(dselfw*sw**2)/e**2
       dself33=(cw**2*dselfz*sw**2)/e**2 + 
     -    (2*cw*dselfyz*sw**3)/e**2 + (dselfy*sw**4)/e**2


       S= 16*pi*(dself33 -dself3Q)
       T= 4*pi/sw**2/cw**2/MZ2*(self11 -self33)
       U = 16*pi*(dself11 -dself33)

c       print*, "S,T,U= ",S,T,U
      end

