c------------------------------------------------------------------------------c
c   For the computation of the O(alpha_t^2) corrections                        c
c   written by Thi Nhung Dao,  last update: 6/2021                             c
c------------------------------------------------------------------------------c
c uncomment it to printout details for checking
c#define printMT4
c
c This part calculate the atat correction in the MSSM limit, which means
c that we set lambda and kappa to zero value but keep MUE as input parameter.
c In the following we describe the important subroutines
* subroutine ReHSelf_atat 
*  give finite part of the neutral Higgs selfenergies at \alpha_t^2 order
*  in the mass eigensates (h1,h2,h3,h4,h5) to connect with CalcMasses.F
* subroutine selfHH_atat
*  give the finite part of the neutral Higgs selfenergies at \alpha_t^2 order
*  in the interaction eigenstates (a,as,hd,hu,hs)
* subroutine couplingsEW
*  calculate neccessary couplings used in the calculation
* subroutine UnselfHH_atat
*  calculate unrenormalied Higgs selfenergies in the Interation basis
* subroutine CHSelfMT4
*  calculate unrenormalized charged Higgs selfenergy
* subroutine TadMT4
*  calculate unrenormalied tadpole in mass eigenstates
* subroutine CT1loopMT4
*  calculate one-loop counterterms 
* subroutine CT2loopMT4
*  calculate one-loop counterterms induced diagrams for neutral Higgs, charged Higgs and tadpole
* subroutine counterM_atat
*  calculate two-loop counterterm matrix for neutral Higgs boson in interaction basis
*  (hd,hu,hs,ad,au,as)

c#define  checkPS

      subroutine MSQ_MSU_AT_OStoDR(dmsq4fin,dmsur4fin,dAt4fin)
      implicit none
      integer i,j,k,l
      double precision MHp2,MW2,MZ2,dmsq4fin,dmsur4fin
      double precision Qs
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      logical CPcon,MHpmInput,onelswitch,mQrun

      double complex IC,ZUN(2,2),AT,la,Ala,MUE,expETA,expVS,dAt4fin
      common /scale/ Qs
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch

      IC=dcmplx(0,1D0)
      ZUN(1:2,1:2) = nZU(5:6,5:6)
      MW2=Mw**2; MZ2=Mz**2;MHp2=MHpm**2
      Ala= absAlambda*(cos(phiAl) + IC*Sin(phiAl))
      AT= absAu(3)*(cos(phiAu(3)) + IC*Sin(phiAu(3)))
      la= abslambda*(Cos(phil) + IC*Sin(phil))
      expvs=(Cos(phis) + IC*Sin(phis))
      expETA=(Cos(phiu) + IC*Sin(phiu))
      MUE=vs*la*expvS/dsqrt(2d0)

      call couplingsEW(Qs**2,MT,MHp2,v,vS,Tanb,
     -   MW2,MZ2,nMSu(5),nMSu(6),ZUN,AT,MUE,expETA,expvS)
      call CT1loopMT4(.False.,dmsq4fin,dmsur4fin,dAt4fin)

      end subroutine MSQ_MSU_AT_OStoDR
c
c----------------------------------------------------------------------c
c

      subroutine ReHSelf_atat(renfin)
      implicit none
      integer i,j,k,l
      double precision renfin(5,5),reself(5,5),MHp2
      double precision MW2,MZ2, A0k
      double precision Qs
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      logical CPcon,MHpmInput,onelswitch,mQrun

      double complex IC,ZUN(2,2),AT,la,Ala,MUE,expETA,expVS
      common /scale/ Qs
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      IC=dcmplx(0,1D0)
      ZUN(1:2,1:2) = nZU(5:6,5:6)
      MW2=Mw**2; MZ2=Mz**2;MHp2=MHpm**2
      Ala= absAlambda*(cos(phiAl) + IC*Sin(phiAl))
      AT= absAu(3)*(cos(phiAu(3)) + IC*Sin(phiAu(3)))
      la= abslambda*(Cos(phil) + IC*Sin(phil))
      expvs=(Cos(phis) + IC*Sin(phis))
      expETA=(Cos(phiu) + IC*Sin(phiu))
      MUE=vs*la*expvS/dsqrt(2d0)

      call selfHH_atat(Qs**2,mt,MHp2,v,vS,Tanb,MW2,MZ2,
     -   nMSu(5),nMSu(6),ZUN,AT,MUE,expETA,expvS,
     -   reself,MQrun,MHpmInput)

      do i=1,5
      do j=1,5
      renfin(i,j) =0D0
       do k=1,5
       do l=1,5
      renfin(i,j) = renfin(i,j)+ZH5(i,k)*ZH5(j,l)*reself(k,l)
       enddo
       enddo
      enddo
      enddo
#ifdef printMT4
      write(*,*) "===================================================="
      write(*,*) "ZH5(1,1:5)", ZH5(1,1:5)
      write(*,*) "ZH5(2,1:5)", ZH5(2,1:5)
      write(*,*) "ZH5(3,1:5)", ZH5(3,1:5)
      write(*,*) "ZH5(4,1:5)", ZH5(4,1:5)
      write(*,*) "ZH5(5,1:5)", ZH5(5,1:5)
      write(*,*) " atat contributions in basis (a1,a2,h1,h2,h3)"
      write(*,*) "renfin(1,1:5)=",renfin(1,1:5)   
      write(*,*) "renfin(2,1:5)=",renfin(2,1:5)
      write(*,*) "renfin(3,1:5)=",renfin(3,1:5)     
      write(*,*) "renfin(4,1:5)=",renfin(4,1:5) 
      write(*,*) "renfin(5,1:5)=",renfin(5,1:5) 
#endif

      end subroutine ReHSelf_atat
c
c----------------------------------------------------------------------c
c
      subroutine selfHH_atat(Q2N,MTN,MHpm2N,vvN,vSN,TBN,
     -   MW2N,MZ2N,MST1,MST2,ZUN,ATN,MUEN,expETAN,expvSN,
     -   reself,MQrun,MHpmInput)
      implicit none
      integer i,j,l,k,gen2,Gen3,Gen4,Gen5,Gen6,h1
      double precision reself(5,5),self(6,6),dMH2MT4(6,6),
     .     aTad(6),selfCT(6,6),TadCT(6),CHselfCT,dtad2MT4(6),
     .     selfIN(6,6),selfCTIN(6,6),selfINS(6,6),CHSELF

      double precision Q2N,MTN,MSt1,MSt2,MHpm2N,vvN,vSN,TBN,MW2N,MZ2N,
     .     S2B,C2B,dmsq4fin,dmsur4fin

      double complex ATN,ZUN(2,2),MUEN,expETAN,expvSN,dAt4fin

      logical MHpmInput,mQrun
#include "TLMT4.h"

      call couplingsEW(Q2N,MTN,MHpm2N,vvN,vSN,TBN,
     -   MW2N,MZ2N,MST1,MST2,ZUN,ATN,MUEN,expETAN,expvSN)

c      print*, "mt",MTN,MST1,MST2,sqrt(MHpm2N),ZUN

      S2B = 2*TB*CB2;          C2B= (CB - SB)*(CB + SB)
c neutral Higgs self
      call UnselfHH_atat(selfIN)
c charged Higgs self
      call CHSelfMT4(CHself)
c tadpoles
      call TadMT4(aTad)
c one-loop counterterms
      call CT1loopMT4(MQrun,dmsq4fin,dmsur4fin,dAt4fin)
c two-loop counterterms
      call CT2loopMT4(selfCT,CHselfCT,TadCT)
      CHself = CHself + CHselfCT
      do i=1,6
       dtad2MT4(i)=0D0
      do gen3=1,6
       dtad2MT4(i)=dtad2MT4(i)+ZH(gen3,i)*(aTad(gen3)+TadCT(gen3))
      enddo
      enddo

#ifdef printMT4
      Write(*,*), "___________________________________________________"
      Write(*,*), " Tadpole in mass basis including genuine 2 loop and 1 loop with CT insertion"
      Write(*,*), "aTad(1-6)",aTad(1:6)+TadCT(1:6)

      Write(*,*), "___________________________________________________"
      Write(*,*), " Tadpole in interaction basis including genuine 2 loop and 1 loop with CT insertion"
      Write(*,*), "dtad2MT4(1-6)",dtad2MT4(1:6)
      Write(*,*), "___________________________________________________"    
#endif

      do i=1,6
       do j=1,6 
       selfCTIN(i,j)=0D0
      do gen3=1,6
        do gen2=1,6
       selfCTIN(i,j)=selfCTIN(i,j)+ZH(gen2,i)*ZH(gen3,j)*selfCT(gen2,gen3)
        enddo
      enddo
        enddo
      enddo
      selfIN(1:6,1:6)=selfIN(1:6,1:6) + selfCTIN(1:6,1:6)
#ifdef printMT4
 
      Write(*,*) "___________________________________________________"
      Write(*,*) "Neutral Higgs selfenergies in interaction state"
      Write(*,*) "Sum of 2loop diagram + 1 loop CT"
      Write(*,*) " finite part in interaction basis"

      Write(*,*) "selfIN(1,1->6)+selfCTIN(1,1->6)",selfIN(1,1:6)
      Write(*,*) "selfIN(2,2->6)+selfCTIN(2,2->6)",selfIN(2,2:6)
      Write(*,*) "selfIN(3,3->6)+selfCTIN(3,3->6)",selfIN(3,3:6)
      Write(*,*) "selfIN(4,4->6)+selfCTIN(4,4->6)",selfIN(4,4:6)
      Write(*,*) "selfIN(5,5->6)+selfCTIN(5,5->6)",selfIN(5,5:6)
      Write(*,*) "selfIN(6,6->6)+selfCTIN(6,6->6)",selfIN(6,6)
#endif 
#ifdef checkPS
      print*, "dMA2L", self(5,5)+selfCT(5,5)
      print*, "SB2*selfINS(4,4)+S2B*selfINS(4,5)+CB2*selfINS(5,5)",SB2*selfIN(4,4)+S2B*selfIN(4,5)+CB2*selfIN(5,5)
      print*, "CB2*selfINS(4,4) - S2B*selfINS(4,5) + SB2*selfINS(5,5)",CB2*selfIN(4,4) - S2B*selfIN(4,5) + SB2*selfIN(5,5)
      print*, "CHself", CHself
      CHself= SB2*selfIN(4,4)+S2B*selfIN(4,5)+CB2*selfIN(5,5)
#endif
      
      call counterM_atat(CHself,dtad2MT4,dMH2MT4)

#ifdef printMT4
      Write(*,*), "______Neutral Higgs CT matrix___________"
      Write(*,*), " finite part in interaction basis"

      Write(*,*), "dMH2MT4(1,1->6)",dMH2MT4(1,1:6)
      Write(*,*), "dMH2MT4(2,2->6)",dMH2MT4(2,2:6)
      Write(*,*), "dMH2MT4(3,3->6)",dMH2MT4(3,3:6)
      Write(*,*), "dMH2MT4(4,4->6)",dMH2MT4(4,4:6)
      Write(*,*), "dMH2MT4(5,5->6)",dMH2MT4(5,5:6)
      Write(*,*), "dMH2MT4(6,6->6)",dMH2MT4(6,6:6)
      Write(*,*), "___________________________________________________"
#endif   


      do i=1,6
       do j=i,6
      SelfINS(i,j) = SelfIN(i,j) - dMH2MT4(i,j)
       enddo
      enddo
      do i=1,6
       do j=i,6
      SelfINS(j,i) = SelfINS(i,j) 
       enddo
      enddo
c rotate to (a,as,hd,hu,hs) basis
#ifdef printMT4
      Write(*,*) "Goldstone mixing"
      Write(*,*)CB2*selfINS(4,4) - S2B*selfINS(4,5) + SB2*selfINS(5,5)
      Write(*,*)CB2*selfINS(4,5) - SB2*selfINS(4,5) + CB*SB*(selfINS(4,4) - selfINS(5,5))
      Write(*,*)0
      Write(*,*)CB*selfINS(1,4) - SB*selfINS(1,5)
      Write(*,*)CB*selfINS(2,4) - SB*selfINS(2,5)
#endif
      ReSelf(1,1)=SB2*selfINS(4,4)+S2B*selfINS(4,5)+CB2*selfINS(5,5)
      ReSelf(1,3)=SB*selfINS(1,4)+CB*selfINS(1,5)
      ReSelf(1,4)=SB*selfINS(2,4)+CB*selfINS(2,5)
      ReSelf(3,1)=SB*selfINS(1,4)+CB*selfINS(1,5)

      ReSelf(3,3)=selfINS(1,1)
      ReSelf(3,4)=selfINS(1,2)
      ReSelf(4,1)=SB*selfINS(2,4) + CB*selfINS(2,5)
      ReSelf(4,3)=selfINS(1,2)
      ReSelf(4,4)=selfINS(2,2)

#ifdef printMT4
      Write(*,*) "------------------------------------------------"
      Write(*,*) "renormalized self in interaction state hd,hu,hs,au,ad,as"
      Write(*,*) "selfINS(1,1->6)",selfINS(1,1:6)
      Write(*,*) "selfINS(2,2->6)",selfINS(2,2:6)
      Write(*,*) "selfINS(3,3->6)",selfINS(3,3:6)
      Write(*,*) "selfINS(4,4->6)",selfINS(4,4:6)
      Write(*,*) "selfINS(5,5->6)",selfINS(5,5:6)
      Write(*,*) "selfINS(6,6->6)",selfINS(6,6)
      Write(*,*) "------------------------------------------------"
      Write(*,*) "renormalized self in interaction state a,as,hu,hd,hs"
      Write(*,*) "reself(1,1->6)",reself(1,1:5)
      Write(*,*) "reself(2,2->6)",reself(2,2:5)
      Write(*,*) "reself(3,3->6)",reself(3,3:5)
      Write(*,*) "reself(4,4->6)",reself(4,4:5)
      Write(*,*) "reself(5,5->6)",reself(5,5:5)

      do i=1,6
       do j=1,6 
       self(i,j)=0D0
      do gen3=1,6
        do gen2=1,6
       self(i,j)=self(i,j)+ZH(i,gen2)*ZH(j,gen3)*selfINS(gen2,gen3)
        enddo
      enddo
        enddo
      enddo
      Write(*,*) "renormalized self in mass eigenstate as,hs,h,H,A,G"
      Write(*,*) "self(1,1->6)",self(1,1:6)
      Write(*,*) "self(2,2->6)",self(2,2:6)
      Write(*,*) "self(3,3->6)",self(3,3:6)
      Write(*,*) "self(4,4->6)",self(4,4:6)
      Write(*,*) "self(5,5->6)",self(5,5:6)
      Write(*,*) "self(6,6->6)",self(6,6:6)
      stop
#endif 
c)
      end subroutine selfHH_atat


c**********************************************************************
      subroutine CT2loopMT4(selfCT,CHselfCT,TadCT)
      implicit none
      integer i,j,k,l,i1,i2,i3,I6G3,I6G4,I6G5,I6G2
      double precision pi2,fac
      double precision CHselfCT,selfCT(6,6),TadCT(6)
#include "TLMT4.h"


         pi2=pi**2
         fac= 1/16D0/pi2
         ATC=DCONJG(ATT)
         MUEC=DCONJG(MUE)
        do i=1,6
         do j=1,6
         dZHiggs4(i,j) = ZH(i,2)*ZH(j,2)*dZHu4+ ZH(i,5)*ZH(j,5)*dZHu4
         enddo
        enddo

         dZCHiggs4(1,1) =SB**2*dZHu4
         dZCHiggs4(1,2) =SB*CB*dZHu4
         dZCHiggs4(2,1) =dZCHiggs4(1,2)
         dZCHiggs4(2,2) =CB**2*dZHu4
       dS3TB(1)=(CB**4*dTanB4*M1*sqrt2)/(Sqrt(expETA)*SB**2*vv)-(CB*
     &  sqrt2*dMT4(1))/(Sqrt(expETA)*SB*vv)+(CB*M1*sqrt2*dv4(1))
     &  /(Sqrt(expETA)*SB*vv2) + CTBS3(2)*dZCHiggs4(2,2)/2D0+
     & CTBS3(1)*dZCHiggs4(2,1)/2D0
       dS3TB(2)=-((CB*sqrt2*dMT4(2))/(Sqrt(expETA)*SB*vv))+(CB*M1*
     &  sqrt2*dv4(2))/(Sqrt(expETA)*SB*vv2)
       dS3TB(3)=-((CB*sqrt2*dMT4(3))/(Sqrt(expETA)*SB*vv))+(CB*M1*
     &  sqrt2*dv4(3))/(Sqrt(expETA)*SB*vv2)

       dCS3SSS14(1)=(-4*CB**5*dTanB4*M12)/(SB**3*vv2)+(4*CB**2*M1*
     &  dMT4(1))/(SB**2*vv2)-(4*CB**2*M12*dv4(1))/(SB**2*vv**3)+
     &  CS3S3S14S14(2,2)*dZCHiggs4(2,2)+CS3S3S14S14(2,1)*dZCHiggs4(2,1)

       dCS3SSS14(2)=(4*CB**2*M1*dMT4(2))/(SB**2*vv2)-(4*CB**2*M12*
     &  dv4(2))/(SB**2*vv**3)
       dCS3SSS14(3)=(4*CB**2*M1*dMT4(3))/(SB**2*vv2)-(4*CB**2*M12*
     &  dv4(3))/(SB**2*vv**3)

         do j=1,2
       dCS3S13S14(j,1)=-((dMUE4*M1*sqrt2*ZUC(j,2))/vv)-(CB*M1*sqrt2*
     &  DCONJG(dAT4(1))*ZUC(j,2))/(expETA*SB*vv)+(CB**3*dTanB4*M1*(CB*
     &  expETA*M1*sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*
     &  sqrt2*ZUC(j,2)))/(expETA*SB**2*vv)+(M1*dv4(1)*(CB*expETA*M1*
     &  sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*sqrt2*
     &  ZUC(j,2)))/(expETA*SB*vv2)+dMT4(1)*(-((CB*M1*sqrt2*ZUC(j,1))
     &  /(SB*vv))-(CB*expETA*M1*sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+
     &  expETA*MUE*SB*sqrt2*ZUC(j,2))/(expETA*SB*vv))+
     &  CS3S13S14(j)*dZCHiggs4(2,2)/2D0+ CS3S13S(1,j)*dZChiggs4(1,2)/2D0

            do i=2,3
       dCS3S13S14(j,i)=-((CB*M1*sqrt2*DCONJG(dAT4(i))*ZUC(j,2))
     &  /(expETA*SB*vv))+(M1*dv4(i)*(CB*expETA*M1*sqrt2*ZUC(j,1)+ATC*
     &  CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*sqrt2*ZUC(j,2)))/(expETA*SB*
     &  vv2)+dMT4(i)*(-((CB*M1*sqrt2*ZUC(j,1))/(SB*vv))-(CB*expETA*M1*
     &  sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*sqrt2*
     &  ZUC(j,2))/(expETA*SB*vv))
            enddo

        dCS3S14S13(j,1:3)=DCONJG(dCS3S13S14(j,1:3))

           do k=1,2
       dCS3S13S(j,k,1)=(-4*CB**5*dTanB4*M12*ZU(k,2)*ZUC(j,2))/(SB**3*
     &  vv2)+(4*CB**2*M1*dMT4(1)*ZU(k,2)*ZUC(j,2))/(SB**2*vv2)-(4*CB**
     &  2*M12*dv4(1)*ZU(k,2)*ZUC(j,2))/(SB**2*vv**3)
     &  +CS3S3S13S13(2,2,j,k)*dZChiggs4(2,2)+CS3S3S13S13(2,1,j,k)*dZChiggs4(2,1)
  
       dCS3S13S(j,k,2)=(4*CB**2*M1*dMT4(2)*ZU(k,2)*ZUC(j,2))/(SB**2*
     &  vv2)-(4*CB**2*M12*dv4(2)*ZU(k,2)*ZUC(j,2))/(SB**2*vv**3)
       dCS3S13S(j,k,3)=(4*CB**2*M1*dMT4(3)*ZU(k,2)*ZUC(j,2))/(SB**2*
     &  vv2)-(4*CB**2*M12*dv4(3)*ZU(k,2)*ZUC(j,2))/(SB**2*vv**3)
       dCS3S13SC(j,k,1:3)=      DCONJG( dCS3S13S(j,k,1:3))
           
            do l=1,6
              i=1
       dCSSS(l,k,j,i)=DCONJG(dMUE4)*(-((M1*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(SB*vv))+
     &  (IC*M1*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv))+dAT4(i)*((expETA*M1*
     &  ZH(l,2)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(expETA*IC*M1*ZH(l,5)*ZU(j,2)
     &  *ZUC(k,1))/(SB*vv))+dlambdaC4*(-((CB*M1*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2))+(CB*IC*M1*ZH(l,6)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2))+dMUE4*(-((M1*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  vv))-(IC*M1*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*vv))+DCONJG(dAT4(i))*
     &  ((M1*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)-(IC*M1*ZH(l,5)*
     &  ZU(j,1)*ZUC(k,2))/(expETA*SB*vv))+DCONJG(dlambdaC4)*(-((CB*
     &  expvS*M1*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2))-(CB*expvS*IC*M1*
     &  ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2))-CB**2*dTanB4*SB*(-((M1*
     &  DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2))+(IC*M1*
     &  DCONJG(la)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2)-(expvS*
     &  la*M1*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)-(expvS*IC*la*M1*
     &  ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2))+CB**3*dTanB4*((-2*M12*
     &  ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv)+(M1*MUEC*ZH(l,1)*ZU(j,2)*
     &  ZUC(k,1))/(SB**2*vv)-(ATT*expETA*M1*ZH(l,2)*ZU(j,2)*ZUC(k,1))
     &  /(SB**2*vv)+(CB*M1*DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB**2*sqrt2)-(IC*M1*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB**2*vv)-
     &  (ATT*expETA*IC*M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB**2*vv)-(CB*IC*
     &  M1*DCONJG(la)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2)+
     &  (M1*MUE*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB**2*vv)-(ATC*M1*ZH(l,2)*
     &  ZU(j,1)*ZUC(k,2))/(expETA*SB**2*vv)+(CB*expvS*la*M1*ZH(l,3)*
     &  ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2)+(IC*M1*MUE*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB**2*vv)+(ATC*IC*M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))
     &  /(expETA*SB**2*vv)+(CB*expvS*IC*la*M1*ZH(l,6)*ZU(j,1)*ZUC(k,2))
     &  /(SB**2*sqrt2)-(2*M12*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv))+
     &  dMT4(i)*((4*M1*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB*vv)-(MUEC*ZH(l,1)*
     &  ZU(j,2)*ZUC(k,1))/(SB*vv)+(ATT*expETA*ZH(l,2)*ZU(j,2)*ZUC(k,1))
     &  /(SB*vv)-(CB*DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*
     &  sqrt2)+(IC*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(ATT*expETA*
     &  IC*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(CB*IC*DCONJG(la)*ZH(l,6)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2)-(MUE*ZH(l,1)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv)+(ATC*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)
     &  -(CB*expvS*la*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)-(IC*MUE*
     &  ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*vv)-(ATC*IC*ZH(l,5)*ZU(j,1)*
     &  ZUC(k,2))/(expETA*SB*vv)-(CB*expvS*IC*la*ZH(l,6)*ZU(j,1)*
     &  ZUC(k,2))/(SB*sqrt2)+(4*M1*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB*vv))+
     &  dv4(i)*((-2*M12*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB*vv2)+(M1*MUEC*
     &  ZH(l,1)*ZU(j,2)*ZUC(k,1))/(SB*vv2)-(ATT*expETA*M1*ZH(l,2)*
     &  ZU(j,2)*ZUC(k,1))/(SB*vv2)-(IC*M1*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1)
     &  )/(SB*vv2)-(ATT*expETA*IC*M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv2)
     &  +(M1*MUE*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*vv2)-(ATC*M1*ZH(l,2)*
     &  ZU(j,1)*ZUC(k,2))/(expETA*SB*vv2)+(IC*M1*MUE*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv2)+(ATC*IC*M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))
     &  /(expETA*SB*vv2)-(2*M12*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB*vv2))
            do i=1,6
               dCSSS(l,k,j,1)=dCSSS(l,k,j,1)+dZHiggs4(l,i)*CSSS(i,k,j)/2D0
            enddo            
            do i=2,3
       dCSSS(l,k,j,i)=dAT4(i)*((expETA*M1*ZH(l,2)*ZU(j,2)*ZUC(k,1))
     &  /(SB*vv)+(expETA*IC*M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv))+
     &  DCONJG(dAT4(i))*((M1*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)-
     &  (IC*M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv))+dMT4(i)*((4*
     &  M1*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB*vv)-(MUEC*ZH(l,1)*ZU(j,2)*
     &  ZUC(k,1))/(SB*vv)+(ATT*expETA*ZH(l,2)*ZU(j,2)*ZUC(k,1))/(SB*vv)
     &  -(CB*DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2)+(IC*
     &  MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(ATT*expETA*IC*ZH(l,5)*
     &  ZU(j,2)*ZUC(k,1))/(SB*vv)+(CB*IC*DCONJG(la)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2)-(MUE*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  vv)+(ATC*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)-(CB*expvS*la*
     &  ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)-(IC*MUE*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv)-(ATC*IC*ZH(l,5)*ZU(j,1)*ZUC(k,2))/(expETA*SB*
     &  vv)-(CB*expvS*IC*la*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)+(4*M1*
     &  ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB*vv))+dv4(i)*((-2*M12*ZH(l,2)*
     &  ZU(j,1)*ZUC(k,1))/(SB*vv2)+(M1*MUEC*ZH(l,1)*ZU(j,2)*ZUC(k,1))
     &  /(SB*vv2)-(ATT*expETA*M1*ZH(l,2)*ZU(j,2)*ZUC(k,1))/(SB*vv2)-
     &  (IC*M1*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv2)-(ATT*expETA*IC*
     &  M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv2)+(M1*MUE*ZH(l,1)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv2)-(ATC*M1*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*
     &  SB*vv2)+(IC*M1*MUE*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*vv2)+(ATC*IC*
     &  M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv2)-(2*M12*ZH(l,2)*
     &  ZU(j,2)*ZUC(k,2))/(SB*vv2))

            enddo
            do i1=1,6

       dCSSSS(l,i1,k,j,1)=dlambdaC4*IC*((IC*M1*ZH(i1,3)*ZH(l,1)*ZU(j,2)
     &  *ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,6)*ZH(l,1)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(IC*M1*ZH(i1,1)*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,4)*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,3)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*M1*ZH(i1,6)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,1)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*M1*ZH(i1,4)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv))+IC*DCONJG(dlambdaC4)*((expvS*IC*
     &  M1*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)+(expvS*IC*M1*
     &  ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*M1*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*M1*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv))+IC*dv4(1)*
     &  ((4*IC*M12*ZH(i1,2)*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv**3)+(4*
     &  IC*M12*ZH(i1,5)*ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**2*vv**3)-(IC*M1*
     &  DCONJG(la)*ZH(i1,3)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(M1*DCONJG(la)*ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv2)-(IC*M1*DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,4)*ZH(l,3)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,3)*
     &  ZH(l,4)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)+(IC*M1*
     &  DCONJG(la)*ZH(i1,6)*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(M1*DCONJG(la)*ZH(i1,1)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv2)+(IC*M1*DCONJG(la)*ZH(i1,4)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(expvS*IC*la*M1*ZH(i1,3)*ZH(l,1)
     &  *ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,6)*ZH(l,1)
     &  *ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)-(expvS*IC*la*M1*ZH(i1,1)*
     &  ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,4)*
     &  ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,3)*
     &  ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*IC*la*M1*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*IC*la*
     &  M1*ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(4*IC*M12*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv**3)+(4*IC*M12*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**2*vv**3))+CB**3*dTanB4*
     &  IC*((4*IC*M12*ZH(i1,2)*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**3*vv2)+
     &  (4*IC*M12*ZH(i1,5)*ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**3*vv2)-(IC*
     &  M1*DCONJG(la)*ZH(i1,3)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*
     &  sqrt2*vv)-(M1*DCONJG(la)*ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB**2*sqrt2*vv)-(IC*M1*DCONJG(la)*ZH(i1,1)*ZH(l,3)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2*vv)-(M1*DCONJG(la)*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2*vv)-(M1*
     &  DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*
     &  sqrt2*vv)+(IC*M1*DCONJG(la)*ZH(i1,6)*ZH(l,4)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB**2*sqrt2*vv)-(M1*DCONJG(la)*ZH(i1,1)*ZH(l,6)*ZU(j,2)
     &  *ZUC(k,1))/(expvS*SB**2*sqrt2*vv)+(IC*M1*DCONJG(la)*ZH(i1,4)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2*vv)-(expvS*IC*la*
     &  M1*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)+(expvS*
     &  la*M1*ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)-
     &  (expvS*IC*la*M1*ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB**2*
     &  sqrt2*vv)+(expvS*la*M1*ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB**
     &  2*sqrt2*vv)+(expvS*la*M1*ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))
     &  /(SB**2*sqrt2*vv)+(expvS*IC*la*M1*ZH(i1,6)*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB**2*sqrt2*vv)+(expvS*la*M1*ZH(i1,1)*ZH(l,6)*
     &  ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)+(expvS*IC*la*M1*ZH(i1,4)*
     &  ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)+(4*IC*M12*ZH(i1,2)*
     &  ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**3*vv2)+(4*IC*M12*ZH(i1,5)*
     &  ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**3*vv2))+IC*dMT4(1)*((-4*IC*M1*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)-(4*IC*M1*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)+(IC*DCONJG(la)*
     &  ZH(i1,3)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+
     &  (DCONJG(la)*ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv)+(IC*DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*ZH(i1,6)*ZH(l,4)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,1)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(expvS*
     &  IC*la*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*
     &  la*ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)+(expvS*IC*
     &  la*ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(4*IC*M1*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2)-(4*IC*M1*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2))

           do i=1,6
       dCSSSS(l,i1,k,j,1)=dCSSSS(l,i1,k,j,1)+dZHiggs4(l,i)*
     &       CSSSS(i,i1,k,j)/2D0+dZHiggs4(i1,i)*CSSSS(l,i,k,j)/2D0 
           enddo       

           do i=2,3
       dCSSSS(l,i1,k,j,i)=IC*dv4(i)*((4*IC*M12*ZH(i1,2)*ZH(l,2)*ZU(j,1)
     &  *ZUC(k,1))/(SB**2*vv**3)+(4*IC*M12*ZH(i1,5)*ZH(l,5)*ZU(j,1)*
     &  ZUC(k,1))/(SB**2*vv**3)-(IC*M1*DCONJG(la)*ZH(i1,3)*ZH(l,1)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,6)*
     &  ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(IC*M1*
     &  DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(M1*DCONJG(la)*ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2*vv2)+(IC*M1*DCONJG(la)*ZH(i1,6)*ZH(l,4)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,1)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)+(IC*M1*
     &  DCONJG(la)*ZH(i1,4)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(expvS*IC*la*M1*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  sqrt2*vv2)+(expvS*la*M1*ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  sqrt2*vv2)-(expvS*IC*la*M1*ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))
     &  /(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))
     &  /(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))
     &  /(SB*sqrt2*vv2)+(expvS*IC*la*M1*ZH(i1,6)*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,1)*ZH(l,6)*ZU(j,1)*
     &  ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*IC*la*M1*ZH(i1,4)*ZH(l,6)*
     &  ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(4*IC*M12*ZH(i1,2)*ZH(l,2)*
     &  ZU(j,2)*ZUC(k,2))/(SB**2*vv**3)+(4*IC*M12*ZH(i1,5)*ZH(l,5)*
     &  ZU(j,2)*ZUC(k,2))/(SB**2*vv**3))+IC*dMT4(i)*((-4*IC*M1*ZH(i1,2)
     &  *ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)-(4*IC*M1*ZH(i1,5)*
     &  ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)+(IC*DCONJG(la)*ZH(i1,3)*
     &  ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(DCONJG(la)*
     &  ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(IC*
     &  DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv)+(DCONJG(la)*ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*
     &  sqrt2*vv)+(DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*ZH(i1,6)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,1)*ZH(l,6)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*ZH(i1,4)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(expvS*IC*la*
     &  ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)+(expvS*IC*la*
     &  ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(4*IC*M1*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2)-(4*IC*M1*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2))
            enddo
            enddo

           enddo
         enddo
        enddo

 
       do l=ihiggs,ihiggse
       dhy(l,1)=-((CB**3*dTanB4*M1*(ZH(l,2)+IC*ZH(l,5)))/(SB**2*vv))+
     &  (dMT4(1)*(ZH(l,2)+IC*ZH(l,5)))/(SB*vv)-(M1*dv4(1)*(ZH(l,2)+IC*
     &  ZH(l,5)))/(SB*vv2) 
       do i=ihiggs,ihiggse
       dhy(l,1)= dhy(l,1)+dZHiggs4(l,i)* hy(i)/2D0
       enddo
     
       dhyc(l,1)=DCONJG(dhy(l,1))
       do i=2,3
       dhy(l,i)=(dMT4(i)*(ZH(l,2)+IC*ZH(l,5)))/(SB*vv)-(M1*dv4(i)*
     &  (ZH(l,2)+IC*ZH(l,5)))/(SB*vv2)
       dhyc(l,i)=DCONJG(dhy(l,i))
       enddo
       enddo

       do j=ihiggs,ihiggse
        do i=ihiggs,ihiggse
       selfCT(i,j)=6*(-(M12*dhy(j,3)*hy(i))+a0t1*dhyc(j,1)*hy(i)-2*
     &  aet1*dhyc(j,1)*hy(i)-2*a0t1*dhyc(j,2)*hy(i)+M12*dhyc(j,2)*hy(i)
     &  -2*M12*dhyc(j,3)*hy(i)-M12*dhy(i,3)*hy(j)+a0t1*dhyc(i,1)*hy(j)-
     &  2*aet1*dhyc(i,1)*hy(j)-2*a0t1*dhyc(i,2)*hy(j)+M12*dhyc(i,2)*
     &  hy(j)-2*M12*dhyc(i,3)*hy(j)-2*M12*dhy(j,3)*hyc(i)+a0t1*
     &  dhyc(j,1)*hyc(i)-aet1*dhyc(j,1)*hyc(i)-a0t1*dhyc(j,2)*hyc(i)+
     &  M12*dhyc(j,2)*hyc(i)-M12*dhyc(j,3)*hyc(i)+dhy(j,1)*((a0t1-aet1)
     &  *hy(i)+(a0t1-2*aet1)*hyc(i))+dhy(j,2)*((-a0t1+M12)*hy(i)+(-2*
     &  a0t1+M12)*hyc(i))-2*M12*dhy(i,3)*hyc(j)+a0t1*dhyc(i,1)*hyc(j)-
     &  aet1*dhyc(i,1)*hyc(j)-a0t1*dhyc(i,2)*hyc(j)+M12*dhyc(i,2)*
     &  hyc(j)-M12*dhyc(i,3)*hyc(j)+dhy(i,1)*((a0t1-aet1)*hy(j)+(a0t1-
     &  2*aet1)*hyc(j))+dhy(i,2)*((-a0t1+M12)*hy(j)+(-2*a0t1+M12)*
     &  hyc(j))-(2*M1*(hy(i)*((aet1*dmt4(1)+a0t1*(-dmt4(1)+dmt4(2))+
     &  M12*(LogM12*dmt4(1)-2*dmt4(2)+dmt4(3)))*hy(j)+(-2*a0t1*dmt4(1)+
     &  2*aet1*dmt4(1)+LogM12*M12*dmt4(1)+2*a0t1*dmt4(2)-3*M12*dmt4(2)+
     &  2*M12*dmt4(3))*hyc(j))+hyc(i)*((-2*a0t1*dmt4(1)+2*aet1*dmt4(1)+
     &  LogM12*M12*dmt4(1)+2*a0t1*dmt4(2)-3*M12*dmt4(2)+2*M12*dmt4(3))*
     &  hy(j)+(aet1*dmt4(1)+a0t1*(-dmt4(1)+dmt4(2))+M12*(LogM12*dmt4(1)
     &  -2*dmt4(2)+dmt4(3)))*hyc(j))))/M12)
       Do  I6G3= 1,2
       selfCT(i,j)=3*(aAETN(I6G3)*dCSSSS(i,j,I6G3,I6G3,1)+aA0TN(I6G3)*
     &  dCSSSS(i,j,I6G3,I6G3,2)+dCSSSS(i,j,I6G3,I6G3,3)*MassSu2(I6G3))+
     &  selfCT(i,j)
       Do  I6G4= 1,2
       selfCT(i,j)=3*(CSSS(j,I6G4,I6G3)*(aBEpT1(I6G3,I6G4)*
     &  dCSSS(i,I6G3,I6G4,1)+aB0T1(I6G3,I6G4)*dCSSS(i,I6G3,I6G4,2)+
     &  dCSSS(i,I6G3,I6G4,3))+CSSS(i,I6G3,I6G4)*(aBEpT1(I6G3,I6G4)*
     &  dCSSS(j,I6G4,I6G3,1)+aB0T1(I6G3,I6G4)*dCSSS(j,I6G4,I6G3,2)+
     &  dCSSS(j,I6G4,I6G3,3))+CSSSS(i,j,I6G4,I6G3)*(aBEpT1(I6G3,I6G4)*
     &  dMSf4(I6G4,I6G3,1)+aB0T1(I6G3,I6G4)*dMSf4(I6G4,I6G3,2)+
     &  dMSf4(I6G4,I6G3,3)))+selfCT(i,j)
       Do  I6G5= 1,2
       selfCT(i,j)=3*(aCEpT(I6G3,I6G4,I6G5)*(CSSS(i,I6G3,I6G5)*
     &  CSSS(j,I6G5,I6G4)*dMSf4(I6G3,I6G4,1)+CSSS(i,I6G5,I6G3)*
     &  CSSS(j,I6G4,I6G5)*dMSf4(I6G4,I6G3,1))+aC0T(I6G3,I6G4,I6G5)*
     &  (CSSS(i,I6G3,I6G5)*CSSS(j,I6G5,I6G4)*dMSf4(I6G3,I6G4,2)+
     &  CSSS(i,I6G5,I6G3)*CSSS(j,I6G4,I6G5)*dMSf4(I6G4,I6G3,2)))+
     &  selfCT(i,j)
       enddo
       enddo
       enddo
      selfCT(i,j)=selfCT(i,j)*fac
       enddo
       enddo

       CHselfCT=3*(-2*CTBS3(2)*(aet1*DCONJG(dS3TB(1))+a0t1*
     &  DCONJG(dS3TB(2))+M12*DCONJG(dS3TB(3)))-4*CTBS3(2)*M1*CTBS3C(2)*
     &  (((-a0t1+aet1)*dmt4(1))/M12+(-1+a0t1/M12)*dmt4(2)+dmt4(3))-2*
     &  CTBS3C(2)*(aet1*dS3TB(1)+a0t1*dS3TB(2)+M12*dS3TB(3)))
       Do  I6G3= 1,2
       CHselfCT=CHselfCT+3*(aAETN(I6G3)*dCS3S13S(I6G3,I6G3,1)+
     &  aA0TN(I6G3)*dCS3S13S(I6G3,I6G3,2)+CS3S14S13(I6G3)*
     &  (aBEpTH1(I6G3)*dCS3S13S14(I6G3,1)+aB0TH1(I6G3)*
     &  dCS3S13S14(I6G3,2)+dCS3S13S14(I6G3,3))+CS3S13S14(I6G3)*
     &  (aBEpTH1(I6G3)*dCS3S14S13(I6G3,1)+aB0TH1(I6G3)*
     &  dCS3S14S13(I6G3,2)+dCS3S14S13(I6G3,3))+dCS3S13S(I6G3,I6G3,3)*
     &  MassSu2(I6G3))
       enddo
       Do  I6G3= 1,2
       Do  I6G4= 1,2
       CHselfCT=CHselfCT+3*(aCEpTH1(I6G3,I6G4)*CS3S13S14(I6G3)*
     &  CS3S14S13(I6G4)*dMSf4(I6G3,I6G4,1)+aC0TH1(I6G3,I6G4)*
     &  CS3S13S14(I6G3)*CS3S14S13(I6G4)*dMSf4(I6G3,I6G4,2)+
     &  CS3SSS13(I6G4,I6G3)*(aBEpT1(I6G3,I6G4)*dMSf4(I6G4,I6G3,1)+
     &  aB0T1(I6G3,I6G4)*dMSf4(I6G4,I6G3,2)+dMSf4(I6G4,I6G3,3)))
       enddo
       enddo

       Do  I6G5= 1,2
       CHselfCT=CHselfCT+3*CS3S13S14(I6G5)*CS3S14S13(I6G5)*
     &  (aCEpTH2(I6G5)*dMSQ4(1)+aC0TH2(I6G5)*dMSQ4(2))
       enddo

       CHselfCT=CHselfCT+3*(aet5*dCS3SSS14(1)+a0t5*dCS3SSS14(2)+MSQ2*
     &  dCS3SSS14(3)+aBEpT5*CS3SSS14*dMSQ4(1)+aB0T5*CS3SSS14*dMSQ4(2)+
     &  CS3SSS14*dMSQ4(3))
       CHselfCT=CHselfCT*fac

       do i=ihiggs,ihiggse
       TadCT(i)=3*(-2*M1*(aet1*(dhy(i,1)+dhyc(i,1))+a0t1*(dhy(i,2)+
     &  dhyc(i,2))+M12*(dhy(i,3)+dhyc(i,3)))+2*(2*a0t1*dmt4(1)-3*aet1*
     &  dmt4(1)-3*a0t1*dmt4(2)+2*M12*dmt4(2)-3*M12*dmt4(3))*(hy(i)+
     &  hyc(i)))
       Do  I6G2= 1,2
       TadCT(i)=3*(aAETN(I6G2)*dCSSS(i,I6G2,I6G2,1)+aA0TN(I6G2)*
     &  dCSSS(i,I6G2,I6G2,2)+dCSSS(i,I6G2,I6G2,3)*MassSu2(I6G2))+
     &  TadCT(i)
       enddo
       Do  I6G2= 1,2
       Do  I6G3= 1,2
       TadCT(i)=3*CSSS(i,I6G3,I6G2)*(aBEpT1(I6G2,I6G3)*
     &  dMSf4(I6G3,I6G2,1)+aB0T1(I6G2,I6G3)*dMSf4(I6G3,I6G2,2)+
     &  dMSf4(I6G3,I6G2,3))+TadCT(i)
       enddo
       enddo

       TadCT(i)= TadCT(i)*fac
       enddo

#ifdef printMT4
      Write(*,*), "___________________________________________________"
      Write(*,*), "Neutral Higgs Oneloop CT induce diagrams: finite part"
      Write(*,*), "selfCT(1,1-6)",selfCT(1,1),selfCT(1,2),selfCT(1,3),selfCT(1,4),selfCT(1,5),selfCT(1,6)
      Write(*,*), "selfCT(2,2-6)",selfCT(2,2),selfCT(2,3),selfCT(2,4),selfCT(2,5),selfCT(2,6)
      Write(*,*), "selfCT(3,3-6)",selfCT(3,3),selfCT(3,4),selfCT(3,5),selfCT(3,6)
      Write(*,*), "selfCT(4,4-6)",selfCT(4,4),selfCT(4,5),selfCT(4,6)
      Write(*,*), "selfCT(5,5-6)",selfCT(5,5),selfCT(5,6)
      Write(*,*), "selfCT(6,6-6)",selfCT(6,6)
      Write(*,*), "___________________________________________________"

      Write(*,*), "charged Higgs Oneloop CT induce diagrams, DP, SP, finite="  
      Write(*,*),     'CHselfCTDP,CHselfCTSP,CHselfCT=  ',CHselfCT

      Write(*,*), "___________________________________________________"
      Write(*,*), "Tadpole Oneloop CT induce diagrams, DP, SP, finite="  
      Write(*,*), "TadCT(1)",TadCT(1:6)
#endif


      end subroutine CT2loopMT4


c
c**********************************************************************
c
      subroutine CHSelfMT4(SCH)
      implicit none
      integer i,j,gen2,gen3,gen4,h1
      double precision mha,mha2,fac,M22m42I,mahs,M22m42,
     .  M22mMSQ,M42mMSQ,M22mMSQI,M42mMSQI,M14,SCH,
     .  M24,M44,I0A,A0T,AEpT
      double precision IT1,IT2
      double precision I0Acha(6)

      double precision FD111NH5(6,2),FD111NH55(6),FD111NHM1(6,4),
     .     FD111NHM10(6,4),FD211NHM1(6,4),aK0TNH(6),aK0T0NH(6)
      double precision FD111CHMS(2,2,2),FD111CHMSQ(2),FD111CH5(2,2),
     .   FD211CHMSQ(10,2,2),FD211CH0M1(10,2)  
      double complex abrx(15),coef(10),aSCH,toef1(10)
      common/I0NChaFD/IT1,IT2
      common/MT4NHiggTB/FD111NH5,FD111NH55,FD111NHM1,
     .     FD111NHM10,FD211NHM1,aK0TNH,aK0T0NH
      common/MT4CHiggs/FD111CHMS,FD111CHMSQ,FD111CH5,FD211CHMSQ,FD211CH0M1

#include "TLMT4.h"


      fac=1D0/16D0/Pi**4; M14=M12**2;M24=M22**2;M44=M42**2
      M22m42=M22-M42; M22mMSQ=M22-MSQ2; M42mMSQ=M42-MSQ2
      M22m42I= 1/M22m42;M22mMSQI=1/M22mMSQ;M42mMSQI=1/M42mMSQ

c( diagrams can be reduced to one-loop
       aSCH=0
       do gen3=1,2
        do gen4=1,2
         do h1=ihiggs,ihiggse 
       aSCH=aSCH-(3*(A0TNH(h1)*aC0TH1(gen4,gen3)*CS3S13S14(gen
     &  4)*CS3S14S13(gen3)*CSSSS(h1,h1,gen3,gen4)+A0TNH(h1)*aB0T1(gen3,
     &  gen4)*CS3SSS13(gen3,gen4)*CSSSS(h1,h1,gen4,gen3)+AETNH(h1)*CS3S
     &  SS13(gen3,gen4)*CSSSS(h1,h1,gen4,gen3)+aCEpTH1(gen4,gen3)*CS3S1
     &  3S14(gen4)*CS3S14S13(gen3)*CSSSS(h1,h1,gen3,gen4)*Mah2(h1)+aBEp
     &  T1(gen3,gen4)*CS3SSS13(gen3,gen4)*CSSSS(h1,h1,gen4,gen3)*Mah2(h
     &  1)))/32d0
         enddo
        enddo
       enddo

       aSCH=aSCH-(3*(A0THp*aB0T5*CS3SSS14**2+AETHp*CS3SSS14**2+aBEpT5*
     &  CS3SSS14**2*MHp2))/16d0

       do gen3=1,2
       aSCH=aSCH-(3*(A0THp*CS3SSS14*aC0TH2(gen3)*CS3S13S14(gen3)*
     &  CS3S14S13(gen3)+CS3SSS14*MHp2*aCEpTH2(gen3)*CS3S13S14(gen3)*
     &  CS3S14S13(gen3)))/16d0
        do gen4=1,2
       aSCH=aSCH-(3*(A0THp*aC0TH1(gen4,gen3)*CS3S13S14(gen4)*C
     &  S3S14S13(gen3)*CS3SSS13(gen3,gen4)+MHp2*aCEpTH1(gen4,gen3)*CS3S
     &  13S14(gen4)*CS3S14S13(gen3)*CS3SSS13(gen3,gen4)+AETHp*CS3SSS13(
     &  gen3,gen4)*CS3SSS13(gen4,gen3)+A0THp*aB0T1(gen3,gen4)*CS3SSS13(
     &  gen3,gen4)*CS3SSS13(gen4,gen3)+MHp2*aBEpT1(gen3,gen4)*CS3SSS13(
     &  gen3,gen4)*CS3SSS13(gen4,gen3)))/16d0

       aSCH=aSCH-(3*(aA0TN(gen4)*aC0TH2(gen3)*CS14S13S14S13MT4
     &  (gen4,gen4)*CS3S13S14(gen3)*CS3S14S13(gen3)+a0t5*aC0TH1(gen4,ge
     &  n3)*CS14S13S14S13MT4(gen3,gen4)*CS3S13S14(gen4)*CS3S14S13(gen3)
     &  +MSQ2*aCEpTH1(gen4,gen3)*CS14S13S14S13MT4(gen3,gen4)*CS3S13S14(
     &  gen4)*CS3S14S13(gen3)+3*aB0TH1(gen3)*aB0TH1(gen4)*CS14S13S14S13
     &  MT4(gen4,gen3)*CS3S13S14(gen3)*CS3S14S13(gen4)+3*aBEpTH1(gen3)*
     &  CS14S13S14S13MT4(gen4,gen3)*CS3S13S14(gen3)*CS3S14S13(gen4)+3*a
     &  BEpTH1(gen4)*CS14S13S14S13MT4(gen4,gen3)*CS3S13S14(gen3)*CS3S14
     &  S13(gen4)+aet5*CS14S13S14S13MT4(gen4,gen3)*CS3SSS13(gen3,gen4)+
     &  a0t5*aB0T1(gen3,gen4)*CS14S13S14S13MT4(gen4,gen3)*CS3SSS13(gen3
     &  ,gen4)+MSQ2*aBEpT1(gen3,gen4)*CS14S13S14S13MT4(gen4,gen3)*CS3SS
     &  S13(gen3,gen4)+aCEpTH2(gen3)*CS14S13S14S13MT4(gen4,gen4)*CS3S13
     &  S14(gen3)*CS3S14S13(gen3)*MassSu2(gen4)))/16d0

        do gen2=1,2
       aSCH=aSCH-(3*(3*aA0TN(gen2)*aC0TH1(gen4,gen3)*CS13S13S1
     &  3S13MT4(gen3,gen2,gen2,gen4)*CS3S13S14(gen4)*CS3S14S13(gen3)+aA
     &  0TN(gen2)*aC0TH1(gen4,gen3)*CS13S13S13S13MT4(gen3,gen2,gen4,gen
     &  2)*CS3S13S14(gen4)*CS3S14S13(gen3)+aAETN(gen3)*CS13S13S13S13MT4
     &  (gen4,gen3,gen2,gen3)*CS3SSS13(gen2,gen4)+aA0TN(gen3)*aB0T1(gen
     &  2,gen4)*CS13S13S13S13MT4(gen4,gen3,gen2,gen3)*CS3SSS13(gen2,gen
     &  4)+3*aAETN(gen3)*CS13S13S13S13MT4(gen4,gen3,gen3,gen2)*CS3SSS13
     &  (gen2,gen4)+3*aA0TN(gen3)*aB0T1(gen2,gen4)*CS13S13S13S13MT4(gen
     &  4,gen3,gen3,gen2)*CS3SSS13(gen2,gen4)+3*aCEpTH1(gen4,gen3)*CS13
     &  S13S13S13MT4(gen3,gen2,gen2,gen4)*CS3S13S14(gen4)*CS3S14S13(gen
     &  3)*MassSu2(gen2)+aCEpTH1(gen4,gen3)*CS13S13S13S13MT4(gen3,gen2,
     &  gen4,gen2)*CS3S13S14(gen4)*CS3S14S13(gen3)*MassSu2(gen2)+aBEpT1
     &  (gen2,gen4)*CS13S13S13S13MT4(gen4,gen3,gen2,gen3)*CS3SSS13(gen2
     &  ,gen4)*MassSu2(gen3)+3*aBEpT1(gen2,gen4)*CS13S13S13S13MT4(gen4,
     &  gen3,gen3,gen2)*CS3SSS13(gen2,gen4)*MassSu2(gen3)))/16d0
        enddo
        enddo
       enddo

      do gen3=1,2
       aSCH=aSCH-(3*(aB0T5*CS3SSS14*aA0TN(gen3)*
     &  CS14S13S14S13MT4(gen3,gen3)+CS3SSS14*aAETN(gen3)*
     &  CS14S13S14S13MT4(gen3,gen3)+aBEpT5*CS3SSS14*
     &  CS14S13S14S13MT4(gen3,gen3)*MassSu2(gen3)))/16d0
       enddo
      
      call NHiggs_TBN(M12,M22,M42,MSQ2,Mah2,a0t1,a0t2,a0t4,a0t5
     .     ,A0TNH,aet1,aet2,aet4,aet5,AETNH,Q2)

      do h1=ihiggs,ihiggse
       aSCH=aSCH-(3*CTBS3(2)*CTBS3C(2)*(M14*FD211NHM1(h1,3)*hy(h
     &  1)**2-M12*FD111NHM1(h1,2)*hy(h1)*hyc(h1)-FD111NHM1(h1,4)*hy(h1)
     &  *hyc(h1)+M12*FD111NHM10(h1,2)*hy(h1)*hyc(h1)+FD111NHM10(h1,4)*h
     &  y(h1)*hyc(h1)+M14*FD211NHM1(h1,2)*hy(h1)*hyc(h1)+M12*FD211NHM1(
     &  h1,4)*hy(h1)*hyc(h1)+M14*FD211NHM1(h1,3)*hyc(h1)**2-M12*FD111NH
     &  M1(h1,3)*(hy(h1)**2+hyc(h1)**2)+M12*FD111NHM10(h1,3)*(hy(h1)**2
     &  +hyc(h1)**2)))/(8.*M14)
      enddo


      abrx(1:13)=0D0
      do h1=ihiggs,ihiggse
       abrx(1)=abrx(1)+(3*M22m42I*CSSS(h1,1,2)*(CSSS(h1,1,1)*FD11
     &  1MH(h1,1,1)-CSSS(h1,1,1)*FD111MH(h1,1,2)+CSSS(h1,2,2)*FD111MH(h
     &  1,1,2)-CSSS(h1,2,2)*FD111MH(h1,2,2)))/16.
       abrx(2)=abrx(2)+(3*M22m42I*CSSS(h1,2,1)*(CSSS(h1,1,1)*FD11
     &  1MH(h1,1,1)-CSSS(h1,1,1)*FD111MH(h1,1,2)+CSSS(h1,2,2)*FD111MH(h
     &  1,1,2)-CSSS(h1,2,2)*FD111MH(h1,2,2)))/16.
       abrx(3)=abrx(3)+(3*(CS1S3SS(h1,2,1)*CS1S3SSC(h1,2,1)*FD111
     &  NH5(h1,1)+CS1S3SS(h1,2,2)*CS1S3SSC(h1,2,2)*FD111NH5(h1,2)))/16.
     &  
       abrx(4)=abrx(4)+(3*M22mMSQI*(CS1S3SS(h1,2,1)*CSSS(h1,1,1)*
     &  FD111MH(h1,1,1)+CS1S3SS(h1,2,2)*CSSS(h1,1,2)*FD111MH(h1,1,2)-CS
     &  1S3SS(h1,2,1)*CSSS(h1,1,1)*FD111NH5(h1,1)-CS1S3SS(h1,2,2)*CSSS(
     &  h1,1,2)*FD111NH5(h1,2)))/16.
       abrx(5)=abrx(5)+(3*M22mMSQI*(CS1S3SSC(h1,2,1)*CSSS(h1,1,1)
     &  *FD111MH(h1,1,1)+CS1S3SSC(h1,2,2)*CSSS(h1,2,1)*FD111MH(h1,1,2)-
     &  CS1S3SSC(h1,2,1)*CSSS(h1,1,1)*FD111NH5(h1,1)-CS1S3SSC(h1,2,2)*C
     &  SSS(h1,2,1)*FD111NH5(h1,2)))/16.
       abrx(6)=abrx(6)+(3*M42mMSQI*(CS1S3SS(h1,2,1)*CSSS(h1,2,1)*
     &  FD111MH(h1,1,2)+CS1S3SS(h1,2,2)*CSSS(h1,2,2)*FD111MH(h1,2,2)-CS
     &  1S3SS(h1,2,1)*CSSS(h1,2,1)*FD111NH5(h1,1)-CS1S3SS(h1,2,2)*CSSS(
     &  h1,2,2)*FD111NH5(h1,2)))/16.
       abrx(7)=abrx(7)+(3*M42mMSQI*(CS1S3SSC(h1,2,1)*CSSS(h1,1,2)
     &  *FD111MH(h1,1,2)+CS1S3SSC(h1,2,2)*CSSS(h1,2,2)*FD111MH(h1,2,2)-
     &  CS1S3SSC(h1,2,1)*CSSS(h1,1,2)*FD111NH5(h1,1)-CS1S3SSC(h1,2,2)*C
     &  SSS(h1,2,2)*FD111NH5(h1,2)))/16.
       abrx(8)=abrx(8)+(3*M22m42I*M22mMSQI*M42mMSQI*CSSS(h1,1,2)*
     &  (M42mMSQ*CSSS(h1,1,1)*FD111MH(h1,1,1)-M22*CSSS(h1,1,1)*FD111MH(
     &  h1,1,2)+MSQ2*CSSS(h1,1,1)*FD111MH(h1,1,2)+M42mMSQ*CSSS(h1,2,2)*
     &  FD111MH(h1,1,2)-M22*CSSS(h1,2,2)*FD111MH(h1,2,2)+MSQ2*CSSS(h1,2
     &  ,2)*FD111MH(h1,2,2)+M22m42*CSSS(h1,1,1)*FD111NH5(h1,1)+M22m42*C
     &  SSS(h1,2,2)*FD111NH5(h1,2)))/16.
       abrx(9)=abrx(9)+(3*M22m42I*M22mMSQI*M42mMSQI*CSSS(h1,2,1)*
     &  (M42mMSQ*CSSS(h1,1,1)*FD111MH(h1,1,1)-M22*CSSS(h1,1,1)*FD111MH(
     &  h1,1,2)+MSQ2*CSSS(h1,1,1)*FD111MH(h1,1,2)+M42mMSQ*CSSS(h1,2,2)*
     &  FD111MH(h1,1,2)-M22*CSSS(h1,2,2)*FD111MH(h1,2,2)+MSQ2*CSSS(h1,2
     &  ,2)*FD111MH(h1,2,2)+M22m42*CSSS(h1,1,1)*FD111NH5(h1,1)+M22m42*C
     &  SSS(h1,2,2)*FD111NH5(h1,2)))/16.
       abrx(10)=abrx(10)+(3*(CSSS(h1,1,1)**2*FD211MH(h1,1,1)+CSSS(
     &  h1,1,2)*CSSS(h1,2,1)*FD211MH(h1,1,2)))/16.
       abrx(11)=abrx(11)+(3*M22mMSQI**2*(-(CSSS(h1,1,1)**2*FD111MH
     &  (h1,1,1))-CSSS(h1,1,2)*CSSS(h1,2,1)*FD111MH(h1,1,2)+CSSS(h1,1,1
     &  )**2*FD111NH5(h1,1)+CSSS(h1,1,2)*CSSS(h1,2,1)*FD111NH5(h1,2)+M2
     &  2mMSQ*CSSS(h1,1,1)**2*FD211MH(h1,1,1)+M22mMSQ*CSSS(h1,1,2)*CSSS
     &  (h1,2,1)*FD211MH(h1,1,2)))/16.
       abrx(12)=abrx(12)+(3*(CSSS(h1,1,2)*CSSS(h1,2,1)*FD211MH(h1,
     &  2,1)+CSSS(h1,2,2)**2*FD211MH(h1,2,2)))/16.
       abrx(13)=abrx(13)+(3*M42mMSQI**2*(-(CSSS(h1,1,2)*CSSS(h1,2,
     &  1)*FD111MH(h1,1,2))-CSSS(h1,2,2)**2*FD111MH(h1,2,2)+CSSS(h1,1,2
     &  )*CSSS(h1,2,1)*FD111NH5(h1,1)+CSSS(h1,2,2)**2*FD111NH5(h1,2)+M4
     &  2mMSQ*CSSS(h1,1,2)*CSSS(h1,2,1)*FD211MH(h1,2,1)+M42mMSQ*CSSS(h1
     &  ,2,2)**2*FD211MH(h1,2,2)))/16.
      enddo
       aSCH=aSCH+abrx(3)+abrx(4)*CS3S14S13(1)+abrx(6)*CS3S14S1
     &  3(2)+CS3S13S14(1)*(abrx(5)+abrx(11)*CS3S14S13(1)+abrx(9)*CS3S14
     &  S13(2))+CS3S13S14(2)*(abrx(7)+abrx(8)*CS3S14S13(1)+abrx(13)*CS3
     &  S14S13(2))+abrx(10)*CS3SSS13(1,1)+abrx(2)*CS3SSS13(1,2)+abrx(1)
     &  *CS3SSS13(2,1)+abrx(12)*CS3SSS13(2,2)

       call CHiggsN(M12,MHp2,M22,M42,MSQ2,aet1,AETHp,aet2,
     &   aet4,aet5,a0t1,A0THp,a0t2,a0t4,a0t5,Q2)

       aSCH=aSCH+(-3*CTBS3(2)*CTBS3(1)*CTBS3C(1)*CTBS3C(2)*(a0t1**2+
     &  2*a0t1*M12-3*aet1*M12+M12**2+M12*I0A(0D0,0D0,M12,Q2)+2*M12*I0A(
     &  0D0,M12,M12,Q2)))/(16.*M12)
       aSCH=aSCH+(-3*CTBS3(2)*CTBS3(2)*CTBS3C(2)**2*(a0t1**2-a0t
     &  1*A0THp+2*a0t1*M12+2*A0THp*M12-3*aet1*M12-4*AETHp*M12+M12**2+M1
     &  2*MHp2+(M12+MHp2)*I0A(0D0,M12,MHp2,Q2)+(2*M12-MHp2)*I0A(M12,M12,M
     &  Hp2,Q2)))/(16.*M12)


       do h1=1,2
       abrx(1)=(3*M22m42I*(FD111CH(h1,1)-FD111CH(h1,2)))/16.
       abrx(2)=(3*FD111CHMS(h1,1,1))/16.
       abrx(3)=(-3*M22mMSQI*(FD111CH(h1,1)-FD111CHMS(h1,1,1)))/16
     &  .
       abrx(4)=(3*FD111CHMS(h1,1,2))/16.
       abrx(5)=(-3*M22mMSQI*(FD111CH(h1,2)-FD111CHMS(h1,1,2)))/16
     &  .
       abrx(6)=(-3*M42mMSQI*(FD111CH(h1,1)-FD111CHMS(h1,1,2)))/16
     &  .
       abrx(7)=(3*FD111CHMS(h1,2,2))/16.
       abrx(8)=(-3*M42mMSQI*(FD111CH(h1,2)-FD111CHMS(h1,2,2)))/16
     &  .
       abrx(9)=(-3*M22m42I*M22mMSQI*M42mMSQI*((-M42+MSQ2)*FD111CH
     &  (h1,1)+(M22-MSQ2)*FD111CH(h1,2)-M22m42*FD111CHMSQ(h1)))/16.
       abrx(10)=(3*FD211CH(h1,1))/16.
       abrx(11)=(3*FD211CH(h1,2))/16.
       abrx(12)=(3*M22mMSQI**2*(-2*FD111CH(h1,1)+FD111CHMS(h1,1,1)
     &  +FD111CHMSQ(h1)+M22*FD211CH(h1,1)-MSQ2*FD211CH(h1,1)-M22*FD211C
     &  HMSQ(1,h1,1)+MSQ2*FD211CHMSQ(1,h1,1)))/16.
       abrx(13)=(-3*M42mMSQI**2*(FD111CH(h1,1)-FD111CHMS(h1,1,2)+(
     &  M42-MSQ2)*FD211CHMSQ(1,h1,1)))/16.
       abrx(14)=(-3*M22mMSQI**2*(FD111CH(h1,2)-FD111CHMS(h1,1,2)+(
     &  M22-MSQ2)*FD211CHMSQ(1,h1,2)))/16.
       abrx(15)=(3*M42mMSQI**2*(-2*FD111CH(h1,2)+FD111CHMS(h1,2,2)
     &  +FD111CHMSQ(h1)+M42*FD211CH(h1,2)-MSQ2*FD211CH(h1,2)-M42*FD211C
     &  HMSQ(1,h1,2)+MSQ2*FD211CHMSQ(1,h1,2)))/16.
       aSCH=aSCH+(abrx(2)*CS3S3S13S13(2,h1,1,1)*CS3S3S13S13(h1
     &  ,2,1,1)+abrx(4)*CS3S3S13S13(2,h1,2,1)*CS3S3S13S13(h1,2,1,2)+CS3
     &  S13SC(h1,1)*(abrx(3)*CS3S13S14(1)*CS3S3S13S13(h1,2,1,1)+abrx(6)
     &  *CS3S13S14(2)*CS3S3S13S13(h1,2,1,2))+abrx(4)*CS3S3S13S13(2,h1,1
     &  ,2)*CS3S3S13S13(h1,2,2,1)+abrx(7)*CS3S3S13S13(2,h1,2,2)*CS3S3S1
     &  3S13(h1,2,2,2)+CS3S13SC(h1,2)*(abrx(5)*CS3S13S14(1)*CS3S3S13S13
     &  (h1,2,2,1)+abrx(8)*CS3S13S14(2)*CS3S3S13S13(h1,2,2,2))+CS3S13S(
     &  h1,1)*(abrx(3)*CS3S14S13(1)*CS3S3S13S13(2,h1,1,1)+abrx(6)*CS3S1
     &  4S13(2)*CS3S3S13S13(2,h1,2,1)+CS3S13SC(h1,1)*(abrx(12)*CS3S13S1
     &  4(1)*CS3S14S13(1)+abrx(13)*CS3S13S14(2)*CS3S14S13(2)+abrx(10)*C
     &  S3SSS13(1,1))+CS3S13SC(h1,2)*(abrx(9)*CS3S13S14(2)*CS3S14S13(1)
     &  +abrx(1)*CS3SSS13(2,1)))+CS3S13S(h1,2)*(abrx(5)*CS3S14S13(1)*CS
     &  3S3S13S13(2,h1,1,2)+abrx(8)*CS3S14S13(2)*CS3S3S13S13(2,h1,2,2)+
     &  CS3S13SC(h1,1)*(abrx(9)*CS3S13S14(1)*CS3S14S13(2)+abrx(1)*CS3SS
     &  S13(1,2))+CS3S13SC(h1,2)*(abrx(14)*CS3S13S14(1)*CS3S14S13(1)+ab
     &  rx(15)*CS3S13S14(2)*CS3S14S13(2)+abrx(11)*CS3SSS13(2,2))))
      enddo

       do h1=1,2
       abrx(1)=(3*CS3S3S14S14(2,h1)*CS3S3S14S14(h1,2)*FD111CHMSQ(h1))
     &         /16.
       abrx(2)=(3*M22mMSQI*CS3S3S14S14(2,h1)*(FD111CH(h1,1)-
     &  FD111CHMSQ(h1)))/16.
       abrx(3)=(3*M42mMSQI*CS3S3S14S14(2,h1)*(FD111CH(h1,2)-
     &  FD111CHMSQ(h1)))/16.
       abrx(4)=(3*M22mMSQI*CS3S3S14S14(h1,2)*(FD111CH(h1,1)-
     &  FD111CHMSQ(h1)))/16.
       abrx(5)=(3*M42mMSQI*CS3S3S14S14(h1,2)*(FD111CH(h1,2)-
     &  FD111CHMSQ(h1)))/16.
       abrx(6)=(3*CS3SSS14*FD211CHMSQ(1,h1,1))/16.
       abrx(7)=(3*CS3SSS14*FD211CHMSQ(1,h1,2))/16.
       aSCH=aSCH+(abrx(1)+abrx(4)*CS3S13S14(1)*CS3S13SC(h1,1)+abrx(5)*
     &  CS3S13S14(2)*CS3S13SC(h1,2)+CS3S13S(h1,1)*(abrx(6)*
     &  CS3S13SC(h1,1)+abrx(2)*CS3S14S13(1))+CS3S13S(h1,2)*(abrx(7)*
     &  CS3S13SC(h1,2)+abrx(3)*CS3S14S13(2)))
       enddo

      do h1=ineu,ineue

       abrx(1)=(3*MNeu(h1)*(a0t2*A0TNeu(h1)+M22*AETNeu(h1)+FD111C
     &  hi(3,h1,1)-M12*FD211ChiM1(4,h1,1)+aet2*MNeu2(h1)))/(8.*M1**3)
       abrx(2)=(3*MNeu(h1)*(a0t4*A0TNeu(h1)+M42*AETNeu(h1)+FD111C
     &  hi(3,h1,2)-M12*FD211ChiM1(4,h1,2)+aet4*MNeu2(h1)))/(8.*M1**3)
       abrx(3)=(3*(-(M12*FD111Chi(8,h1,1))+M12*FD111Chi(9,h1,1)+M
     &  14*FD211ChiM1(5,h1,1)-M14*FD211ChiM1(8,h1,1)+aet2*M12*MNeu(h1)*
     &  *4+a0t2*M12*A0TNeu(h1)*MNeu2(h1)+M12*M22*AETNeu(h1)*MNeu2(h1)))
     &  /(8.*M12*M14)
       abrx(4)=(3*(-(M12*FD111Chi(8,h1,2))+M12*FD111Chi(9,h1,2)+M
     &  14*FD211ChiM1(5,h1,2)-M14*FD211ChiM1(8,h1,2)+aet4*M12*MNeu(h1)*
     &  *4+a0t4*M12*A0TNeu(h1)*MNeu2(h1)+M12*M42*AETNeu(h1)*MNeu2(h1)))
     &  /(8.*M12*M14)
       abrx(5)=(3*(-(aet2*M22)+a0t2*A0TNeu(h1)-2*FD111Chi(2,h1,1)
     &  +2*FD111Chi(3,h1,1)+2*M12*FD211ChiM1(2,h1,1)-2*M12*FD211ChiM1(4
     &  ,h1,1)+M22*I0A(0D0,M22,MNeu2(h1),Q2)+2*aet2*MNeu2(h1)+AETNeu(h1)*
     &  MNeu2(h1)-I0A(0D0,M22,MNeu2(h1),Q2)*MNeu2(h1)))/(16.*M12)
       abrx(6)=(3*(-(aet4*M42)+a0t4*A0TNeu(h1)-2*FD111Chi(2,h1,2)
     &  +2*FD111Chi(3,h1,2)+2*M12*FD211ChiM1(2,h1,2)-2*M12*FD211ChiM1(4
     &  ,h1,2)+M42*I0A(0D0,M42,MNeu2(h1),Q2)+2*aet4*MNeu2(h1)+AETNeu(h1)*
     &  MNeu2(h1)-I0A(0D0,M42,MNeu2(h1),Q2)*MNeu2(h1)))/(16.*M12)
       aSCH=aSCH+CTBS3(2)*(CF11FSL(h1,1)*(abrx(5)*CF11FSLC(h1,1)
     &  +abrx(1)*CF11FSRC(h1,1))+CF11FSR(h1,1)*(abrx(1)*CF11FSLC(h1,1)+
     &  abrx(3)*CF11FSRC(h1,1))+CF11FSL(h1,2)*(abrx(6)*CF11FSLC(h1,2)+a
     &  brx(2)*CF11FSRC(h1,2))+CF11FSR(h1,2)*(abrx(2)*CF11FSLC(h1,2)+ab
     &  rx(4)*CF11FSRC(h1,2)))*CTBS3C(2)
      enddo



      do h1=ineu,ineue
       abrx(1)=(3*M22m42I*(FD111Chi(2,h1,1)-FD111Chi(2,h1,2)-FD11
     &  1Chi(3,h1,1)+FD111Chi(3,h1,2)))/8.
       abrx(2)=(3*M22m42I*M22mMSQI*M42mMSQI*((M42-MSQ2)*FD111Chi(
     &  2,h1,1)+(-M22+MSQ2)*FD111Chi(2,h1,2)-M42*FD111Chi(3,h1,1)+MSQ2*
     &  FD111Chi(3,h1,1)+M22*FD111Chi(3,h1,2)-MSQ2*FD111Chi(3,h1,2)+M22
     &  *FD111Chi5M1(2,h1)-M42*FD111Chi5M1(2,h1)-M22*FD111Chi5M1(3,h1)+
     &  M42*FD111Chi5M1(3,h1)))/8.
       abrx(3)=(3*(FD211Chi(2,h1,1)-FD211Chi(3,h1,1)))/8.
       abrx(4)=(3*M22mMSQI**2*(-FD111Chi(2,h1,1)+FD111Chi(3,h1,1)
     &  +FD111Chi5M1(2,h1)-FD111Chi5M1(3,h1)+M22*FD211Chi(2,h1,1)-MSQ2*
     &  FD211Chi(2,h1,1)-M22*FD211Chi(3,h1,1)+MSQ2*FD211Chi(3,h1,1)))/8
     &  .
       abrx(5)=(3*(FD211Chi(2,h1,2)-FD211Chi(3,h1,2)))/8.
       abrx(6)=(3*M42mMSQI**2*(-FD111Chi(2,h1,2)+FD111Chi(3,h1,2)
     &  +FD111Chi5M1(2,h1)-FD111Chi5M1(3,h1)+M42*FD211Chi(2,h1,2)-MSQ2*
     &  FD211Chi(2,h1,2)-M42*FD211Chi(3,h1,2)+MSQ2*FD211Chi(3,h1,2)))/8
     &  .
       abrx(7)=(3*M1*M22m42I*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2))
     &  *MNeu(h1))/8.
       abrx(8)=(-3*M1*M22m42I*M22mMSQI*M42mMSQI*((M42-MSQ2)*FD111
     &  Chi(1,h1,1)+(-M22+MSQ2)*FD111Chi(1,h1,2)+M22m42*FD111Chi5M1(1,h
     &  1))*MNeu(h1))/8.
       abrx(9)=(-3*M1*FD211Chi(1,h1,1)*MNeu(h1))/8.
       abrx(10)=(3*M1*M22mMSQI**2*(FD111Chi(1,h1,1)-FD111Chi5M1(1,
     &  h1)+(-M22+MSQ2)*FD211Chi(1,h1,1))*MNeu(h1))/8.
       abrx(11)=(-3*M1*FD211Chi(1,h1,2)*MNeu(h1))/8.
       abrx(12)=(3*M1*M42mMSQI**2*(FD111Chi(1,h1,2)-FD111Chi5M1(1,
     &  h1)+(-M42+MSQ2)*FD211Chi(1,h1,2))*MNeu(h1))/8.
       aSCH=aSCH+(CF11FSR(h1,1)*(CF11FSRC(h1,1)*(abrx(4)*CS3S1
     &  3S14(1)*CS3S14S13(1)+abrx(3)*CS3SSS13(1,1))+CF11FSLC(h1,1)*(abr
     &  x(10)*CS3S13S14(1)*CS3S14S13(1)+abrx(9)*CS3SSS13(1,1))+CF11FSRC
     &  (h1,2)*(abrx(2)*CS3S13S14(1)*CS3S14S13(2)+abrx(1)*CS3SSS13(1,2)
     &  )+CF11FSLC(h1,2)*(abrx(8)*CS3S13S14(1)*CS3S14S13(2)+abrx(7)*CS3
     &  SSS13(1,2)))+CF11FSL(h1,1)*(CF11FSLC(h1,1)*(abrx(4)*CS3S13S14(1
     &  )*CS3S14S13(1)+abrx(3)*CS3SSS13(1,1))+CF11FSRC(h1,1)*(abrx(10)*
     &  CS3S13S14(1)*CS3S14S13(1)+abrx(9)*CS3SSS13(1,1))+CF11FSLC(h1,2)
     &  *(abrx(2)*CS3S13S14(1)*CS3S14S13(2)+abrx(1)*CS3SSS13(1,2))+CF11
     &  FSRC(h1,2)*(abrx(8)*CS3S13S14(1)*CS3S14S13(2)+abrx(7)*CS3SSS13(
     &  1,2)))+CF11FSR(h1,2)*(CF11FSRC(h1,1)*(abrx(2)*CS3S13S14(2)*CS3S
     &  14S13(1)+abrx(1)*CS3SSS13(2,1))+CF11FSLC(h1,1)*(abrx(8)*CS3S13S
     &  14(2)*CS3S14S13(1)+abrx(7)*CS3SSS13(2,1))+CF11FSRC(h1,2)*(abrx(
     &  6)*CS3S13S14(2)*CS3S14S13(2)+abrx(5)*CS3SSS13(2,2))+CF11FSLC(h1
     &  ,2)*(abrx(12)*CS3S13S14(2)*CS3S14S13(2)+abrx(11)*CS3SSS13(2,2))
     &  )+CF11FSL(h1,2)*(CF11FSLC(h1,1)*(abrx(2)*CS3S13S14(2)*CS3S14S13
     &  (1)+abrx(1)*CS3SSS13(2,1))+CF11FSRC(h1,1)*(abrx(8)*CS3S13S14(2)
     &  *CS3S14S13(1)+abrx(7)*CS3SSS13(2,1))+CF11FSLC(h1,2)*(abrx(6)*CS
     &  3S13S14(2)*CS3S14S13(2)+abrx(5)*CS3SSS13(2,2))+CF11FSRC(h1,2)*(
     &  abrx(12)*CS3S13S14(2)*CS3S14S13(2)+abrx(11)*CS3SSS13(2,2))))
      enddo


       abrx(1)=(3*CS3SSS14*(FD211Cha5(2)-FD211Cha5(3)))/8.
       aSCH=aSCH+CF12F3S*CF12F3SC*abrx(1)
       I0Acha(1)=I0A(0D0,M22,MCha2,Q2)
       I0Acha(2)=I0A(0D0,M42,MCha2,Q2)
       I0Acha(3)=I0A(0D0,MSQ2,MCha2,Q2)
       I0Acha(4)=I0A(M12,M22,MCha2,Q2)
       I0Acha(5)=I0A(M12,M42,MCha2,Q2)
       I0Acha(6)=I0A(M12,MSQ2,MCha2,Q2)
       abrx(1)=(3*(a0t5*a0tcha*M22-aet2*M24+M24*M22+2*aet5*M22*MCha2+
     &  M24*MCha2+a0t2*(a0tcha*(M22-2*MSQ2)+2*(M22-MCha2)*(M22-MSQ2))+2
     &  *aet2*M22*MSQ2-aet5*M22*MSQ2-M24*MSQ2-2*aet2*MCha2*MSQ2-M22*MCh
     &  a2*MSQ2+M24*I0Acha(1)+M22*MCha2*I0Acha(1)-2*M22*MSQ2*I0Acha(1)-
     &  M22*MCha2*I0Acha(3)+M22*MSQ2*I0Acha(3)))/(16.*(M2**3-M2*MSQ2)**
     &  2)
       abrx(2)=(3*(M4**6+a0t5*a0tcha*M42-aet4*M44+2*aet5*M42*MCha
     &  2+M44*MCha2+a0t4*(a0tcha*(M42-2*MSQ2)+2*(M42-MCha2)*(M42-MSQ2))
     &  +2*aet4*M42*MSQ2-aet5*M42*MSQ2-M44*MSQ2-2*aet4*MCha2*MSQ2-M42*M
     &  Cha2*MSQ2+M44*I0Acha(2)+M42*MCha2*I0Acha(2)-2*M42*MSQ2*I0Acha(2
     &  )-M42*MCha2*I0Acha(3)+M42*MSQ2*I0Acha(3)))/(16.*(M4**3-M4*MSQ2)
     &  **2)
       abrx(3)=(3*(a0t5*a0tcha*M22m42+a0t2*a0tcha*M42-aet2*M22*M4
     &  2+aet4*M22*M42-2*aet4*M22*MCha2+2*aet5*M22*MCha2+2*aet2*M42*MCh
     &  a2-2*aet5*M42*MCha2-a0t2*a0tcha*MSQ2+aet2*M22*MSQ2-aet5*M22*MSQ
     &  2-aet4*M42*MSQ2+aet5*M42*MSQ2-2*aet2*MCha2*MSQ2+2*aet4*MCha2*MS
     &  Q2+a0t4*a0tcha*(-M22+MSQ2)+M22*M42*I0Acha(1)-M42*MCha2*I0Acha(1
     &  )-M22*MSQ2*I0Acha(1)+MCha2*MSQ2*I0Acha(1)-M22*M42*I0Acha(2)+M22
     &  *MCha2*I0Acha(2)+M42*MSQ2*I0Acha(2)-MCha2*MSQ2*I0Acha(2)-M22*MC
     &  ha2*I0Acha(3)+M42*MCha2*I0Acha(3)+M22*MSQ2*I0Acha(3)-M42*MSQ2*I
     &  0Acha(3)))/(16.*M22m42*M22mMSQ*M42mMSQ)
       abrx(4)=(3*(a0t1*(a0t2-a0t5)+a0t2*a0tcha-a0t5*a0tcha+2*aet
     &  2*M12-2*aet5*M12-aet2*M22+2*aet2*MCha2-2*aet5*MCha2+aet5*MSQ2-2
     &  *M22*FD211Cha5(2)+2*MSQ2*FD211Cha5(2)+2*M22*FD211Cha5(3)-2*MSQ2
     &  *FD211Cha5(3)-M12*I0Acha(4)+M22*I0Acha(4)-MCha2*I0Acha(4)+M12*I
     &  0Acha(6)+MCha2*I0Acha(6)-MSQ2*I0Acha(6)))/(16.*M22mMSQ**2)
       abrx(5)=(3*(a0t1*(a0t4-a0t5)+a0t4*a0tcha-a0t5*a0tcha+2*aet
     &  4*M12-2*aet5*M12-aet4*M42+2*aet4*MCha2-2*aet5*MCha2+aet5*MSQ2-2
     &  *M42*FD211Cha5(2)+2*MSQ2*FD211Cha5(2)+2*M42*FD211Cha5(3)-2*MSQ2
     &  *FD211Cha5(3)-M12*I0Acha(5)+M42*I0Acha(5)-MCha2*I0Acha(5)+M12*I
     &  0Acha(6)+MCha2*I0Acha(6)-MSQ2*I0Acha(6)))/(16.*M42mMSQ**2)
       abrx(6)=(3*M42mMSQI**2*(-FD111Chi(2,h1,2)+FD111Chi(3,h1,2)
     &  +FD111Chi5M1(2,h1)-FD111Chi5M1(3,h1)+M42*FD211Chi(2,h1,2)-MSQ2*
     &  FD211Chi(2,h1,2)-M42*FD211Chi(3,h1,2)+MSQ2*FD211Chi(3,h1,2)))/8
     &  .
       abrx(7)=(3*M1*M22m42I*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2))
     &  *MNeu(h1))/8.
       abrx(8)=(-3*M1*M22m42I*M22mMSQI*M42mMSQI*((M42-MSQ2)*FD111
     &  Chi(1,h1,1)+(-M22+MSQ2)*FD111Chi(1,h1,2)+M22m42*FD111Chi5M1(1,h
     &  1))*MNeu(h1))/8.
       abrx(9)=(-3*M1*FD211Chi(1,h1,1)*MNeu(h1))/8.
       abrx(10)=(3*M1*M22mMSQI**2*(FD111Chi(1,h1,1)-FD111Chi5M1(1,
     &  h1)+(-M22+MSQ2)*FD211Chi(1,h1,1))*MNeu(h1))/8.
       abrx(11)=(-3*M1*FD211Chi(1,h1,2)*MNeu(h1))/8.
       abrx(12)=(3*M1*M42mMSQI**2*(FD111Chi(1,h1,2)-FD111Chi5M1(1,
     &  h1)+(-M42+MSQ2)*FD211Chi(1,h1,2))*MNeu(h1))/8.
       aSCH=aSCH+(CF12F4S(1)*(abrx(1)*CF12F4SC(1)*CS3S13S14(1)
     &  *CS3S14S13(1)+abrx(3)*CF12F4SC(2)*CS3S13S14(1)*CS3S14S13(2))+CF
     &  12F3S*CF12F3SC*(abrx(4)*CS3S13S14(1)*CS3S14S13(1)+abrx(5)*CS3S1
     &  3S14(2)*CS3S14S13(2))+CF12F4S(2)*(abrx(3)*CF12F4SC(1)*CS3S13S14
     &  (2)*CS3S14S13(1)+abrx(2)*CF12F4SC(2)*CS3S13S14(2)*CS3S14S13(2))
     &  )


       abrx(1)=(3*(-2*aet2*(M22-MCha2)+2*a0t2*(a0tcha+M22-MCha2)+
     &  M22*(M22+MCha2+2*I0Acha(1))))/(16.*M22)
       abrx(2)=(3*(-2*aet4*(M42-MCha2)+2*a0t4*(a0tcha+M42-MCha2)+
     &  M42*(M42+MCha2+2*I0Acha(2))))/(16.*M42)
       abrx(3)=(3*(a0t2*a0tcha-a0t4*a0tcha-aet2*M22+aet4*M42+2*ae
     &  t2*MCha2-2*aet4*MCha2+M22*I0Acha(1)-MCha2*I0Acha(1)-M42*I0Acha(
     &  2)+MCha2*I0Acha(2)))/(16.*M22m42)
       abrx(4)=(3*(a0t1*(a0t2-a0tcha)+aet1*M12+2*aet2*M12-M22*I0A
     &  cha(1)+MCha2*I0Acha(1)-M12*I0Acha(4)+M22*I0Acha(4)-MCha2*I0Acha
     &  (4)))/(16.*M12)
       abrx(5)=(3*(a0t1*(a0t4-a0tcha)+aet1*M12+2*aet4*M12-M42*I0A
     &  cha(2)+MCha2*I0Acha(2)-M12*I0Acha(5)+M42*I0Acha(5)-MCha2*I0Acha
     &  (5)))/(16.*M12)
       abrx(6)=(3*(2*aet1*M12*M14+2*aet5*M12*M14-M14*M14+2*a0tcha
     &  *M12*MCha**4-2*aet1*M12*MCha**4-2*aet5*M12*MCha**4-2*aetcha*M12
     &  *MCha**4+M14*MCha**4+2*aet1*MCha**6+2*aet5*MCha**6+aetcha*MCha*
     &  *6+M12*MCha**6-2*a0tcha*M12**2*MCha2+2*aet1*M12**2*MCha2+4*aet5
     &  *M12**2*MCha2+4*aetcha*M12**2*MCha2-6*aet1*M14*MCha2-8*aet5*M14
     &  *MCha2-4*aetcha*M14*MCha2-M12*M14*MCha2-2*aet1*MCha**4*MCha2-2*
     &  aet5*MCha**4*MCha2-aetcha*MCha**4*MCha2+4*aet1*M12*MCha2**2+4*a
     &  et5*M12*MCha2**2+2*aetcha*M12*MCha2**2-aet5*MSQ2**3-M12*MSQ2**3-2
     &  *a0tcha*M12**2*MSQ2+2*aet1*M12**2*MSQ2+4*aet5*M12**2*MSQ2+4*aet
     &  cha*M12**2*MSQ2+2*a0tcha*M14*MSQ2-6*aet1*M14*MSQ2-8*aet5*M14*MS
     &  Q2-4*aetcha*M14*MSQ2+M12*M14*MSQ2-4*aet1*MCha**4*MSQ2-4*aet5*MC
     &  ha**4*MSQ2-2*aetcha*MCha**4*MSQ2+M12*MCha2**2*MSQ2-2*a0tcha*M12*
     &  MCha2*MSQ2-4*aet1*M12*MCha2*MSQ2-4*aet5*M12*MCha2*MSQ2+2*M14*MC
     &  ha2*MSQ2+4*aet1*MCha2**2*MSQ2+4*aet5*MCha2**2*MSQ2+2*aetcha*MCh
     &  a2**2*MSQ2+2*aet1*M12*MSQ2**2+2*aet5*M12*MSQ2**2+M14*MSQ2**2-M1
     &  2*MCha2*MSQ2**2+aet5*MSQ2**3+a0t1*(-2*M12*M14+2*M12*MCha**4-MCh
     &  a**6+2*M14*MCha2+MCha**4*MCha2-2*M12*MCha2**2+4*M14*MSQ2+2*MCha
     &  **4*MSQ2+2*M12*MCha2*MSQ2-2*MCha2**2*MSQ2-2*M12*MSQ2**2-a0tcha*
     &  (2*M14-2*M12*(MCha2+MSQ2))+a0t5*(2*M12*(MCha2+MSQ2)-2*M12*(3*MC
     &  ha2+MSQ2)))+a0t5*(a0tcha*(2*M14-4*M12*MSQ2+2*M12*(MCha2+MSQ2))+
     &  2*M12*(-M14+(MCha2-MSQ2)*MSQ2+M12*(MCha2+2*MSQ2)))-2*M12*M14*I0
     &  Acha(6)+2*M12*MCha**4*I0Acha(6)-MCha**6*I0Acha(6)-2*M12**2*MCha
     &  2*I0Acha(6)+4*M14*MCha2*I0Acha(6)+MCha**4*MCha2*I0Acha(6)-2*M12
     &  *MCha2**2*I0Acha(6)+MSQ2**3*I0Acha(6)-2*M12**2*MSQ2*I0Acha(6)+6*
     &  M14*MSQ2*I0Acha(6)+2*MCha**4*MSQ2*I0Acha(6)+2*M12*MCha2*MSQ2*I0
     &  Acha(6)-2*MCha2**2*MSQ2*I0Acha(6)-2*M12*MSQ2**2*I0Acha(6)-MSQ2*
     &  *3*I0Acha(6)))/(16.*M12*(M14+MSQ2**2+MCha2**2-2*MCha2*(M12+MSQ2)
     &  -2*M12*MSQ2))
       aSCH=aSCH+(CF12F4S(1)*(abrx(1)*CF12F4SC(1)*CS3SSS13(1,1
     &  )+abrx(3)*CF12F4SC(2)*CS3SSS13(1,2))+CF12F4S(2)*(abrx(3)*CF12F4
     &  SC(1)*CS3SSS13(2,1)+abrx(2)*CF12F4SC(2)*CS3SSS13(2,2))+CTBS3(2)*(
     &  CF12F3S*CF12F3SC*abrx(6)+abrx(4)*CF12F4S(1)*CF12F4SC(1)+abrx(5)
     &  *CF12F4S(2)*CF12F4SC(2))*CTBS3C(2))

       SCH=aSCH*fac
#ifdef printMT4 
       Write(*,*), "====================================================="
       Write(*,*), "Charged Higgs self"
       Write(*,*), "finite      = ",SCH
#endif

      end subroutine CHSelfMT4

c**********************************************************************
      subroutine TadMT4(atad)
      implicit none
      integer i,j,gen3,gen4,h1,gen2
      double precision mha,mha2,fac,M22m42I,mahs,aetx,a0tx,atad(6),
     .   coef(10),I0ACha(2),I0A,M22m42,M24,M44,AEPT,K0T0
      double precision IT1,IT2
      double precision FD211MHM1(10,6),FD211CHM1(10,2)
      double complex abrx(6)
      common/I0NChaFD/IT1,IT2
#include "TLMT4.h"
      fac=1D0/16D0/Pi**4
      M22m42=(M22-M42)
      M22m42I= 1/M22m42
      M24=M22**2;  M44=M42**2

      do h1=ihiggs,ihiggse
       abrx(h1)=0
       mahs=Mah2(h1);a0tx=a0tnh(h1);aetx=aetnh(h1)
       call FD211C(M12,M12,Mahs,a0t1,a0t1,a0tx,aet1,aet1,aetx,Q2,coef,0)
       do gen3=1,10
        FD211MHM1(gen3,h1)=coef(gen3)
       enddo
      enddo      
      mahs=MHp2
      call FD211C(M12,0D0,MHp2,a0t1,0D0,a0thp,aet1,0D0,aethp,Q2,coef,0)
      do gen3=1,10
        FD211CHM1(gen3,2)=coef(gen3)
      enddo   
      mahs=0D0
      call FD211C(M12,0D0,Mahs,a0t1,0D0,0D0,aet1,0D0,0D0,Q2,coef,0)
      do gen3=1,10
        FD211CHM1(gen3,1)=coef(gen3)
      enddo   

       I0Acha(1)=I0A(0D0,M22,MCha2,Q2)
       I0Acha(2)=I0A(0D0,M42,MCha2,Q2)
       abrx(1:6)=0
      do h1=ineu,ineue
       abrx(1)=abrx(1)+(M22m42I*(CF11FSR(h1,2)*(CF11FSRC(h1,1)*(F
     &  D111Chi(2,h1,1)-FD111Chi(2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1
     &  ,2))+M1*CF11FSLC(h1,1)*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNe
     &  u(h1))+CF11FSL(h1,2)*(CF11FSLC(h1,1)*(FD111Chi(2,h1,1)-FD111Chi
     &  (2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1,2))+M1*CF11FSRC(h1,1)*(
     &  -FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNeu(h1))))/6.
       abrx(2)=abrx(2)+(M22m42I*(CF11FSR(h1,1)*(CF11FSRC(h1,2)*(F
     &  D111Chi(2,h1,1)-FD111Chi(2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1
     &  ,2))+M1*CF11FSLC(h1,2)*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNe
     &  u(h1))+CF11FSL(h1,1)*(CF11FSLC(h1,2)*(FD111Chi(2,h1,1)-FD111Chi
     &  (2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1,2))+M1*CF11FSRC(h1,2)*(
     &  -FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNeu(h1))))/6.
       abrx(3)=abrx(3)+(CF11FSR(h1,1)*(CF11FSRC(h1,1)*(FD211Chi(2
     &  ,h1,1)-FD211Chi(3,h1,1))-M1*CF11FSLC(h1,1)*FD211Chi(1,h1,1)*MNe
     &  u(h1))+CF11FSL(h1,1)*(CF11FSLC(h1,1)*(FD211Chi(2,h1,1)-FD211Chi
     &  (3,h1,1))-M1*CF11FSRC(h1,1)*FD211Chi(1,h1,1)*MNeu(h1)))/6.
       abrx(4)=abrx(4)+(CF11FSR(h1,2)*(CF11FSRC(h1,2)*(FD211Chi(2
     &  ,h1,2)-FD211Chi(3,h1,2))-M1*CF11FSLC(h1,2)*FD211Chi(1,h1,2)*MNe
     &  u(h1))+CF11FSL(h1,2)*(CF11FSLC(h1,2)*(FD211Chi(2,h1,2)-FD211Chi
     &  (3,h1,2))-M1*CF11FSRC(h1,2)*FD211Chi(1,h1,2)*MNeu(h1)))/6.
       abrx(5)=abrx(5)+(M1*CF11FSL(h1,1)*CF11FSLC(h1,1)*(FD211Chi
     &  M1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSR(h1,1)*CF11FSRC(h1,1)*
     &  (FD211ChiM1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSL(h1,2)*CF11FS
     &  LC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))+M1*CF11FSR(h1,
     &  2)*CF11FSRC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))-M12*C
     &  F11FSL(h1,1)*CF11FSRC(h1,1)*FD211ChiM1(1,h1,1)*MNeu(h1)-M12*CF1
     &  1FSL(h1,2)*CF11FSRC(h1,2)*FD211ChiM1(1,h1,2)*MNeu(h1)-CF11FSLC(
     &  h1,1)*CF11FSR(h1,1)*FD211ChiM1(4,h1,1)*MNeu(h1)-CF11FSLC(h1,2)*
     &  CF11FSR(h1,2)*FD211ChiM1(4,h1,2)*MNeu(h1))/6.
       abrx(6)=abrx(6)+(M1*CF11FSL(h1,1)*CF11FSLC(h1,1)*(FD211Chi
     &  M1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSR(h1,1)*CF11FSRC(h1,1)*
     &  (FD211ChiM1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSL(h1,2)*CF11FS
     &  LC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))+M1*CF11FSR(h1,
     &  2)*CF11FSRC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))-M12*C
     &  F11FSLC(h1,1)*CF11FSR(h1,1)*FD211ChiM1(1,h1,1)*MNeu(h1)-M12*CF1
     &  1FSLC(h1,2)*CF11FSR(h1,2)*FD211ChiM1(1,h1,2)*MNeu(h1)-CF11FSL(h
     &  1,1)*CF11FSRC(h1,1)*FD211ChiM1(4,h1,1)*MNeu(h1)-CF11FSL(h1,2)*C
     &  F11FSRC(h1,2)*FD211ChiM1(4,h1,2)*MNeu(h1))/6.
      enddo

      do i=ihiggs,ihiggse
      atad(i)=0
        do gen2=1,2
         do gen3=1,2
          do h1=ihiggs,ihiggse
       atad(i)=atad(i)-(3*CSSS(i,gen2,gen3)*CSSSS(h1,h1,gen3,gen2)*
     &  (A0TNH(h1)*aB0T1(gen2,gen3)+AETNH(h1)+aBEpT1(gen2,gen3)*Mah2(
     &  h1)))/32d0
          enddo
       atad(i)=atad(i)-(3*(AETHp+A0THp*aB0T1(gen2,gen3)+MHp2*aBEpT1
     &  (gen2,gen3))*CS3S3S13S13(2,2,gen3,gen2)*CSSS(i,gen2,gen3))/16d0
          do gen4=1,2
       atad(i)=atad(i)-(3*(CS13S13S13S13MT4(gen3,gen4,gen2,gen4)+
     &  3*CS13S13S13S13MT4(gen3,gen4,gen4,gen2))*CSSS(i,gen2,gen3)*(aAE
     &  TN(gen4)+aA0TN(gen4)*aB0T1(gen2,gen3)+aBEpT1(gen2,gen3)*MassSu2
     &  (gen4)))/16d0
          enddo
       atad(i)=atad(i)-(3*(aet5+a0t5*aB0T1(gen2,gen3)+MSQ2*aBEpT1(gen2,
     &  gen3))*CS14S13S14S13MT4(gen3,gen2)*CSSS(i,gen2,gen3))/16d0
         enddo
        enddo

       do h1=ihiggs,ihiggse
       atad(i)=atad(i)+CSSS(h1,1,1)*((3*CSSSS(i,h1,1,1)*FD111MH(h
     &  1,1,1))/16.+(3*CSSS(h1,2,1)*CSSS(i,1,2)*(FD111MH(h1,1,1)-FD111M
     &  H(h1,1,2)))/(16.*(M22-M42))+(3*CSSS(h1,1,2)*CSSS(i,2,1)*(FD111M
     &  H(h1,1,1)-FD111MH(h1,1,2)))/(16.*(M22-M42)))+CSSS(h1,1,2)*((3*C
     &  SSSS(i,h1,2,1)*FD111MH(h1,1,2))/16.+(3*CSSS(h1,2,2)*CSSS(i,2,1)
     &  *(FD111MH(h1,1,2)-FD111MH(h1,2,2)))/(16.*(M22-M42)))+(3*CSSS(h1
     &  ,2,2)*CSSSS(i,h1,2,2)*FD111MH(h1,2,2))/16.+(3*CSSS(h1,1,1)**2*C
     &  SSS(i,1,1)*FD211MH(h1,1,1))/16.+CSSS(h1,2,1)*((3*CSSSS(i,h1,1,2
     &  )*FD111MH(h1,1,2))/16.+(3*CSSS(h1,2,2)*CSSS(i,1,2)*(FD111MH(h1,
     &  1,2)-FD111MH(h1,2,2)))/(16.*(M22-M42))+CSSS(h1,1,2)*((3*CSSS(i,
     &  1,1)*FD211MH(h1,1,2))/16.+(3*CSSS(i,2,2)*FD211MH(h1,2,1))/16.))
     &  +(3*CSSS(h1,2,2)**2*CSSS(i,2,2)*FD211MH(h1,2,2))/16.-(3*M1*hy(i
     &  )*(M12*FD211MHM1(1,h1)*hy(h1)**2+hyc(h1)*(2*FD211MHM1(2,h1)*hy(
     &  h1)+FD211MHM1(4,h1)*hyc(h1))))/8.-(3*M1*(FD211MHM1(4,h1)*hy(h1)
     &  **2+hyc(h1)*(2*FD211MHM1(2,h1)*hy(h1)+M12*FD211MHM1(1,h1)*hyc(h
     &  1)))*hyc(i))/8d0
       enddo

       do h1=1,2
       atad(i)=atad(i)+(3*M22m42I*CS3S13S(h1,2)*CS3S13SC(h1,1)*CS
     &  SS(i,1,2)*(FD111CH(h1,1)-FD111CH(h1,2)))/16.+(3*M22m42I*CS3S13S
     &  (h1,1)*CS3S13SC(h1,2)*CSSS(i,2,1)*(FD111CH(h1,1)-FD111CH(h1,2))
     &  )/16.+(3*(CS1S3SSC(i,h1,1)*CS3S13S(h1,1)*FD111CH(h1,1)+CS1S3SS(
     &  i,h1,1)*CS3S13SC(h1,1)*FD111CH(h1,1)+(CS1S3SSC(i,h1,2)*CS3S13S(
     &  h1,2)+CS1S3SS(i,h1,2)*CS3S13SC(h1,2))*FD111CH(h1,2)))/16.+(3*CS
     &  3S13S(h1,1)*CS3S13SC(h1,1)*CSSS(i,1,1)*FD211CH(h1,1))/16.+(3*CS
     &  3S13S(h1,2)*CS3S13SC(h1,2)*CSSS(i,2,2)*FD211CH(h1,2))/16.-(3*M1
     &  *CTBS3(h1)*CTBS3C(h1)*FD211CHM1(2,h1)*hy(i))/8.-(3*M1*CTBS3(h1)
     &  *CTBS3C(h1)*FD211CHM1(2,h1)*hyc(i))/8d0
       enddo

       atad(i)=atad(i)+((3*CF12F3S*CF12F3SC*M1*(FD211Cha(2)-FD211
     &  Cha(4))*(hy(i)+hyc(i)))/8.+(3*CF12F4S(1)*CF12F4SC(1)*CSSS(i,1,1
     &  )*(-2*aet2*M22+M22**2+2*a0t2*(a0tcha+M22-MCha2)+2*aet2*MCha2+M2
     &  2*MCha2+2*M22*I0Acha(1)))/(16.*M22)+(3*CF12F4S(2)*CF12F4SC(2)*C
     &  SSS(i,2,2)*(-2*aet4*M42+M42**2+2*a0t4*(a0tcha+M42-MCha2)+2*aet4
     &  *MCha2+M42*MCha2+2*M42*I0Acha(2)))/(16.*M42)+(3*M22m42I*CF12F4S
     &  (1)*CF12F4SC(2)*CSSS(i,1,2)*(a0t2*a0tcha-a0t4*a0tcha-aet2*M22+a
     &  et4*M42+2*aet2*MCha2-2*aet4*MCha2+(M22-MCha2)*I0Acha(1)+(-M42+M
     &  Cha2)*I0Acha(2)))/16.+(3*M22m42I*CF12F4S(2)*CF12F4SC(1)*CSSS(i,
     &  2,1)*(a0t2*a0tcha-a0t4*a0tcha-aet2*M22+aet4*M42+2*aet2*MCha2-2*
     &  aet4*MCha2+(M22-MCha2)*I0Acha(1)+(-M42+MCha2)*I0Acha(2)))/16d0)

       atad(i)=atad(i)+(9*(abrx(3)*CSSS(i,1,1)+abrx(2)*CSSS(i,1,2)
     &  +abrx(1)*CSSS(i,2,1)+abrx(4)*CSSS(i,2,2)+abrx(5)*hy(i)+abrx(6)
     &  *hyc(i)))/4d0
       atad(i)=atad(i)*fac
      enddo

      end subroutine TadMT4
c
c**********************************************************************
c
      subroutine counterM_atat(CHself,dtad2MT4,dMH2MT4)
      implicit none
      double precision CHself,dMHp2MT4,dtad2MT4(6),dMH2MT4(6,6)
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision ImAkappaDR,ImAlambdaDR,ReAkappaDR,ReAlambdaDR 
      double precision phiIL,beta,dReAlambda
      double precision selfCHatat,dMHp2atat
      common /DRpara/ ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /MHpmatat/selfCHatat,dMHp2atat
#include "TLMT4.h"
      logical MHpmInput,CPcon,mQrun,onelswitch
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch


      phiIl=phis+phil+phiu
      beta=atan(TB)

      dMHp2MT4 = CHself
      if(MHpmInput) then
      dReAlambda=(-(SB*Cos(phiIL)*(-(CB*dMHp2MT4*SB*vv) 
     -         +SB**3*dtad2MT4(1)+CB**3*dtad2MT4(2))) + dtad2MT4(4)*
     -         Sin(phiIL))/(SB*vv*Abs(MUE)) +(dv4(2)*(8*CB*SB**2*vv2
     -         *Abs(MUE)**2*Cos(phiIL)*dv4(2) + vS**2*(2*SB*Cos(phiIL)*
     -         (4*SB**3*dtad1MT4(1,2) + 3*CB*dtad1MT4(2,2)+Cos(3*beta)*
     -         dtad1MT4(2,2)) - 8*dtad1MT4(4,2)*Sin(phiIL))))/
     -         (4.*SB*vS**2*vv**2*Abs(MUE))
      ReAlambdaDR=ReAlambdaDR+dReAlambda

      ImAlambdaDR=ImAlambdaDR+(dtad2MT4(4))/(SB*vv*Abs(MUE)*Cos(phiIL))
     -         - (dReAlambda)*Tan(phiIL)+
     -         (-2*dtad1MT4(4,2)*dv4(2))/(SB*vv**2*Abs(MUE)*cos(phiIL))

      else
      selfCHatat=dMHp2MT4
      dMHp2MT4=(SB**2*dtad2MT4(1))/(CB*vv) + (CB**2*dtad2MT4(2))/(SB*vv) - 
     -  (dtad2MT4(4)*Tan(phiIL))/(CB*SB**2*vv)+
     -  (dv4(2)*(-2*vv2*Abs(MUE)**2*dv4(2) + 
     -  vS**2*((dtad1MT4(4,2)*Tan(phiIL))/(CB*SB**2) + 
     -  SB*TB*(-2*dtad1MT4(1,2) + dtad1MT4(4,2)*Tan(phiIL)) + 
     -  CB*((-2*dtad1MT4(2,2))/TB + 2*dtad1MT4(4,2)*Tan(phiIL) + 
     -  (dtad1MT4(4,2)*Tan(phiIL))/TB**2))))/(vS**2*vv**2)
c Nhung corrected the missing piece here 6/2021
      dMHp2atat=dMHp2MT4
c end Nhung corrected the missing piece here 6/2021
      ImAlambdaDR=ImAlambdaDR+(dtad2MT4(4))/(SB*vv*Abs(MUE)*Cos(phiIL))+
     -         (-2*dtad1MT4(4,2)*dv4(2))/(SB*vv**2*Abs(MUE)*cos(phiIL))


      endif

      if(abskappa.gt.1D-14)then
      ImAkappaDR=ImAkappaDR+(Sqrt(2d0)*(CB*vv*dtad2MT4(4) - 
     -      vS*dtad2MT4(6))/cos(phik + 3*phis))/(abskappa*vS**3)+
     -      (2*CB*dv4(2)*(Sqrt(2d0)*dtad1MT4(4,2) + 3*abskappa*SB*
     -      vS*Abs(MUE)*dv4(2)*Sin(phik - phil + 2*phis - phiu)))/
     -      (abskappa*vS**3*cos(phik + 3*phis))
      else
      ImAkappaDR=ImAkappaDR
      endif

c Higgs mass CT matrix

       dMH2MT4(1,1)=SB2*dMHp2MT4+((CB*CB2+2*CB*SB2)*dtad2MT4(1))
     &  /vv-(CB2*SB*dtad2MT4(2))/vv
       dMH2MT4(1,2)=-(CB*SB*dMHp2MT4)+(SB**3*dtad2MT4(1))/vv+
     &  (CB*CB2*dtad2MT4(2))/vv
       dMH2MT4(1,5)=dtad2MT4(4)/(SB*vv)
       dMH2MT4(2,2)=CB2*dMHp2MT4-(CB*SB2*dtad2MT4(1))/vv+((2*
     &  CB2*SB+SB**3)*dtad2MT4(2))/vv
       dMH2MT4(2,4)=dtad2MT4(4)/(SB*vv)
       dMH2MT4(4,4)=SB2*dMHp2MT4+((CB*CB2+2*CB*SB2)*dtad2MT4(1))
     &  /vv-(CB2*SB*dtad2MT4(2))/vv
       dMH2MT4(4,5)=CB*SB*dMHp2MT4-(SB**3*dtad2MT4(1))/vv-(CB*
     &  CB2*dtad2MT4(2))/vv
       dMH2MT4(5,5)=CB2*dMHp2MT4-(CB*SB2*dtad2MT4(1))/vv+((2*
     &  CB2*SB+SB**3)*dtad2MT4(2))/vv

c
c============= 

       dMH2MT4(1,1)=dMH2MT4(1,1)+((2*CB**3*dTanB4*SB*vv2*
     &  dMHp1MT4(3))/vv**2+dtad1MT4(2,3)*(-((CB**5*dTanB4)/vv)+(CB**2*
     &  SB*dv4(1))/vv**2)+dtad1MT4(1,3)*((CB**4*dTanB4*SB)/vv-((CB**3+
     &  2*CB*SB**2)*dv4(1))/vv**2)-((CB**3+2*CB*SB**2)*dtad1MT4(1,2)*
     &  dv4(2))/vv**2+(CB**2*SB*dtad1MT4(2,2)*dv4(2))/vv**2-((CB**3+2*
     &  CB*SB**2)*dtad1MT4(1,1)*dv4(3))/vv**2+(CB**2*SB*dtad1MT4(2,1)*
     &  dv4(3))/vv**2)
       dMH2MT4(1,2)=dMH2MT4(1,2)+((CB**2*dTanB4*(-CB**2+SB**2)*vv2*
     &  dMHp1MT4(3))/vv**2+dtad1MT4(2,3)*(-((CB**4*dTanB4*SB)/vv)-(CB**
     &  3*dv4(1))/vv**2)+dtad1MT4(1,3)*((CB**3*dTanB4*SB**2)/vv-(SB**3*
     &  dv4(1))/vv**2)-(SB**3*dtad1MT4(1,2)*dv4(2))/vv**2-(CB**3*
     &  dtad1MT4(2,2)*dv4(2))/vv**2-(SB**3*dtad1MT4(1,1)*dv4(3))/vv**2-
     &  (CB**3*dtad1MT4(2,1)*dv4(3))/vv**2)
       dMH2MT4(1,5)=dMH2MT4(1,5)+(dtad1MT4(4,3)*(-((CB**3*dTanB4)
     &  /(SB**2*vv))-dv4(1)/(SB*vv**2))-(dtad1MT4(4,2)*dv4(2))/(SB*vv**
     &  2)-(dtad1MT4(4,1)*dv4(3))/(SB*vv**2))
       dMH2MT4(2,2)=dMH2MT4(2,2)+((-2*CB**3*dTanB4*SB*vv2*
     &  dMHp1MT4(3))/vv**2+dtad1MT4(1,3)*((CB**2*dTanB4*SB**3)/vv+(CB*
     &  SB**2*dv4(1))/vv**2)+dtad1MT4(2,3)*(-((CB**3*dTanB4*SB**2)/vv)-
     &  ((2*CB**2*SB+SB**3)*dv4(1))/vv**2)+(CB*SB**2*dtad1MT4(1,2)*
     &  dv4(2))/vv**2-((2*CB**2*SB+SB**3)*dtad1MT4(2,2)*dv4(2))/vv**2+
     &  (CB*SB**2*dtad1MT4(1,1)*dv4(3))/vv**2-((2*CB**2*SB+SB**3)*
     &  dtad1MT4(2,1)*dv4(3))/vv**2)
       dMH2MT4(2,4)=dMH2MT4(2,4)+(dtad1MT4(4,3)*(-((CB**3*dTanB4)
     &  /(SB**2*vv))-dv4(1)/(SB*vv**2))-(dtad1MT4(4,2)*dv4(2))/(SB*vv**
     &  2)-(dtad1MT4(4,1)*dv4(3))/(SB*vv**2))

       dMH2MT4(4,4)=dMH2MT4(4,4)+ (CB*(-(CB**2*dtad1MT4(1,2)*dv4(2)) -
     &      2*SB**2*dtad1MT4(1,2)*dv4(2) -(CB**2 + 2*SB**2)*dtad1MT4(1,2) 
     &      *dv4(2) + 2*CB*SB*dtad1MT4(2,2)*dv4(2)))/(2.*vv**2)
       dMH2MT4(4,5)=dMH2MT4(4,5)+(2*SB**3*dtad1MT4(1,2)*dv4(2) +
     &      2*CB**3*dtad1MT4(2,2)*dv4(2))/(2.*vv**2)
       dMH2MT4(5,5)=dMH2MT4(5,5)+(SB*(2*CB*SB*dtad1MT4(1,2)*dv4(2) 
     &      - 4*CB**2*dtad1MT4(2,2)*dv4(2) - 
     &      2*SB**2*dtad1MT4(2,2)*dv4(2)))/(2.*vv**2)

      end subroutine counterM_atat
c
c**********************************************************************
c
      subroutine UnselfHH_atat(selfIN)
      implicit none
      integer i,j,l,k,gen2,Gen3,Gen4,Gen5,Gen6,h1
      double precision A0T,AEpT,B0T,BEpT,C0T,CEpT,K0T0N,M22m42,
     .       M22m42s,M22m42I,M22m42sI,mahs,a0th,aeth,M24,M44,
     .       FDMH(4,2,2),abrx(32),coef(5),coefC(10),coeff1(10),
     .       self(6,6),selfIN(6,6),fac
#include "TLMT4.h"
      fac=1D0/pi**4/16D0
      LogMSQ2=Log(MSQ2/Q2)
      logcha=Log(MCha2/Q2)
      A0T1=A0T(M12,Q2);   AET1=AEpT(M12,Q2)
      A0T2=A0T(M22,Q2);   AET2=AEpT(M22,Q2)
      A0T4=A0T(M42,Q2);   AET4=AEpT(M42,Q2)
      aA0TN(1)=A0T2;  aAETN(1)=AET2
      aA0TN(2)=A0T4;  aAETN(2)=AET4
      M24=M22**2;M44=M42**2
      M22m42=M22-M42; M22m42s=M22m42**2;M22m42I=1D0/M22m42
      M22m42sI=1D0/M22m42s


      do i=1,2
       do j=1,2
        aB0T1(j,i)= B0T(MassSu2(i),MassSu2(j),Q2)
        aBEpT1(j,i)=BEpT(MassSu2(i),MassSu2(j),Q2)
       enddo
      enddo

      do i=1,2
       do j=1,2
        Do k=1,2
       aC0T(i,j,k)=C0T(MassSu2(i),MassSu2(j),MassSu2(k),Q2)
       aCEpT(i,j,k)=CEpT(MassSu2(i),MassSu2(j),MassSu2(k),Q2)
        enddo
       enddo
      enddo

      do j=1,2
        aC0TH2(j)  = C0T(MSQ2,MSQ2,MassSu2(j),Q2)
        aCEpTH2(j) = CEpT(MSQ2,MSQ2,MassSu2(j),Q2)
        aB0TH1(j)  = B0T(MSQ2, MassSu2(j),Q2)
        aBEpTH1(j) = BEpT(MSQ2, MassSu2(j),Q2)
        do i=1,2
        aC0TH1(i,j)  = C0T(MSQ2,MassSu2(i),MassSu2(j),Q2)
        aCEpTH1(i,j) = CEpT(MSQ2,MassSu2(i),MassSu2(j),Q2)
        enddo
      enddo
      A0T5= A0T(MSQ2,Q2);      AET5= AEpT(MSQ2,Q2)
      aB0T5= B0T(MSQ2,MSQ2,Q2); aBEpT5=BEpT(MSQ2,MSQ2,Q2)
      aB0TN1=B0T(M12, M12, Q2); aBEpTN1=BEpT(M12,M12,Q2)


      do i=ihiggs,ihiggse
        A0TNH(i) =A0T(Mah2(i),Q2)
        AETNH(i)=AEpT(Mah2(i),Q2)
      enddo   

      A0THp= A0T(MHp2N,Q2);      AETHp= AEpT(MHp2N,Q2)
      A0THpN(2)=A0THp; AETHpN(2)=AETHp
      A0THpN(1)=0D0; AETHpN(1)=0D0


      do i=ihiggs,ihiggse
        Mahs=Mah2(i); a0th=A0TNH(i); aeth=AETNH(i)
        call NHiggs_TR(Mahs,M22,M42,aeth,aet2,aet4,a0th,a0t2,a0t4,
     .    Q2,FDMH)
       do gen4=1,2
        do gen5=1,2
       FD111MH(i,gen4,gen5)=FDMH(1,gen4,gen5)
       FD211MH(i,gen4,gen5)=FDMH(2,gen4,gen5)
       FD311MH(i,gen4,gen5)=FDMH(3,gen4,gen5)
       FD221MH(i,gen4,gen5)=FDMH(4,gen4,gen5)
        enddo
       enddo
      enddo
       Mahs=MHp2N; a0th=A0THp; aeth=AETHp
        call NHiggs_TR(Mahs,M22,MSQ2,aeth,aet2,aet5,a0th,a0t2,a0t5,
     .    Q2,FDMH)
        FD111CH(2,1)=FDMH(1,1,2)
        FD211CH(2,1)=FDMH(2,1,2)
        FD311CH(2,1)=FDMH(3,1,2)
        call NHiggs_TR(Mahs,M42,MSQ2,aeth,aet4,aet5,a0th,a0t4,a0t5,
     .    Q2,FDMH)
        FD111CH(2,2)=FDMH(1,1,2)
        FD211CH(2,2)=FDMH(2,1,2)
        FD311CH(2,2)=FDMH(3,1,2)
        FD111CH(1,1)=K0T0N(0D0,M22,MSQ2,0D0,aet2,aet5,Q2)
        FD211CH(1,1)=(a0t5*M22 +aet5*M22+M24+a0t2*(a0t5+2*M22-MSQ2)+ 
     -    aet2*MSQ2 + M22*MSQ2 + M22*FD111CH(1,1))/
     -  (M24 - M22*MSQ2)
        FD311CH(1,1)=-((M22 - MSQ2)*(M22*(a0t5 + M22 - MSQ2) + 
     -       a0t2*(-2*M22 + MSQ2)))/
     -  (2.*M24*(M22 - MSQ2)**2)

        FD111CH(1,2)=K0T0N(0D0,M42,MSQ2,0D0,aet4,aet5,Q2)
        FD211CH(1,2)=(a0t5*M42 +aet5*M42+M44+a0t4*(a0t5+2*M42-MSQ2)+ 
     -    aet4*MSQ2 + M42*MSQ2 + M42*FD111CH(1,2))/
     -  (M44 - M42*MSQ2)
        FD311CH(1,2)=-((M42 - MSQ2)*(M42*(a0t5 + M42 - MSQ2) + 
     -       a0t4*(-2*M42 + MSQ2)))/
     -  (2.*M44*(M42 - MSQ2)**2)

        do h1=ihiggs,ihiggse
         if (Mah2(h1).gt.0)then
         logmha(h1)=Log(Mah2(h1)/Q2)
         else
         logmha(h1)=0D0
         endif
        enddo

        logmhpm(1)=0
	logmhpm(2)=Log(MHp2N/Q2)
      do i=ihiggs,ihiggse
       Mahs=Mah2(i); a0th=A0TNH(i); aeth=AETNH(i)
       call NHiggs_TB(M12,Mahs,aet1,aeth,a0t1,a0th,Q2,coef)
       do gen4=1,5
        FDTH(i,gen4)=coef(gen4)
       enddo 
      enddo

      call CHiggs_TB(M12,MHp2N,aet1,AETHp,a0t1,A0THp,Q2,coef)
      FDTCH(2,1)=coef(1);FDTCH(2,2)=coef(2)
      FDTCH(1,1)=coef(3);FDTCH(1,2)=coef(4)


      do i=ineu,ineue
      mahs=MNeu2(i)
      A0TNeu(i)= A0T(mahs,Q2);  AETNeu(i)= AEpT(mahs,Q2)
      a0th=A0TNeu(i);  aeth=AETNeu(i)
      call FD111C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            coeff1,0)
      FD111Chi(1:10,i,1)  =coeff1(1:10)
      call FD211C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            coeff1,0)
      FD211Chi(1:10,i,1)  =coeff1(1:10)
      call FD211C(M12,M22,mahs,a0t1,a0t2,a0th,aet1,aet2,aeth,Q2,
     .            coeff1,0)
      FD211ChiM1(1:10,i,1)=coeff1(1:10)
      call FD221C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            coeff1,0)
      FD221Chi(1:10,i,1)=coeff1(1:10)
      call FD311C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            coeff1,0)
      FD311Chi(1:10,i,1)=coeff1(1:10)
      call FD311C(M12,M22,mahs,a0t1,a0t2,a0th,aet1,aet2,aeth,Q2,
     .            coeff1,0)
      FD311ChiM1(1:10,i,1)=coeff1(1:10)

      call FD111C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            coeff1,0)
      FD111Chi(1:10,i,2)  =coeff1(1:10)
      call FD211C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            coeff1,0)
      FD211Chi(1:10,i,2)  =coeff1(1:10)
      call FD211C(M12,M42,mahs,a0t1,a0t4,a0th,aet1,aet4,aeth,Q2,
     .            coeff1,0)
      FD211ChiM1(1:10,i,2)=coeff1(1:10) 
      call FD221C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            coeff1,0)
      FD221Chi(1:10,i,2)  =coeff1(1:10)
      call FD311C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            coeff1,0)
      FD311Chi(1:10,i,2)  =coeff1(1:10) 
      call FD311C(M12,M42,mahs,a0t1,a0t4,a0th,aet1,aet4,aeth,Q2,
     .            coeff1,0)
      FD311ChiM1(1:10,i,2)=coeff1(1:10)

      call FD221C(Mahs,M12,M22,a0th,a0t1,a0t2,aeth,aet1,aet2,Q2,
     .            coeff1,0)
       FD221ChiM1MS(1:10,i,1)=coeff1(1:10)
      call FD221C(Mahs,M12,M42,a0th,a0t1,a0t4,aeth,aet1,aet4,Q2,
     .            coeff1,0)
       FD221ChiM1MS(1:10,i,2)=coeff1(1:10)
      call FD211C(Mahs,M22,M12,a0th,a0t2,a0t1,aeth,aet2,aet1,Q2,
     .            coeff1,0)
       FD211ChiMSM1(1:10,i,1)=coeff1(1:10)
      call FD211C(Mahs,M42,M12,Q2,a0th,a0t4,a0t1,aeth,aet4,aet1,
     .            coeff1,0)
       FD211ChiMSM1(1:10,i,2)=coeff1(1:10)
      call FD221C(Mahs,M22,M12,a0th,a0t2,a0t1,aeth,aet2,aet1,Q2,
     .            coeff1,0)
       FD221ChiMSM1(1:10,i,1)=coeff1(1:10)
      call FD221C(Mahs,M42,M12,a0th,a0t4,a0t1,aeth,aet4,aet1,Q2,
     .            coeff1,0)
       FD221ChiMSM1(1:10,i,2)=coeff1(1:10)
      call FD311C(Mahs,M12,M22,a0th,a0t1,a0t2,aeth,aet1,aet2,Q2,
     .            coeff1,0)
       FD311ChiM1MS(1:10,i,1)=coeff1(1:10)
      call FD311C(Mahs,M12,M42,a0th,a0t1,a0t4,aeth,aet1,aet4,Q2,
     .            coeff1,0)
       FD311ChiM1MS(1:10,i,2)=coeff1(1:10)
      call FD111C(MSQ2,M12,Mahs,a0t5,a0t1,a0th,aet5,aet1,aeth,Q2,
     .            coeff1,0)
       FD111Chi5M1(1:10,i)= coeff1(1:10)
      enddo

      a0tcha=A0T(MCha2,Q2); aetcha=AEpT(MCha2,Q2)
      call FD311C(M12,MSQ2,MCha2,a0t1,a0t5,a0tcha,aet1,aet5,aetcha,Q2,
     .            coeff1,0)
      FD311Cha(1:10)=coeff1(1:10)
      call FD311C(MCha2,M12,MSQ2,a0tcha,a0t1,a0t5,aetcha,aet1,aet5,Q2,
     .            coeff1,0)
      FD311ChaM1(1:10)=coeff1(1:10)
      call FD221C(MCha2,M12,MSQ2,a0tcha,a0t1,a0t5,aetcha,aet1,aet5,Q2,
     .            coeff1,0)
      FD221Cha(1:10)=coeff1(1:10)
      call FD211C(M12,MSQ2,MCha2,a0t1,a0t5,a0tcha,aet1,aet5,aetcha,Q2,
     .            coeff1,0)
      FD211Cha(1:10) =coeff1(1:10)
      call FD211C(MSQ2,M12,MCha2,a0t5,a0t1,a0tcha,aet5,aet1,aetcha,Q2,
     .            coeff1,0)
      FD211Cha5(1:10)=coeff1(1:10)
      call ChaFD(Mcha2,M22,M42,a0tcha,aetcha,a0t2,aet2,a0t4,aet4,Q2,
     .           coefc)
      do i=1,7
      FDCha(i)=coefc(i)
      enddo


      FDNeu(1:22)=0

      do h1=ineu,ineue
       abrx(1)=(3*M22m42I*(FD111Chi(2,h1,1)-FD111Chi(2,h1,2)-FD11
     &  1Chi(3,h1,1)+FD111Chi(3,h1,2)))/8.
       abrx(2)=(3*(FD211Chi(2,h1,1)-FD211Chi(3,h1,1)))/8.
       abrx(3)=(3*M22m42sI*(-FD111Chi(2,h1,1)+FD111Chi(2,h1,2)+FD
     &  111Chi(3,h1,1)-FD111Chi(3,h1,2)+M22m42*FD211Chi(2,h1,1)-M22m42*
     &  FD211Chi(3,h1,1)))/8.
       abrx(4)=(3*(FD211Chi(2,h1,2)-FD211Chi(3,h1,2)))/8.
       abrx(5)=(3*M22m42sI*(FD111Chi(2,h1,1)-FD111Chi(2,h1,2)-FD1
     &  11Chi(3,h1,1)+FD111Chi(3,h1,2)-M22m42*FD211Chi(2,h1,2)+M22m42*F
     &  D211Chi(3,h1,2)))/8.
       abrx(6)=(3*M1*M22m42I*(FD211ChiM1(2,h1,1)-FD211ChiM1(2,h1,
     &  2)-FD211ChiM1(4,h1,1)+FD211ChiM1(4,h1,2)))/8.
       abrx(7)=(3*M1*(FD221Chi(2,h1,1)-FD221Chi(5,h1,1)))/8.
       abrx(8)=(3*M1*(FD221Chi(2,h1,2)-FD221Chi(5,h1,2)))/8.
       abrx(9)=(3*(FD311Chi(2,h1,1)-FD311Chi(5,h1,1)))/4.
       abrx(10)=(3*(FD311Chi(2,h1,2)-FD311Chi(5,h1,2)))/4.
       abrx(11)=(3*M12*(FD311ChiM1(2,h1,1)-FD311ChiM1(3,h1,1)))/4.
       abrx(12)=(3*M12*(FD311ChiM1(2,h1,2)-FD311ChiM1(3,h1,2)))/4.
       abrx(13)=(3*(M12*FD311ChiM1(2,h1,1)-M12*FD311ChiM1(3,h1,1)+
     &  FD311ChiM1(4,h1,1)-FD311ChiM1(7,h1,1)))/8.
       abrx(14)=(3*(M12*FD311ChiM1(2,h1,2)-M12*FD311ChiM1(3,h1,2)+
     &  FD311ChiM1(4,h1,2)-FD311ChiM1(7,h1,2)))/8.
       abrx(15)=(-3*M1*FD211Chi(1,h1,1)*MNeu(h1))/8.
       abrx(16)=(-3*M1*FD211Chi(1,h1,2)*MNeu(h1))/8.
       abrx(17)=(-3*M12*FD221Chi(1,h1,1)*MNeu(h1))/8.
       abrx(18)=(-3*M12*FD221Chi(1,h1,2)*MNeu(h1))/8.
       abrx(19)=(-3*FD221Chi(5,h1,1)*MNeu(h1))/8.
       abrx(20)=(-3*FD221Chi(5,h1,2)*MNeu(h1))/8.
       abrx(21)=(-3*M1*FD311Chi(1,h1,1)*MNeu(h1))/4.
       abrx(22)=(-3*M1*FD311Chi(1,h1,2)*MNeu(h1))/4.
       abrx(23)=(-3*M1**3*FD311ChiM1(1,h1,1)*MNeu(h1))/4.
       abrx(24)=(-3*M1**3*FD311ChiM1(1,h1,2)*MNeu(h1))/4.
       abrx(25)=(-3*M1*FD311ChiM1(3,h1,1)*MNeu(h1))/4.
       abrx(26)=(-3*M1*FD311ChiM1(3,h1,2)*MNeu(h1))/4.
       abrx(27)=(3*M1*M22m42I*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2))
     &  *MNeu(h1))/8.
       abrx(28)=(-3*M1*M22m42sI*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2
     &  )+M22m42*FD211Chi(1,h1,1))*MNeu(h1))/8.
       abrx(29)=(3*M1*M22m42sI*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2)
     &  +M22m42*FD211Chi(1,h1,2))*MNeu(h1))/8.
       abrx(30)=(3*M12*M22m42I*(-FD211ChiM1(1,h1,1)+FD211ChiM1(1,h
     &  1,2))*MNeu(h1))/8.
       abrx(31)=(3*M22m42I*(-FD211ChiM1(4,h1,1)+FD211ChiM1(4,h1,2)
     &  )*MNeu(h1))/8.

       FDNeu(1)=FDNeu(1)+(CF11FSR(h1,1)*(abrx(15)*CF11FSLC(h1,1)+ab
     &  rx(2)*CF11FSRC(h1,1))+CF11FSL(h1,1)*(abrx(2)*CF11FSLC(h1,1)+abr
     &  x(15)*CF11FSRC(h1,1)))
       FDNeu(2)=FDNeu(2)+(CF11FSR(h1,1)*(abrx(19)*CF11FSLC(h1,1)+ab
     &  rx(7)*CF11FSRC(h1,1))+CF11FSL(h1,1)*(abrx(7)*CF11FSLC(h1,1)+abr
     &  x(17)*CF11FSRC(h1,1)))
       FDNeu(3)=FDNeu(3)+(CF11FSR(h1,1)*(abrx(17)*CF11FSLC(h1,1)+ab
     &  rx(7)*CF11FSRC(h1,1))+CF11FSL(h1,1)*(abrx(7)*CF11FSLC(h1,1)+abr
     &  x(19)*CF11FSRC(h1,1)))
       FDNeu(4)=FDNeu(4)+(CF11FSR(h1,1)*(abrx(21)*CF11FSLC(h1,1)+ab
     &  rx(9)*CF11FSRC(h1,1))+CF11FSL(h1,1)*(abrx(9)*CF11FSLC(h1,1)+abr
     &  x(21)*CF11FSRC(h1,1)))
       FDNeu(5)=FDNeu(5)+(CF11FSR(h1,2)*(abrx(27)*CF11FSLC(h1,1)+ab
     &  rx(1)*CF11FSRC(h1,1))+CF11FSL(h1,2)*(abrx(1)*CF11FSLC(h1,1)+abr
     &  x(27)*CF11FSRC(h1,1)))
       FDNeu(6)=FDNeu(6)+(CF11FSR(h1,2)*(abrx(28)*CF11FSLC(h1,1)+ab
     &  rx(3)*CF11FSRC(h1,1))+CF11FSL(h1,2)*(abrx(3)*CF11FSLC(h1,1)+abr
     &  x(28)*CF11FSRC(h1,1)))
       FDNeu(7)=FDNeu(7)+(CF11FSR(h1,2)*(abrx(29)*CF11FSLC(h1,1)+ab
     &  rx(5)*CF11FSRC(h1,1))+CF11FSL(h1,2)*(abrx(5)*CF11FSLC(h1,1)+abr
     &  x(29)*CF11FSRC(h1,1)))
       FDNeu(8)=FDNeu(8)+(CF11FSR(h1,2)*(abrx(31)*CF11FSLC(h1,1)+ab
     &  rx(6)*CF11FSRC(h1,1))+CF11FSL(h1,2)*(abrx(6)*CF11FSLC(h1,1)+abr
     &  x(30)*CF11FSRC(h1,1)))
       FDNeu(9)=FDNeu(9)+(CF11FSR(h1,2)*(abrx(30)*CF11FSLC(h1,1)+ab
     &  rx(6)*CF11FSRC(h1,1))+CF11FSL(h1,2)*(abrx(6)*CF11FSLC(h1,1)+abr
     &  x(31)*CF11FSRC(h1,1)))
       FDNeu(10)=FDNeu(10)+(CF11FSR(h1,2)*(abrx(16)*CF11FSLC(h1,2)+a
     &  brx(4)*CF11FSRC(h1,2))+CF11FSL(h1,2)*(abrx(4)*CF11FSLC(h1,2)+ab
     &  rx(16)*CF11FSRC(h1,2)))
       FDNeu(11)=FDNeu(11)+(CF11FSR(h1,2)*(abrx(20)*CF11FSLC(h1,2)+a
     &  brx(8)*CF11FSRC(h1,2))+CF11FSL(h1,2)*(abrx(8)*CF11FSLC(h1,2)+ab
     &  rx(18)*CF11FSRC(h1,2)))
       FDNeu(12)=FDNeu(12)+(CF11FSR(h1,2)*(abrx(18)*CF11FSLC(h1,2)+a
     &  brx(8)*CF11FSRC(h1,2))+CF11FSL(h1,2)*(abrx(8)*CF11FSLC(h1,2)+ab
     &  rx(20)*CF11FSRC(h1,2)))
       FDNeu(13)=FDNeu(13)+(CF11FSR(h1,2)*(abrx(22)*CF11FSLC(h1,2)+a
     &  brx(10)*CF11FSRC(h1,2))+CF11FSL(h1,2)*(abrx(10)*CF11FSLC(h1,2)+
     &  abrx(22)*CF11FSRC(h1,2)))
       FDNeu(14)=FDNeu(14)+(abrx(25)*CF11FSLC(h1,1)*CF11FSR(h1,1)+ab
     &  rx(26)*CF11FSLC(h1,2)*CF11FSR(h1,2)+abrx(11)*CF11FSR(h1,1)*CF11
     &  FSRC(h1,1)+CF11FSL(h1,1)*(abrx(11)*CF11FSLC(h1,1)+abrx(23)*CF11
     &  FSRC(h1,1))+abrx(12)*CF11FSR(h1,2)*CF11FSRC(h1,2)+CF11FSL(h1,2)
     &  *(abrx(12)*CF11FSLC(h1,2)+abrx(24)*CF11FSRC(h1,2)))
       FDNeu(15)=FDNeu(15)+(abrx(23)*CF11FSLC(h1,1)*CF11FSR(h1,1)+ab
     &  rx(24)*CF11FSLC(h1,2)*CF11FSR(h1,2)+abrx(11)*CF11FSR(h1,1)*CF11
     &  FSRC(h1,1)+CF11FSL(h1,1)*(abrx(11)*CF11FSLC(h1,1)+abrx(25)*CF11
     &  FSRC(h1,1))+abrx(12)*CF11FSR(h1,2)*CF11FSRC(h1,2)+CF11FSL(h1,2)
     &  *(abrx(12)*CF11FSLC(h1,2)+abrx(26)*CF11FSRC(h1,2)))
       FDNeu(16)=FDNeu(16)+(abrx(25)*CF11FSLC(h1,1)*CF11FSR(h1,1)+ab
     &  rx(26)*CF11FSLC(h1,2)*CF11FSR(h1,2)+abrx(13)*CF11FSR(h1,1)*CF11
     &  FSRC(h1,1)+CF11FSL(h1,1)*(abrx(13)*CF11FSLC(h1,1)+abrx(25)*CF11
     &  FSRC(h1,1))+abrx(14)*CF11FSR(h1,2)*CF11FSRC(h1,2)+CF11FSL(h1,2)
     &  *(abrx(14)*CF11FSLC(h1,2)+abrx(26)*CF11FSRC(h1,2)))
       FDNeu(17)=FDNeu(17)+(CF11FSR(h1,1)*(abrx(27)*CF11FSLC(h1,2)+a
     &  brx(1)*CF11FSRC(h1,2))+CF11FSL(h1,1)*(abrx(1)*CF11FSLC(h1,2)+ab
     &  rx(27)*CF11FSRC(h1,2)))
       FDNeu(18)=FDNeu(18)+(CF11FSR(h1,1)*(abrx(28)*CF11FSLC(h1,2)+a
     &  brx(3)*CF11FSRC(h1,2))+CF11FSL(h1,1)*(abrx(3)*CF11FSLC(h1,2)+ab
     &  rx(28)*CF11FSRC(h1,2)))
       FDNeu(19)=FDNeu(19)+(CF11FSR(h1,1)*(abrx(29)*CF11FSLC(h1,2)+a
     &  brx(5)*CF11FSRC(h1,2))+CF11FSL(h1,1)*(abrx(5)*CF11FSLC(h1,2)+ab
     &  rx(29)*CF11FSRC(h1,2)))
       FDNeu(20)=FDNeu(20)+(abrx(28)*CF11FSLC(h1,1)*CF11FSR(h1,1)+ab
     &  rx(29)*CF11FSLC(h1,2)*CF11FSR(h1,2)+abrx(3)*CF11FSR(h1,1)*CF11F
     &  SRC(h1,1)+CF11FSL(h1,1)*(abrx(3)*CF11FSLC(h1,1)+abrx(28)*CF11FS
     &  RC(h1,1))+abrx(5)*CF11FSR(h1,2)*CF11FSRC(h1,2)+CF11FSL(h1,2)*(a
     &  brx(5)*CF11FSLC(h1,2)+abrx(29)*CF11FSRC(h1,2)))
       FDNeu(21)=FDNeu(21)+(CF11FSR(h1,1)*(abrx(31)*CF11FSLC(h1,2)+a
     &  brx(6)*CF11FSRC(h1,2))+CF11FSL(h1,1)*(abrx(6)*CF11FSLC(h1,2)+ab
     &  rx(30)*CF11FSRC(h1,2)))
       FDNeu(22)=FDNeu(22)+(CF11FSR(h1,1)*(abrx(30)*CF11FSLC(h1,2)+a
     &  brx(6)*CF11FSRC(h1,2))+CF11FSL(h1,1)*(abrx(6)*CF11FSLC(h1,2)+ab
     &  rx(31)*CF11FSRC(h1,2)))
      enddo
c end calculate loop integrals
c calculate Higgs selfenergies from genuine two-loop diagrams
      do i=ihiggs,ihiggse
      do j=i,ihiggse
       self(i,j)=0D0
        do gen3=1,2
         do gen4=1,2
          do gen5=1,2
           do gen6=1,2
          self(i,j)=self(i,j)+(3*(CS13S13S13S13MT4(gen4,gen6,gen5,gen6)+ 
     -    3*CS13S13S13S13MT4(gen4,gen6,gen6,gen5))*
     -    (-(CSSS(i,gen3,gen4)*CSSS(j,gen5,gen3)*
     -    (aA0TN(gen6)*aC0T(gen3,gen4,gen5) + 
     -    aCEpT(gen3,gen4,gen5)*MassSu2(gen6))) - 
     -    CSSS(i,gen5,gen3)*CSSS(j,gen3,gen4)*
     -    (aA0TN(gen6)*aC0T(gen3,gen5,gen4) + 
     -    aCEpT(gen3,gen5,gen4)*MassSu2(gen6))))/16.

          self(i,j)=self(i,j)+(-3*(aB0T1(gen3,gen4)*aB0T1(gen5,gen6) 
     -    + aBEpT1(gen3,gen4) +  aBEpT1(gen5,gen6))*
     -    (CS13S13S13S13MT4(gen4,gen6,gen3,gen5) + 
     -    3*CS13S13S13S13MT4(gen4,gen6,gen5,gen3))*CSSS(i,gen3,gen4)*
     -    CSSS(j,gen5,gen6))/16.
           enddo
          self(i,j)=self(i,j)+(-3*(a0t5*aC0T(gen3,gen4,gen5) + 
     -    MSQ2*aCEpT(gen3,gen4,gen5))*(CS14S13S14S13MT4(gen5,gen4)*
     -    CSSS(i,gen4,gen3)*CSSS(j,gen3,gen5) + 
     -    CS14S13S14S13MT4(gen4,gen5)*CSSS(i,gen3,gen4)*
     -    CSSS(j,gen5,gen3)))/16.

          self(i,j)=self(i,j)+(-3*(CS13S13S13S13MT4(gen4,gen5,gen3,gen5) 
     -    +3*CS13S13S13S13MT4(gen4,gen5,gen5,gen3))*CSSSS(i,j,gen3,gen4)
     -    *(aAETN(gen5) + aA0TN(gen5)*aB0T1(gen3,gen4) + 
     -    aBEpT1(gen3,gen4)*MassSu2(gen5)))/16.
           do gen6=ihiggs,ihiggse
           self(i,j)=self(i,j)+(-3*(CSSS(i,gen3,gen4)*CSSS(j,gen5,gen3)*
     -     CSSSS(gen6,gen6,gen4,gen5) + CSSS(i,gen4,gen3)*
     -     CSSS(j,gen3,gen5)*CSSSS(gen6,gen6,gen5,gen4))*
     -     (A0TNH(gen6)*aC0T(gen3,gen4,gen5) + 
     -     aCEpT(gen3,gen4,gen5)*Mah2(gen6)))/32.
           enddo
          enddo
          self(i,j)=self(i,j)+(-3*(aet5 + a0t5*aB0T1(gen3,gen4) 
     -    +MSQ2*aBEpT1(gen3,gen4))*CS14S13S14S13MT4(gen4,gen3)*
     -    CSSSS(i,j,gen3,gen4))/16.

           do gen6=ihiggs,ihiggse
          self(i,j)=self(i,j)+(-3*CSSSS(gen6,gen6,gen4,gen3)*
     -    CSSSS(i,j,gen3,gen4)*(A0TNH(gen6)*aB0T1(gen3,gen4) +  
     -    AETNH(gen6) + aBEpT1(gen3,gen4)*Mah2(gen6)))/32.
           enddo
         enddo
        enddo

c neutral Higgs diagrams
        do h1=ihiggs,ihiggse
        self(i,j)=self(i,j)+
     &  ((3*M22m42I*(CSSS(h1,2,1)*(CSSS(j,1,2)*CSSSS(i,h1,1
     &  ,1)+CSSS(i,1,2)*CSSSS(j,h1,1,1))+CSSS(h1,1,2)*(CSSS(j,2,1)*CSSS
     &  S(i,h1,1,1)+CSSS(i,2,1)*CSSSS(j,h1,1,1))+CSSS(h1,1,1)*(CSSS(j,2
     &  ,1)*CSSSS(i,h1,1,2)+CSSS(j,1,2)*CSSSS(i,h1,2,1)+CSSS(i,2,1)*CSS
     &  SS(j,h1,1,2)+CSSS(i,1,2)*CSSSS(j,h1,2,1)))*FD111MH(h1,1,1))/16.
     &  -(3*M22m42I*((CSSS(h1,1,1)-CSSS(h1,2,2))*(CSSS(j,2,1)*CSSSS(i,h
     &  1,1,2)+CSSS(j,1,2)*CSSSS(i,h1,2,1)+CSSS(i,2,1)*CSSSS(j,h1,1,2)+
     &  CSSS(i,1,2)*CSSSS(j,h1,2,1))+CSSS(h1,2,1)*(CSSS(j,1,2)*(CSSSS(i
     &  ,h1,1,1)-CSSSS(i,h1,2,2))+CSSS(i,1,2)*(CSSSS(j,h1,1,1)-CSSSS(j,
     &  h1,2,2)))+CSSS(h1,1,2)*(CSSS(j,2,1)*(CSSSS(i,h1,1,1)-CSSSS(i,h1
     &  ,2,2))+CSSS(i,2,1)*(CSSSS(j,h1,1,1)-CSSSS(j,h1,2,2))))*FD111MH(
     &  h1,1,2))/16.-(3*M22m42I*((CSSS(h1,2,1)*CSSS(j,1,2)+CSSS(h1,1,2)
     &  *CSSS(j,2,1))*CSSSS(i,h1,2,2)+CSSS(h1,2,2)*(CSSS(j,2,1)*CSSSS(i
     &  ,h1,1,2)+CSSS(j,1,2)*CSSSS(i,h1,2,1)+CSSS(i,2,1)*CSSSS(j,h1,1,2
     &  )+CSSS(i,1,2)*CSSSS(j,h1,2,1))+(CSSS(h1,2,1)*CSSS(i,1,2)+CSSS(h
     &  1,1,2)*CSSS(i,2,1))*CSSSS(j,h1,2,2))*FD111MH(h1,2,2))/16.+(3*CS
     &  SS(h1,1,1)*(CSSS(j,1,1)*CSSSS(i,h1,1,1)+CSSS(i,1,1)*CSSSS(j,h1,
     &  1,1))*FD211MH(h1,1,1))/8.+(3*(CSSS(h1,2,1)*(CSSS(j,1,1)*CSSSS(i
     &  ,h1,1,2)+CSSS(i,1,1)*CSSSS(j,h1,1,2))+CSSS(h1,1,2)*(CSSS(j,1,1)
     &  *CSSSS(i,h1,2,1)+CSSS(i,1,1)*CSSSS(j,h1,2,1)))*FD211MH(h1,1,2))
     &  /16.+(3*(CSSS(h1,2,1)*(CSSS(j,2,2)*CSSSS(i,h1,1,2)+CSSS(i,2,2)*
     &  CSSSS(j,h1,1,2))+CSSS(h1,1,2)*(CSSS(j,2,2)*CSSSS(i,h1,2,1)+CSSS
     &  (i,2,2)*CSSSS(j,h1,2,1)))*FD211MH(h1,2,1))/16.+(3*CSSS(h1,2,2)*
     &  (CSSS(j,2,2)*CSSSS(i,h1,2,2)+CSSS(i,2,2)*CSSSS(j,h1,2,2))*FD211
     &  MH(h1,2,2))/8.)
     
       abrx(1)=(3*M22m42sI*(FD111MH(h1,1,1)-2*FD111MH(h1,1,2)+FD1
     &  11MH(h1,2,2)))/16.
       abrx(2)=(3*(-(M22m42sI*FD111MH(h1,1,1))+M22m42sI*FD111MH(h
     &  1,1,2)+M22m42I*FD211MH(h1,1,1)))/16.
       abrx(3)=(3*(-(M22m42sI*FD111MH(h1,1,1))+M22m42sI*FD111MH(h
     &  1,1,2)+2*M22m42I*FD211MH(h1,1,1)-M22m42I*FD211MH(h1,1,2)))/16.
       abrx(4)=(3*(-(M22m42sI*FD111MH(h1,1,2))+M22m42sI*FD111MH(h
     &  1,2,2)+M22m42I*FD211MH(h1,1,2)))/16.
       abrx(5)=(-3*(-(M22m42sI*FD111MH(h1,1,1))+M22m42sI*FD111MH(
     &  h1,1,2)+M22m42I*FD211MH(h1,2,1)))/16.
       abrx(6)=(3*(M22m42sI*FD111MH(h1,1,1)-2*M22m42sI*FD111MH(h1
     &  ,1,2)+M22m42sI*FD111MH(h1,2,2)+M22m42I*FD211MH(h1,1,2)-M22m42I*
     &  FD211MH(h1,2,1)))/16.
       abrx(7)=(3*(M22m42sI*FD111MH(h1,1,2)-M22m42sI*FD111MH(h1,2
     &  ,2)+M22m42I*(FD211MH(h1,2,1)-2*FD211MH(h1,2,2))))/16.
       abrx(8)=(-3*(-(M22m42sI*FD111MH(h1,1,2))+M22m42sI*FD111MH(
     &  h1,2,2)+M22m42I*FD211MH(h1,2,2)))/16.
       abrx(9)=(3*FD221MH(h1,1,2))/16.
       abrx(10)=(3*FD221MH(h1,2,1))/16.
       abrx(11)=(3*(FD221MH(h1,1,1)+2*FD311MH(h1,1,1)))/16.
       abrx(12)=(3*FD311MH(h1,1,2))/8.
       abrx(13)=(3*FD311MH(h1,2,1))/8.
       abrx(14)=(3*(FD221MH(h1,2,2)+2*FD311MH(h1,2,2)))/16.

       self(i,j)=self(i,j)+(abrx(4)*CSSS(h1,1,2)*CSSS(h1,2,2)*CSSS
     &  (i,2,1)*CSSS(j,1,1)+abrx(1)*CSSS(h1,2,1)**2*CSSS(i,1,2)*CSSS(j,
     &  1,2)+abrx(8)*CSSS(h1,2,2)**2*CSSS(i,2,1)*CSSS(j,1,2)+abrx(8)*CS
     &  SS(h1,2,2)**2*CSSS(i,1,2)*CSSS(j,2,1)+abrx(1)*CSSS(h1,1,2)**2*C
     &  SSS(i,2,1)*CSSS(j,2,1)+CSSS(h1,1,1)**2*(abrx(11)*CSSS(i,1,1)*CS
     &  SS(j,1,1)+abrx(2)*CSSS(i,2,1)*CSSS(j,1,2)+abrx(2)*CSSS(i,1,2)*C
     &  SSS(j,2,1))+abrx(14)*CSSS(h1,2,2)**2*CSSS(i,2,2)*CSSS(j,2,2)+CS
     &  SS(h1,1,2)*(abrx(4)*CSSS(h1,2,2)*CSSS(i,1,1)*CSSS(j,2,1)+abrx(7
     &  )*CSSS(h1,2,2)*CSSS(i,2,2)*CSSS(j,2,1)+abrx(7)*CSSS(h1,2,2)*CSS
     &  S(i,2,1)*CSSS(j,2,2))+CSSS(h1,2,1)*(CSSS(h1,1,2)*(abrx(12)*CSSS
     &  (i,1,1)+abrx(10)*CSSS(i,2,2))*CSSS(j,1,1)+abrx(4)*CSSS(h1,2,2)*
     &  CSSS(i,1,1)*CSSS(j,1,2)+abrx(7)*CSSS(h1,2,2)*CSSS(i,2,2)*CSSS(j
     &  ,1,2)+CSSS(i,1,2)*(abrx(4)*CSSS(h1,2,2)*CSSS(j,1,1)+abrx(6)*CSS
     &  S(h1,1,2)*CSSS(j,2,1)+abrx(7)*CSSS(h1,2,2)*CSSS(j,2,2))+CSSS(h1
     &  ,1,2)*(abrx(6)*CSSS(i,2,1)*CSSS(j,1,2)+abrx(9)*CSSS(i,1,1)*CSSS
     &  (j,2,2)+abrx(13)*CSSS(i,2,2)*CSSS(j,2,2)))+CSSS(h1,1,1)*(abrx(3
     &  )*CSSS(h1,1,2)*CSSS(i,2,1)*CSSS(j,1,1)+abrx(1)*CSSS(h1,2,2)*CSS
     &  S(i,2,1)*CSSS(j,1,2)+abrx(1)*CSSS(h1,2,2)*CSSS(i,1,2)*CSSS(j,2,
     &  1)+CSSS(h1,1,2)*(abrx(3)*CSSS(i,1,1)*CSSS(j,2,1)+abrx(5)*CSSS(i
     &  ,2,2)*CSSS(j,2,1)+abrx(5)*CSSS(i,2,1)*CSSS(j,2,2))+CSSS(h1,2,1)
     &  *(abrx(3)*CSSS(i,1,1)*CSSS(j,1,2)+abrx(5)*CSSS(i,2,2)*CSSS(j,1,
     &  2)+CSSS(i,1,2)*(abrx(3)*CSSS(j,1,1)+abrx(5)*CSSS(j,2,2)))))

       self(i,j)=self(i,j)+((3*(CSSS(h1,1,1)*(CSSS(h1,2,1)*CSSSS(i
     &  ,j,1,2)+CSSS(h1,1,2)*CSSSS(i,j,2,1))+M22m42*CSSSS(i,h1,1,1)*CSS
     &  SS(j,h1,1,1))*FD111MH(h1,1,1))/(16.*M22m42)+(3*(-((CSSS(h1,1,1)
     &  -CSSS(h1,2,2))*(CSSS(h1,2,1)*CSSSS(i,j,1,2)+CSSS(h1,1,2)*CSSSS(
     &  i,j,2,1)))+M22m42*(CSSSS(i,h1,2,1)*CSSSS(j,h1,1,2)+CSSSS(i,h1,1
     &  ,2)*CSSSS(j,h1,2,1)))*FD111MH(h1,1,2))/(16.*M22m42)+((-3*CSSS(h
     &  1,2,2)*(CSSS(h1,2,1)*CSSSS(i,j,1,2)+CSSS(h1,1,2)*CSSSS(i,j,2,1)
     &  )+3*M22m42*CSSSS(i,h1,2,2)*CSSSS(j,h1,2,2))*FD111MH(h1,2,2))/(1
     &  6.*M22m42)+(3*CSSS(h1,1,1)**2*CSSSS(i,j,1,1)*FD211MH(h1,1,1))/1
     &  6.+(3*CSSS(h1,1,2)*CSSS(h1,2,1)*CSSSS(i,j,1,1)*FD211MH(h1,1,2))
     &  /16.+(3*CSSS(h1,1,2)*CSSS(h1,2,1)*CSSSS(i,j,2,2)*FD211MH(h1,2,1
     &  ))/16.+(3*CSSS(h1,2,2)**2*CSSSS(i,j,2,2)*FD211MH(h1,2,2))/16.)

        enddo


        do gen3=1,2
         do gen4=1,2
       self(i,j)=self(i,j)+(-3*(AETHp + A0THp*aB0T1(gen3,gen4) + MHp2N*
     -   aBEpT1(gen3,gen4))*CS3SSS13(gen4,gen3)*CSSSS(i,j,gen3,gen4))/16. 
          do gen5=1,2
       self(i,j)=self(i,j)+ (-3*(A0THp*aC0T(gen3,gen4,gen5) + 
     -    MHp2N*aCEpT(gen3,gen4,gen5))*
     -    (CS3SSS13(gen5,gen4)*CSSS(i,gen4,gen3)*CSSS(j,gen3,gen5) + 
     -    CS3SSS13(gen4,gen5)*CSSS(i,gen3,gen4)*CSSS(j,gen5,gen3)))/16.
          enddo
         enddo
        enddo

c charged higgs diagrams
        do h1=1,2
       self(i,j)=self(i,j)+((3*((CS1S3SSC(j,h1,1)*CS3S13S(h1,2)+CS
     &  1S3SS(j,h1,2)*CS3S13SC(h1,1))*CSSS(i,1,2)+(CS1S3SSC(j,h1,2)*CS3
     &  S13S(h1,1)+CS1S3SS(j,h1,1)*CS3S13SC(h1,2))*CSSS(i,2,1)+(CS1S3SS
     &  C(i,h1,1)*CS3S13S(h1,2)+CS1S3SS(i,h1,2)*CS3S13SC(h1,1))*CSSS(j,
     &  1,2)+(CS1S3SSC(i,h1,2)*CS3S13S(h1,1)+CS1S3SS(i,h1,1)*CS3S13SC(h
     &  1,2))*CSSS(j,2,1))*FD111CH(h1,1))/(16.*M22m42)-(3*((CS1S3SSC(j,
     &  h1,1)*CS3S13S(h1,2)+CS1S3SS(j,h1,2)*CS3S13SC(h1,1))*CSSS(i,1,2)
     &  +(CS1S3SSC(j,h1,2)*CS3S13S(h1,1)+CS1S3SS(j,h1,1)*CS3S13SC(h1,2)
     &  )*CSSS(i,2,1)+(CS1S3SSC(i,h1,1)*CS3S13S(h1,2)+CS1S3SS(i,h1,2)*C
     &  S3S13SC(h1,1))*CSSS(j,1,2)+(CS1S3SSC(i,h1,2)*CS3S13S(h1,1)+CS1S
     &  3SS(i,h1,1)*CS3S13SC(h1,2))*CSSS(j,2,1))*FD111CH(h1,2))/(16.*M2
     &  2m42)+(3*((CS1S3SSC(j,h1,1)*CS3S13S(h1,1)+CS1S3SS(j,h1,1)*CS3S1
     &  3SC(h1,1))*CSSS(i,1,1)+(CS1S3SSC(i,h1,1)*CS3S13S(h1,1)+CS1S3SS(
     &  i,h1,1)*CS3S13SC(h1,1))*CSSS(j,1,1))*FD211CH(h1,1))/16.+(3*((CS
     &  1S3SSC(j,h1,2)*CS3S13S(h1,2)+CS1S3SS(j,h1,2)*CS3S13SC(h1,2))*CS
     &  SS(i,2,2)+(CS1S3SSC(i,h1,2)*CS3S13S(h1,2)+CS1S3SS(i,h1,2)*CS3S1
     &  3SC(h1,2))*CSSS(j,2,2))*FD211CH(h1,2))/16.)

       abrx(1)=(3*(-(M22m42sI*FD111CH(h1,1))+M22m42sI*FD111CH(h1,
     &  2)+M22m42I*FD211CH(h1,1)))/16.
       abrx(2)=(-3*(-(M22m42sI*FD111CH(h1,1))+M22m42sI*FD111CH(h1
     &  ,2)+M22m42I*FD211CH(h1,2)))/16.
       abrx(3)=(3*FD311CH(h1,1))/8.
       abrx(4)=(3*FD311CH(h1,2))/8.

       self(i,j)=self(i,j)+((abrx(3)*CS3S13S(h1,1)*CS3S13SC(h1,1)*
     &  CSSS(i,1,1)+abrx(1)*CS3S13S(h1,1)*CS3S13SC(h1,2)*CSSS(i,2,1))*C
     &  SSS(j,1,1)+abrx(2)*CS3S13S(h1,2)*CS3S13SC(h1,1)*CSSS(i,2,2)*CSS
     &  S(j,1,2)+CSSS(i,1,1)*(abrx(1)*CS3S13S(h1,2)*CS3S13SC(h1,1)*CSSS
     &  (j,1,2)+abrx(1)*CS3S13S(h1,1)*CS3S13SC(h1,2)*CSSS(j,2,1))+CSSS(
     &  i,1,2)*(abrx(1)*CS3S13S(h1,2)*CS3S13SC(h1,1)*CSSS(j,1,1)+(abrx(
     &  1)*CS3S13S(h1,1)*CS3S13SC(h1,1)+abrx(2)*CS3S13S(h1,2)*CS3S13SC(
     &  h1,2))*CSSS(j,2,1)+abrx(2)*CS3S13S(h1,2)*CS3S13SC(h1,1)*CSSS(j,
     &  2,2))+CSSS(i,2,1)*((abrx(1)*CS3S13S(h1,1)*CS3S13SC(h1,1)+abrx(2
     &  )*CS3S13S(h1,2)*CS3S13SC(h1,2))*CSSS(j,1,2)+abrx(2)*CS3S13S(h1,
     &  1)*CS3S13SC(h1,2)*CSSS(j,2,2))+CSSS(i,2,2)*(abrx(2)*CS3S13S(h1,
     &  1)*CS3S13SC(h1,2)*CSSS(j,2,1)+abrx(4)*CS3S13S(h1,2)*CS3S13SC(h1
     &  ,2)*CSSS(j,2,2)))

       abrx(1)=(3*M22m42I*(FD111CH(h1,1)-FD111CH(h1,2)))/16.
       abrx(2)=(3*M22m42I*(M2**2-M4**2)*(CS1S3SS(j,h1,1)*CS1S3SSC
     &  (i,h1,1)*FD111CH(h1,1)+CS1S3SS(i,h1,1)*CS1S3SSC(j,h1,1)*FD111CH
     &  (h1,1)+(CS1S3SS(j,h1,2)*CS1S3SSC(i,h1,2)+CS1S3SS(i,h1,2)*CS1S3S
     &  SC(j,h1,2))*FD111CH(h1,2)))/16.
       abrx(3)=(3*FD211CH(h1,1))/16.
       abrx(4)=(3*FD211CH(h1,2))/16.
       self(i,j)=self(i,j)+(abrx(2)+abrx(3)*CS3S13S(h1,1)*CS3S13SC
     &  (h1,1)*CSSSS(i,j,1,1)+abrx(1)*CS3S13S(h1,2)*CS3S13SC(h1,1)*CSSS
     &  S(i,j,1,2)+abrx(1)*CS3S13S(h1,1)*CS3S13SC(h1,2)*CSSSS(i,j,2,1)+
     &  abrx(4)*CS3S13S(h1,2)*CS3S13SC(h1,2)*CSSSS(i,j,2,2))
        enddo

c neutral Higgs top diagrams

        do h1=ihiggs,ihiggse
       self(i,j)=self(i,j)+(3*(FDTH(h1,5)*hy(i)*hy(j)*hyc(h1)**2+F
     &  DTH(h1,3)*hy(h1)**2*hy(j)*hyc(i)+FDTH(h1,4)*hy(h1)*hy(j)*hyc(h1
     &  )*hyc(i)+FDTH(h1,3)*hy(j)*hyc(h1)**2*hyc(i)+FDTH(h1,3)*hy(h1)**
     &  2*hy(i)*hyc(j)+FDTH(h1,4)*hy(h1)*hy(i)*hyc(h1)*hyc(j)+FDTH(h1,3
     &  )*hy(i)*hyc(h1)**2*hyc(j)+FDTH(h1,5)*hy(h1)**2*hyc(i)*hyc(j)+FD
     &  TH(h1,2)*hy(h1)*hyc(h1)*(hy(i)*hy(j)+hyc(i)*hyc(j))+FDTH(h1,1)*
     &  (hy(h1)**2*hy(i)*hy(j)+hyc(h1)**2*hyc(i)*hyc(j))))/8.
        enddo

c charged Higgs top+bottom diagrams

        do h1=1,2
       self(i,j)=self(i,j)+(3*CTBS3(h1)*CTBS3C(h1)*(FDTCH(h1,2)*(hy
     &  (j)*hyc(i)+hy(i)*hyc(j))+FDTCH(h1,1)*(hy(i)*hy(j)+hyc(i)*hyc(j))
     &  ))/8.
        enddo


c 1 neutralino diagrams
       self(i,j)=self(i,j)+((FDNeu(18)*CSSS(i,1,2)+FDNeu(6)*CSSS(i,2
     &  ,1))*CSSS(j,1,1)+FDNeu(19)*CSSS(i,2,2)*CSSS(j,1,2)+CSSS(i,1,1)*(
     &  FDNeu(4)*CSSS(j,1,1)+FDNeu(18)*CSSS(j,1,2)+FDNeu(6)*CSSS(j,2,1))+C
     &  SSS(i,2,1)*(FDNeu(20)*CSSS(j,1,2)+FDNeu(7)*CSSS(j,2,2))+CSSS(i,2,
     &  2)*(FDNeu(7)*CSSS(j,2,1)+FDNeu(13)*CSSS(j,2,2))+CSSS(i,1,2)*(FDNeu
     &  (20)*CSSS(j,2,1)+FDNeu(19)*CSSS(j,2,2))+FDNeu(1)*CSSSS(i,j,1,1)+
     &  FDNeu(17)*CSSSS(i,j,1,2)+FDNeu(5)*CSSSS(i,j,2,1)+FDNeu(10)*CSSSS(i,
     &  j,2,2)+hy(j)*(FDNeu(2)*CSSS(i,1,1)+FDNeu(21)*CSSS(i,1,2)+FDNeu(8)*
     &  CSSS(i,2,1)+FDNeu(11)*CSSS(i,2,2)+FDNeu(16)*hyc(i))+(FDNeu(3)*CSSS
     &  (i,1,1)+FDNeu(22)*CSSS(i,1,2)+FDNeu(9)*CSSS(i,2,1)+FDNeu(12)*CSSS(
     &  i,2,2))*hyc(j)+hyc(i)*(FDNeu(3)*CSSS(j,1,1)+FDNeu(22)*CSSS(j,1,2)
     &  +FDNeu(9)*CSSS(j,2,1)+FDNeu(12)*CSSS(j,2,2)+FDNeu(15)*hyc(j))+hy(i
     &  )*(FDNeu(2)*CSSS(j,1,1)+FDNeu(21)*CSSS(j,1,2)+FDNeu(8)*CSSS(j,2,1)
     &  +FDNeu(11)*CSSS(j,2,2)+FDNeu(14)*hy(j)+FDNeu(16)*hyc(j)))

c 1 chargino diagrams
       self(i,j)=self(i,j)+((FDCha(6)*CF12F4S(1)*CF12F4SC(1)*CSSS(i,1,1)
     &  +FDCha(4)*CF12F4S(2)*CF12F4SC(1)*CSSS(i,2,1))*CSSS(j,1,1)+FDCha
     &  (5)*CF12F4S(1)*CF12F4SC(2)*CSSS(i,2,2)*CSSS(j,1,2)+CSSS(i,1,1)*
     &  (FDCha(4)*CF12F4S(1)*CF12F4SC(2)*CSSS(j,1,2)+FDCha(4)*CF12F4S(2)
     &  *CF12F4SC(1)*CSSS(j,2,1))+CSSS(i,2,1)*((FDCha(4)*CF12F4S(1)*CF12
     &  F4SC(1)+FDCha(5)*CF12F4S(2)*CF12F4SC(2))*CSSS(j,1,2)+FDCha(5)*C
     &  F12F4S(2)*CF12F4SC(1)*CSSS(j,2,2))+CSSS(i,1,2)*(FDCha(4)*CF12F4S
     &  (1)*CF12F4SC(2)*CSSS(j,1,1)+(FDCha(4)*CF12F4S(1)*CF12F4SC(1)+FDC
     &  ha(5)*CF12F4S(2)*CF12F4SC(2))*CSSS(j,2,1)+FDCha(5)*CF12F4S(1)*CF1
     &  2F4SC(2)*CSSS(j,2,2))+CSSS(i,2,2)*(FDCha(5)*CF12F4S(2)*CF12F4SC(
     &  1)*CSSS(j,2,1)+FDCha(7)*CF12F4S(2)*CF12F4SC(2)*CSSS(j,2,2))+FDCha
     &  (2)*CF12F4S(1)*CF12F4SC(1)*CSSSS(i,j,1,1)+FDCha(1)*CF12F4S(1)*CF
     &  12F4SC(2)*CSSSS(i,j,1,2)+FDCha(1)*CF12F4S(2)*CF12F4SC(1)*CSSSS(i
     &  ,j,2,1)+FDCha(3)*CF12F4S(2)*CF12F4SC(2)*CSSSS(i,j,2,2))

      self(i,j)=self(i,j)+(3*CF12F3S*CF12F3SC*(hyc(i)*((M12*FD311
     &  Cha(2)-M12*FD311Cha(3)+FD311Cha(4)-FD311Cha(7))*hy(j)+2*M12*(FD
     &  311Cha(2)-FD311Cha(3))*hyc(j))+hy(i)*(2*M12*(FD311Cha(2)-FD311C
     &  ha(3))*hy(j)+(M12*FD311Cha(2)-M12*FD311Cha(3)+FD311Cha(4)-FD311
     &  Cha(7))*hyc(j))))/8. 

      self(i,j)=self(i,j)*fac
      self(j,i)=self(i,j)
       enddo
      enddo

      do i=1,6
       do j=1,6 
       selfIN(i,j)=0D0
      do gen3=1,6
        do gen2=1,6
       selfIN(i,j)=selfIN(i,j) + ZH(gen2, i)*ZH(gen3, j)* self(gen2,gen3)
        enddo
      enddo
        enddo
      enddo
c end calculate Higgs selfenergies from genuine two-loop diagrams
#ifdef printMT4
      Write(*,*), "___________________________________________________"
      Write(*,*), "Contribution proportional to MT4 finite part"
      Write(*,*), "self(1,1-6)",self(1,1),self(1,2),self(1,3),self(1,4),self(1,5),self(1,6)
      Write(*,*), "self(2,2-6)",self(2,2),self(2,3),self(2,4),self(2,5),self(2,6)
      Write(*,*), "self(3,3-6)",self(3,3),self(3,4),self(3,5),self(3,6)
      Write(*,*), "self(4,4-6)",self(4,4),self(4,5),self(4,6)
      Write(*,*), "self(5,5-6)",self(5,5),self(5,6)
      Write(*,*), "self(6,6-6)",self(6,6)
      Write(*,*), "___________________________________________________"
      Write(*,*), " finite part in interaction basis"
      Write(*,*), "selfIN(1,1-6)",selfIN(1,1),selfIN(1,2),selfIN(1,3),selfIN(1,4),selfIN(1,5),selfIN(1,6)
      Write(*,*), "selfIN(2,2-6)",selfIN(2,2),selfIN(2,3),selfIN(2,4),selfIN(2,5),selfIN(2,6)
      Write(*,*), "selfIN(3,3-6)",selfIN(3,3),selfIN(3,4),selfIN(3,5),selfIN(3,6)
      Write(*,*), "selfIN(4,4-6)",selfIN(4,4),selfIN(4,5),selfIN(4,6)
      Write(*,*), "selfIN(5,5-6)",selfIN(5,5),selfIN(5,6)
      Write(*,*), "selfIN(6,6-6)",selfIN(6,6)
      Write(*,*), "___________________________________________________"
#endif
      end subroutine UnselfHH_atat
c
c**********************************************************************
c
      subroutine CT1loopMT4(MQrun,dmsq4fin,dmsur4fin,dat4fin)
      implicit none
      integer i,j,k,l
      integer I2G2,I5G2,I6G2,I2G3,I2G4,I3G3,I5G3,I5G4,I6G4,I6G3
      double precision MassHpm(2),A0T,AEpT,B0T,BEpT,fac,selfHH1MT4IN(6,6),
     .                 selfHH1MT4(6,6),dmsq4fin,dmsur4fin
      double complex B0P,B1P,BEpP,B1EpP
      double precision Pi2,CF,xF0,dPMW24,
     .     dPMZ24,dMW24,dMZ24,dMWeps,dMZeps
      double complex tad1SP(6),tad1(6),dat4fin
      logical MQrun
#include "TLMT4.h"

      pi2=pi**2
      fac= 1/16D0/pi2
      MassHpm(1)=0D0
      MassHpm(2)=Sqrt(MHp2)
      dZHu4=(-3*M12)/(8.*Pi2*SB2*vv2)

      A0T1=A0T(M12,Q2)
      A0T2=A0T(M22,Q2)
      A0T4=A0T(M42,Q2)
      aA0TN(1)=A0T2
      aA0TN(2)=A0T4
      A0T5= A0T(MSQ2,Q2)
      do i=ihiggs,ihiggse
        A0TNH(i) =A0T(Mah2(i),Q2)
      enddo  
      do i=ineu,ineue
      A0TNeu(i)= A0T(MNeu2(i),Q2)
      enddo
      a0tcha=A0T(MCha2,Q2)

#ifndef CHARGEDHIGGSOS
       dMHp1MT4(1)=(3*(CS3SSS14*MSQ2+CS3S13S14(1)*CS3S14S13(1)+
     &  CS3S13S14(2)*CS3S14S13(2)+M22*CS3SSS13(1,1)+M42*CS3SSS13(2,2)-
     &  2*CTBS3R*M12*CTBS3C(2)))*fac-CB2*MHp2*dZHu4
       dMHp1MT4(2)=(3*(a0t5*(CS3SSS14*(M22-MSQ2)*(-M42+MSQ2)+M42*
     &  CS3S13S14(1)*CS3S14S13(1)-MSQ2*CS3S13S14(1)*CS3S14S13(1)+M22*
     &  CS3S13S14(2)*CS3S14S13(2)-MSQ2*CS3S13S14(2)*CS3S14S13(2))+a0t2*
     &  (M42-MSQ2)*(-(CS3S13S14(1)*CS3S14S13(1))+(-M22+MSQ2)*
     &  CS3SSS13(1,1))+(M22-MSQ2)*(-(a0t4*(CS3S13S14(2)*CS3S14S13(2)+
     &  (M42-MSQ2)*CS3SSS13(2,2)))+2*a0t1*CTBS3R*(M42-MSQ2)*CTBS3C(2)))
     &  )/((M22-MSQ2)*(-M42+MSQ2))*fac
c nhung should set here zero
       dMHp1MT4(3)=(3*(aet5*CS3SSS14+((aet2-aet5)*CS3S13S14(1)*
     &  CS3S14S13(1))/(M22-MSQ2)+((aet4-aet5)*CS3S13S14(2)*CS3S14S13(2)
     &  )/(M42-MSQ2)+aet2*CS3SSS13(1,1)+aet4*CS3SSS13(2,2)-2*aet1*
     &  CTBS3R*CTBS3C(2)))/(16.*Pi2)
       dMHp1MT4(3)=0
#else
      dMHp1MT4(1)=(3*(CS3SSS14*MSQ2+CS3S13S14(1)*CS3S14S13(1)+
     &  CS3S13S14(2)*CS3S14S13(2)+M22*CS3SSS13(1,1)+M42*CS3SSS13(2,2)-
     &  2*CTBS3R*M12*CTBS3C(2)+CTBS3R*MHp2*CTBS3C(2)))*fac
       dMHp1MT4(2)=(3*(a0t5*CS3SSS14+B0P(MHp2,M22,MSQ2,Q2)*CS3S13S14(1)
     &  *CS3S14S13(1)+B0P(MHp2,M42,MSQ2,Q2)*CS3S13S14(2)*CS3S14S13(2)+
     &  a0t2*CS3SSS13(1,1)+a0t4*CS3SSS13(2,2)-a0t1*CTBS3R*CTBS3C(2)-
     &  CTBS3R*M12*B0P(MHp2,0D0,M12,Q2)*CTBS3C(2)+CTBS3R*MHp2*
     &  B0P(MHp2,0D0,M12,Q2)*CTBS3C(2)))*fac
       dMHp1MT4(3)=(3*(aet5*CS3SSS14+BEpP(MHp2,M22,MSQ2,Q2)*
     &  CS3S13S14(1)*CS3S14S13(1)+BEpP(MHp2,M42,MSQ2,Q2)*CS3S13S14(2)*
     &  CS3S14S13(2)+aet2*CS3SSS13(1,1)+aet4*CS3SSS13(2,2)-aet1*CTBS3R*
     &  CTBS3C(2)-CTBS3R*M12*BEpP(MHp2,0D0,M12,Q2)*CTBS3C(2)+CTBS3R*
     &  MHp2*BEpP(MHp2,0D0,M12,Q2)*CTBS3C(2)))/(16.*Pi2)
       dMHp1MT4(3)=0
#endif

#ifdef checkPS
       i=5;j=5 ! for A boson
       dMAA1MT4(1)=(3*(CSSS(i,1,1)*CSSS(j,1,1)+CSSS(i,2,1)*CSSS(j,1,2)+
     &  CSSS(i,1,2)*CSSS(j,2,1)+CSSS(i,2,2)*CSSS(j,2,2)+M22*
     &  CSSSS(i,j,1,1)+M42*CSSSS(i,j,2,2)-2*M12*hy(i)*hy(j)-4*M12*hy(j)
     &  *hyc(i)-4*M12*hy(i)*hyc(j)-2*M12*hyc(i)*hyc(j)))*fac
     &   - CB2*dZHu4*MHp2
       dMAA1MT4(2)=(3*(B0P(0D0,M22,M22,Q2)*CSSS(i,1,1)*CSSS(j,1,1)+
     &  B0P(0D0,M22,M42,Q2)*(CSSS(i,2,1)*CSSS(j,1,2)+CSSS(i,1,2)*
     &  CSSS(j,2,1))+B0P(0D0,M42,M42,Q2)*CSSS(i,2,2)*CSSS(j,2,2)+a0t2*
     &  CSSSS(i,j,1,1)+a0t4*CSSSS(i,j,2,2)-2*M12*B0P(0D0,M12,M12,Q2)*
     &  hy(i)*hy(j)-2*a0t1*hy(j)*hyc(i)-2*M12*B0P(0D0,M12,M12,Q2)*hy(j)*
     &  hyc(i)-2*a0t1*hy(i)*hyc(j)-2*M12*B0P(0D0,M12,M12,Q2)*hy(i)*hyc(j)
     &  -2*M12*B0P(0D0,M12,M12,Q2)*hyc(i)*hyc(j)))*fac
       dMHp1MT4(1:2)=dMAA1MT4(1:2)
       dMHp1MT4(3)=0

       print*, "dMAA1MT4(1:2)", dMAA1MT4(1:2)
#endif

         do i=ihiggs,ihiggse
       tad1SP(i)=fac*(3*M22*CSSS(i,1,1)+3*M42*CSSS(i,2,2)-6*M1**3*(hy(i)+
     &  hyc(i)))
       tad1(i)=fac*(3*a0t2*CSSS(i,1,1)+3*a0t4*CSSS(i,2,2)-6*a0t1*
     &  M1*(hy(i)+hyc(i)))
         enddo

         dTad1MT4(1:6,1:3)=0D0
         do i=1,6
          do j=1,6
         dTad1MT4(i,1) =   dTad1MT4(i,1) + ZH(j,i) * tad1SP(j)   
         dTad1MT4(i,2) =   dTad1MT4(i,2) + ZH(j,i) * tad1(j)   
          enddo  
         enddo

         dTanB4= dZHu4*TB/2D0
         dPMW24 = -3/8.*M12/pi2/vv2
         dPMZ24 = -3/8.*M12/pi2/vv2 

         CF    = 3/16D0/pi2/vv2
         dMW24 = -CF*(-M12 + 2*A0T1+ZU(1,1)*DCONJG(ZU(1,1))*xF0(M22,MSQ2)+
     -      ZU(2,1)*DCONJG(ZU(2,1))*xF0(M42,MSQ2))
         dMZ24 = -(- 2*CF*M12*Log(M12/Q2) +CF*ZU(1,1)*DCONJG(ZU(1,1))
     -      *ZU(1,2)*DCONJG(ZU(1,2))* xF0(M22,M42))

         dv4(1)  =(-CW2/SW2/2.*(dPMW24 - dPMZ24)+1/2.*dPMW24)*vv
         dSW4(1)=- CW**2/SW/2D0 *(dPMW24 - dPMZ24)
         dlambdaC4= -1/2D0* dZHu4* DCONJG(la)
         dMUE4= ( -0.5D0* dZHu4 *MUE)

         dv4(2)= (-CW2/SW2/2.*(dMW24 - dMZ24)+1/2.*dMW24)*vv
         dSW4(2)= -CW2/SW/2D0 *(dMW24 - dMZ24)


c   nhung test paßehr 
c         dSW4(1:3)=0D0
c         dv4(1)=(+1/2.*dPMW24)*vv
c         dv4(2)= (0+1/2.*dMW24)*vv
c end nhung test
       dCW4(1:3) = -((dSW4(1:3)*SW)/CW)
       dmt4(1:3) =0D0    
       dMSf4(1:2,1:2,1:3)=0D0

       dmt4(1)=(CF12F3S*CF12F3SC*M1)/4.+dmt4(1)
       dMSf4(1,1,1)=dMSf4(1,1,1)+(MSQ2*CS14S13S14S13MT4(1,1))
       dMSf4(1,1,1)=(M22-2*MCha2)*CF12F4S(1)*CF12F4SC(1)+dMSf4(1,1,1)
       dMSf4(1,2,1)=((M22+M42-4*MCha2)*CF12F4S(1)*CF12F4SC(2))/2.+dMSf4(1,2,1)
       dMSf4(1,2,1)=dMSf4(1,2,1)+(MSQ2*CS14S13S14S13MT4(2,1))
       dMSf4(2,2,1)=(M42-2*MCha2)*CF12F4S(2)*CF12F4SC(2)+dMSf4(2,2,1)
       dMSf4(2,2,1)=dMSf4(2,2,1)+(MSQ2*CS14S13S14S13MT4(2,2))
       Do  I6G3= ihiggs,ihiggse
       dmt4(1)=dmt4(1)+(M1*(hy(I6G3)**2+hy(I6G3)*hyc(I6G3)+hyc(I6G3)**2))/2.
       dMSf4(1,1,1)=dMSf4(1,1,1)+((CSSSS(I6G3,I6G3,1,1)*Mah2(I6G3))/2.)
       dMSf4(1,2,1)=dMSf4(1,2,1)+((CSSSS(I6G3,I6G3,2,1)*Mah2(I6G3))/2.)
       dMSf4(2,2,1)=dMSf4(2,2,1)+(CSSSS(I6G3,I6G3,2,2)*Mah2(I6G3))/2.
       enddo
 
       Do  I2G3= 1,2
       dmt4(1)=(M1*CTBS3(I2G3)*CTBS3C(I2G3))/4.+dmt4(1)
       dMSf4(1,1,1)=dMSf4(1,1,1)+CS3S3S13S13(I2G3,I2G3,1,1)*MassHpm(I2G3)**2
       dMSf4(1,1,1)=CS3S13S(I2G3,1)*CS3S13SC(I2G3,1)+dMSf4(1,1,1)
       dMSf4(1,2,1)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,1)+dMSf4(1,2,1)
       dMSf4(1,2,1)=dMSf4(1,2,1)+CS3S3S13S13(I2G3,I2G3,2,1)*MassHpm(I2G3)**2
       dMSf4(2,2,1)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,2)+dMSf4(2,2,1)
       dMSf4(2,2,1)=dMSf4(2,2,1)+CS3S3S13S13(I2G3,I2G3,2,2)*MassHpm(I2G3)**2
       dMSf4(1,1,1)=dMSf4(1,1,1)+(
     &  CS13S13S13S13MT4(1,I2G3,1,I2G3)*MassSu2(I2G3)+3*
     &  CS13S13S13S13MT4(1,I2G3,I2G3,1)*MassSu2(I2G3))
       dMSf4(1,2,1)=dMSf4(1,2,1)+(
     &  CS13S13S13S13MT4(2,I2G3,1,I2G3)*MassSu2(I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,1)*MassSu2(I2G3))
       dMSf4(2,2,1)=dMSf4(2,2,1)+(
     &  CS13S13S13S13MT4(2,I2G3,2,I2G3)*MassSu2(I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,2)*MassSu2(I2G3))
       enddo

       Do  I5G3= ineu,ineue
       Do  I6G3= 1,2
       dmt4(1)=dmt4(1)+(CF11FSR(I5G3,I6G3)*(M1*CF11FSRC(I5G3,I6G3)+2*
     &  CF11FSLC(I5G3,I6G3)*MNeu(I5G3))+CF11FSL(I5G3,I6G3)*(M1*
     &  CF11FSLC(I5G3,I6G3)+2*CF11FSRC(I5G3,I6G3)*MNeu(I5G3)))/4d0
       enddo
       enddo

       Do  I5G3= ineu,ineue
       dMSf4(1,1,1)=dMSf4(1,1,1)+(CF11FSL(I5G3,1)*(-2*M1*
     &  CF11FSRC(I5G3,1)*MNeu(I5G3)+CF11FSLC(I5G3,1)*(-2*M12+M22-2*
     &  MNeu2(I5G3)))+CF11FSR(I5G3,1)*(-2*M1*CF11FSLC(I5G3,1)*
     &  MNeu(I5G3)+CF11FSRC(I5G3,1)*(-2*M12+M22-2*MNeu2(I5G3))))
       dMSf4(1,2,1)=dMSf4(1,2,1)+(CF11FSL(I5G3,1)*(-4*M1*
     &  CF11FSRC(I5G3,2)*MNeu(I5G3)+CF11FSLC(I5G3,2)*(-4*M12+M22+M42-4*
     &  MNeu2(I5G3)))+CF11FSR(I5G3,1)*(-4*M1*CF11FSLC(I5G3,2)*
     &  MNeu(I5G3)+CF11FSRC(I5G3,2)*(-4*M12+M22+M42-4*MNeu2(I5G3))))/2.
       dMSf4(2,2,1)=dMSf4(2,2,1)+(CF11FSL(I5G3,2)*(-2*M1*
     &  CF11FSRC(I5G3,2)*MNeu(I5G3)+CF11FSLC(I5G3,2)*(-2*M12+M42-2*
     &  MNeu2(I5G3)))+CF11FSR(I5G3,2)*(-2*M1*CF11FSLC(I5G3,2)*
     &  MNeu(I5G3)+CF11FSRC(I5G3,2)*(-2*M12+M42-2*MNeu2(I5G3))))
       enddo

       Do  I6G3= ihiggs,ihiggse
       Do  I6G4= 1,2
       dMSf4(1,1,1)=CSSS(I6G3,1,I6G4)*CSSS(I6G3,I6G4,1)+dMSf4(1,1,1)
       dMSf4(1,2,1)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,1)+dMSf4(1,2,1)
       dMSf4(2,2,1)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,2)+dMSf4(2,2,1)
       enddo
       enddo
c**********************
       if(MQrun) then
       dMT4(2:3)=0D0;dMSf4(1:2,1:2,2:3)=0D0
       dMSQ4(2:3)=0D0;dMhp1MT4(3) =0D0
       dTad1MT4(1:6,3)=0D0; dv4(3) =0D0
#ifdef checkPS
       dTad1MT4(1:6,1:3)=0D0; dv4(2:3) =0D0;
#endif
       else

       dmt4(2)=-(CF12F3S*CF12F3SC*M1*DBLE(B1P(M12,MCha2,MSQ2,Q2)))/2d0+dmt4(2)

       dMSf4(1,1,2)=(a0t5*CS14S13S14S13MT4(1,1))+dMSf4(1,1,2)
       dMSf4(1,1,2)=CF12F4S(1)*CF12F4SC(1)*(-a0tcha+(M22-MCha2)*
     &  DBLE(B0P(M22,0D0,MCha2,Q2)))+dMSf4(1,1,2)

       dMSf4(1,2,2)=(CF12F4S(1)*CF12F4SC(2)*(-2*a0tcha+(M22-MCha2)*
     &  DBLE(B0P(M22,0D0,MCha2,Q2))+(M42-MCha2)*DBLE(B0P(M42,0D0,MCha2,Q2))
     &  ))/2.+dMSf4(1,2,2)
       dMSf4(1,2,2)=(a0t5*CS14S13S14S13MT4(2,1))+dMSf4(1,2,2)

       dMSf4(2,2,2)=CF12F4S(2)*CF12F4SC(2)*(-A0T(MCha2,Q2)+(M42-MCha2)*
     &  DBLE(B0P(M42,0D0,MCha2,Q2)))+dMSf4(2,2,2)
       dMSf4(2,2,2)=(a0t5*CS14S13S14S13MT4(2,2))+dMSf4(2,2,2)

       Do  I2G3= 1,2
       dmt4(2)=-(M1*DBLE(B1P(M12,0D0,MassHpm(I2G3)**2,Q2))*CTBS3(I2G3)*
     &  CTBS3C(I2G3))/2d0+dmt4(2)
       dMSf4(1,1,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,1,1)+dMSf4(1,1,2)
       dMSf4(1,1,2)=CS3S13S(I2G3,1)*CS3S13SC(I2G3,1)*
     &  DBLE(B0P(M22,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(CS3S13S(I2G3,2)*CS3S13SC(I2G3,1)*
     &  (DBLE(B0P(M22,MSQ2,MassHpm(I2G3)**2,Q2))+
     &  DBLE(B0P(M42,MSQ2,MassHpm(I2G3)**2,Q2))))/2.+dMSf4(1,2,2)
       dMSf4(1,2,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,2,1)
     &              +dMSf4(1,2,2)
       dMSf4(2,2,2)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,2)*
     &  DBLE(B0P(M42,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(2,2,2)
       dMSf4(2,2,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,2,2)
     &  +dMSf4(2,2,2)
       dMSf4(1,1,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(1,I2G3,1,I2G3)+3*
     &  CS13S13S13S13MT4(1,I2G3,I2G3,1)))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(2,I2G3,1,I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,1)))+dMSf4(1,2,2)
       dMSf4(2,2,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(2,I2G3,2,I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,2)))+dMSf4(2,2,2)
       enddo




       Do  I6G3= ihiggs,ihiggse
       dmt4(2)=dmt4(2)+(-(M1*DBLE(B1P(M12,M12,Mah2(I6G3),Q2))*hy(I6G3)*
     &  hyc(I6G3))+(M1*DBLE(B0P(M12,M12,Mah2(I6G3),Q2))*(hy(I6G3)**2+
     &  hyc(I6G3)**2))/2.)
       dMSf4(1,1,2)=((A0TNH(I6G3)*CSSSS(I6G3,I6G3,1,1))/2.)+dMSf4(1,1,2)
       dMSf4(1,2,2)=A0TNH(I6G3)*CSSSS(I6G3,I6G3,2,1)/2.+dMSf4(1,2,2)
       dMSf4(2,2,2)=(A0TNH(I6G3)*CSSSS(I6G3,I6G3,2,2))/2.+dMSf4(2,2,2)
       enddo


       Do  I6G3= ihiggs,ihiggse
       Do  I6G4= 1,2
       dMSf4(1,1,2)=CSSS(I6G3,1,I6G4)*CSSS(I6G3,I6G4,1)*
     &  DBLE(B0P(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,1)*
     &  (DBLE(B0P(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+
     &  DBLE(B0P(M42,Mah2(I6G3),MassSu2(I6G4),Q2))))/2.+dMSf4(1,2,2)
       dMSf4(2,2,2)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,2)*
     &  DBLE(B0P(M42,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(2,2,2)
       enddo
       enddo


       Do  I5G3= ineu,ineue
       Do  I6G3= 1,2
       dmt4(2)=dmt4(2)+(-(M1*DBLE(B1P(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSL(I5G3,I6G3)*CF11FSLC(I5G3,I6G3)+CF11FSR(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3)))+DBLE(B0P(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSLC(I5G3,I6G3)*CF11FSR(I5G3,I6G3)+CF11FSL(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3))*MNeu(I5G3))/2d0
       enddo
       enddo

#ifdef checkformula
       print*, "dmt4(2)",dmt4(2)*fac,Mah2(3),Mah2(4),MHp2, MNeu2(2),mcha2
	print*, "Mah2(3),Mah2(4)",Mah2(3),Mah2(4)
	print*, "m1, SB, sqrt(vv2),Mneu2(2), MSQ2",m1, SB, sqrt(vv2),Mneu2(2),MSQ2
	print*, "DBLE(B1P(M12,MCha2,MSQ2,Q2))",DBLE(B1P(M12,MCha2,MSQ2,Q2))
	print*, "DBLE(B1P(M12,0D0,MHp2,Q2))",DBLE(B1P(M12,0D0,MHp2,Q2))
	print*, "DBLE(B1P(M12,0D0,0D0,Q2))",DBLE(B1P(M12,0D0,0D0,Q2))
	print*, "DBLE(B1P(M12,M12,Mah2(3),Q2))",DBLE(B1P(M12,M12,Mah2(3),Q2))
	print*, "DBLE(B1P(M12,M12,Mah2(4),Q2)) ",DBLE(B1P(M12,M12,Mah2(4),Q2)) 
	print*,DBLE(B1P(M12,Mneu2(2),M22,Q2))+DBLE(B1P(M12,Mneu2(2),M42,Q2)) 
       print*, -m1**3/16D0/pi**2/SB**2/vv2*(DBLE(B1P(M12,MCha2,MSQ2,Q2))+
     &   CB**2*DBLE(B1P(M12,0D0,MHp2,Q2))+SB**2*DBLE(B1P(M12,0D0,0D0,Q2)) 
     &    +2*SB**2*DBLE(B1P(M12,M12,Mah2(3),Q2))  
     &    +2*CB**2*DBLE(B1P(M12,M12,Mah2(4),Q2)) 
     &    +0*(DBLE(B1P(M12,Mneu2(2),M22,Q2))+DBLE(B1P(M12,Mneu2(2),M42,Q2))) 
     &   ), -m1**3/16D0/pi**2/SB**2/vv2,DBLE(B1P(M12,MCha2,MSQ2,Q2)),
     &   CB**2*DBLE(B1P(M12,0D0,MHp2,Q2)),SB**2*DBLE(B1P(M12,0D0,0D0,Q2)) 
     &    ,2*SB**2*DBLE(B1P(M12,M12,Mah2(3),Q2))  
     &    ,2*CB**2,DBLE(B1P(M12,M12,Mah2(4),Q2)) 
     &    ,(DBLE(B1P(M12,Mneu2(2),M22,Q2))+DBLE(B1P(M12,Mneu2(2),M42,Q2))) 
     &   
       stop
#endif

       Do  I5G3= ineu,ineue
       dMSf4(1,1,2)=dMSf4(1,1,2)+(-(a0t1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,1))-A0TNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,1)
     &  -a0t1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-A0TNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-DBLE(B0P(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,1)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,1)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,1)*MNeu(I5G3)+CF11FSRC(I5G3,1)*(M12-M22+
     &  MNeu2(I5G3)))))
       dMSf4(1,2,2)=dMSf4(1,2,2)+(-(a0t1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,2))-A0TNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,2)
     &  -a0t1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-A0TNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-(DBLE(B0P(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M22+
     &  MNeu2(I5G3)))))/2.-(DBLE(B0P(M42,M12,MNeu2(I5G3),Q2))*
     &  (CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))/2.)
       dMSf4(2,2,2)=dMSf4(2,2,2)+(-(A0T(M12,Q2)*(CF11FSL(I5G3,2)*
     &  CF11FSLC(I5G3,2)+CF11FSR(I5G3,2)*CF11FSRC(I5G3,2)))-
     &  A0T(MNeu2(I5G3),Q2)*(CF11FSL(I5G3,2)*CF11FSLC(I5G3,2)+
     &  CF11FSR(I5G3,2)*CF11FSRC(I5G3,2))-DBLE(B0P(M42,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,2)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,2)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))
       enddo
       endif

       dmt4(1:3)=dmt4(1:3)* fac 


       dMSf4(2,1,1:3)=DCONJG(dMSf4(1,2,1:3))
       dMSf4(1:2,1:2,1:3)=dMSf4(1:2,1:2,1:3)*fac

       do i=1,2
        dMSQ4(i)=-2*M1*dMT4(i)+DCONJG(ZU(1,1))*dMSf4(1,1,i)*ZU(1,1)+
     &  DCONJG(ZU(2,1))*dMSf4(2,1,i)*ZU(1,1)+DCONJG(ZU(1,1))*
     &  dMSf4(1,2,i)*ZU(2,1)+DCONJG(ZU(2,1))*dMSf4(2,2,i)*ZU(2,1)
       enddo
 
       dAT4(1)=(-(dTanB4*M1)*(DCONJG(MUE))+M1*TB*DCONJG(dMUE4)
     &  -ATT*expETA*TB**2*dMT4(1)+(TB*DCONJG(MUE)*dMT4(1))
     &  +TB**2*DCONJG(ZU(1,2))*dMSf4(1,1,1)*ZU(1,1)+TB**2*DCONJG(ZU(2,2))*
     &  dMSf4(2,1,1)*ZU(1,1)+TB**2*DCONJG(ZU(1,2))*dMSf4(1,2,1)*ZU(2,1)
     &  +TB**2*DCONJG(ZU(2,2))*dMSf4(2,2,1)*ZU(2,1))/(expETA*M1*TB**2)
       do i=2,2
       dAT4(i)=(-(ATT*expETA*TB**2*dMT4(i))+(TB*DCONJG(MUE)*dMT4(i))
     &  +TB**2*DCONJG(ZU(1,2))*dMSf4(1,1,i)*ZU(1,1)+TB*
     &  *2*DCONJG(ZU(2,2))*dMSf4(2,1,i)*ZU(1,1)+TB**2*DCONJG(ZU(1,2))*
     &  dMSf4(1,2,i)*ZU(2,1)+TB**2*DCONJG(ZU(2,2))*dMSf4(2,2,i)*ZU(2,1)
     &  )/(expETA*M1*TB**2)
       enddo 


       dat4fin=dAT4(2)
       dmsq4fin=dMSQ4(2)
       dmsur4fin=-2*M1*dmt4(2) + dMsf4(1,1,2)*ZU(1,2)*ZUC(1,2) + 
     &  dMsf4(1,2,2)*ZU(2,2)*ZUC(1,2) + dMsf4(2,1,2)*ZU(1,2)*ZUC(2,2) +
     &  dMsf4(2,2,2)*ZU(2,2)*ZUC(2,2)
      
      end subroutine CT1loopMT4
c*
c----------------------------------------------------------------------
c*
      subroutine couplingsEW(Q2N,MTN,MHpm2N,vvN,vSN,TBN,
     -   MW2N,MZ2N,MST1,MST2,ZUN,ATN,MUEN,expETAN,expvSN)
      implicit none
      integer i,j,l,k,l1
      double precision Q2N,MTN,MSt1,MSt2,MHpm2N,vvN,vSN,TBN,MW2N,MZ2N,
     .     S2B,C2B

      double complex ATN,ZUN(2,2),MUEN,expETAN,expvSN

#include "TLMT4.h"

      pi = 2d0*Asin(1d0)
      IC=dcmplx(0,1D0)
      sqrt2= sqrt(2D0)
      ZU(1:2,1:2)=ZUN(1:2,1:2)
      
      Q2=Q2N;vs=vSN;vS2=vS**2
      vv=vvN;vv2= vv**2;MW2=MW2N;MZ2=MZ2N
      CW2=MW2/MZ2; SW2=1-CW2;SW=sqrt(SW2);CW=Sqrt(CW2)     
      TB=TBN;      TB2= TB**2;	       CB2 = 1/(1 + TB2)
      SB2 = CB2*TB2;	       CB = sqrt(CB2);	SB = CB*TB
      S2B = 2*TB*CB2;          C2B= (CB - SB)*(CB + SB)
      ZP(1,1)=-CB; ZP(1,2)=SB; ZP(2,1)=SB;  ZP(2,2)= CB
      expETA=expETAN;expvS=expvSN
      MUE=MUEN; MUEC=DCONJG(MUE)

      ATT=ATN
      MHp2=MHpm2N
      MHp2N=MHpm2N

      mah(3)=0d0;mah(4)=sqrt(MHp2N);mah(5)=mah(4);mah(6)=0d0
      ZH(1:6,1:6)=0d0
      ZH(3,1)=CB; ZH(3,2)=SB
      ZH(4,1)=SB; ZH(4,2)=-CB
      ZH(5,4)=SB; ZH(5,5)=CB
      ZH(6,4)=CB; ZH(6,5)=-SB   

      ihiggs=3;ineu=2
      ihiggse=6;ineue=3
      la=0D0;kappa=0D0
      abslambda = abs(la); abskappa=abs(kappa); 
      la2=abslambda**2;laC=DCONJG(la);kappaC=DCONJG(kappa)

      ZNeu(1:3,1:3)=0d0
      MNeu(1)=0D0;MNeu(2)=abs(MUE);MNeu(3)=abs(MUE)
      ZNeu(2,1)=-Sqrt((MUE/abs(MUE)))/sqrt2
      ZNeu(2,2)=-ZNeu(2,1)
      ZNeu(3,1)=IC*ZNeu(2,1)
      ZNeu(3,2)=-IC*ZNeu(2,2)

      Do i=ineu,ineue 
        MNeu2(i)=MNeu(i)**2; 
        ZNeuC(i,1:3)=DCONJG(ZNeu(i,1:3))
      enddo
      ZUC(1:2,1:2)=DCONJG(ZU(1:2,1:2))
      M1=MTN;   M2=MSt1;  M4=MSt2
      M12=M1**2;M22=M2**2;M42=M4**2
      MassSu(1)=M2; MassSu(2)= M4
      MassSu2(1)=M22; MassSu2(2)= M42
      logm12=Log(M12/Q2); logM22=Log(M22/Q2);logM42=log(M42/Q2)

      MCha=Abs(MUE); MCha2=MCha**2;VCha=MUE/MCha;VChaC=DCONJG(VCha)
      ATC= DCONJG(ATT)

      MSQ2=-M12+M22*ZU(1,1)*ZUC(1,1)+M42*ZU(2,1)*ZUC(2,1)
      Mah2(1:6)= mah(1:6)**2

      do k=1,2
       do j=1,2
         do i=1,6
         CSSS(i,j,k)=ZH(i,5)*(ATT*expETA*IC*ZU(k,2)*ZUC(j,1) - 
     -   (IC*ATC*ZU(k,1)*ZUC(j,2))/expETA) + 
     -   vv*ZH(i,3)*(-((CB*laC*ZU(k,2)*ZUC(j,1))/(expVS*sqrt2)) -
     -   (CB*expVS*la*ZU(k,1)*ZUC(j,2))/sqrt2) + 
     -   vv*ZH(i,6)*((CB*IC*laC*ZU(k,2)*ZUC(j,1))/(expVS*sqrt2) - 
     -   (IC*CB*expVS*la*ZU(k,1)*ZUC(j,2))/sqrt2) + 
     -   ZH(i,1)*(-((MUEC*ZU(k,2)*ZUC(j,1))) - 
     -   (MUE*ZU(k,1)*ZUC(j,2))) + 
     -   ZH(i,4)*((IC*MUEC*ZU(k,2)*ZUC(j,1)) - 
     -   (IC*MUE*ZU(k,1)*ZUC(j,2))) + 
     -   ZH(i,2)*(ATT*expETA*ZU(k,2)*ZUC(j,1) +(ATC*ZU(k,1)*ZUC(j,2))
     -   /expETA + M1*(2*ZU(k,1)*ZUC(j,1) + 2*ZU(k,2)*ZUC(j,2)))
         CSSS(i,j,k)=CSSS(i,j,k)*M1/vv/SB
         enddo
        enddo
      enddo

      do i =1,6
        hy(i)=(M1*(ZH(i,2) + IC*ZH(i,5)))/(SB*vv)
        hyc(i)=(M1*(ZH(i,2) - IC*ZH(i,5)))/(SB*vv)
      enddo

      do k=1,2
        do l1=1,2
         do j=1,6
          do i=1,6
        CSSSS(i,j,l1,k)=ZH(i,3)*(-((laC*(ZH(j,1) - IC*ZH(j,4))*
     -  ZU(k,2)*ZUC(l1,1))/(expVS*sqrt2)) - (expVS*la*(ZH(j,1) + 
     -  IC*ZH(j,4))*ZU(k,1)*ZUC(l1,2))/sqrt2) + ZH(i,6)*
     -  (-((laC*((-IC)*ZH(j,1) - ZH(j,4))*ZU(k,2)*ZUC(l1,1))/
     -  (expVS*sqrt2)) - (IC*expVS*la*(ZH(j,1) + IC*ZH(j,4))*ZU(k,1)*
     -  ZUC(l1,2))/sqrt2) + ZH(i,1)*(-((laC*(ZH(j,3) 
     -  - IC*ZH(j,6))*ZU(k,2)*ZUC(l1,1))/(expVS*sqrt2)) - 
     -  (expVS*la*(ZH(j,3) + IC*ZH(j,6))*ZU(k,1)*ZUC(l1,2))/sqrt2) + 
     -  ZH(i,4)*((IC*laC*(ZH(j,3) - IC*ZH(j,6))*ZU(k,2)*ZUC(l1,1))/
     -  (expVS*sqrt2) - (IC*expVS*la*(ZH(j,3) + IC*ZH(j,6))*ZU(k,1)*
     -  ZUC(l1,2))/sqrt2) + M1*ZH(i,2)*((2*ZH(j,2)*ZU(k,1)*ZUC(l1,1))
     -  /(SB*vv) + (2*ZH(j,2)*ZU(k,2)*ZUC(l1,2))/(SB*vv)) + 
     -  M1*ZH(i,5)*((2*ZH(j,5)*ZU(k,1)*ZUC(l1,1))/(SB*vv) + 
     -  (2*ZH(j,5)*ZU(k,2)*ZUC(l1,2))/(SB*vv))
        CSSSS(i,j,l1,k)=CSSSS(i,j,l1,k)*M1/vv/SB
            enddo   
          enddo
        enddo
       enddo 


      do j=1,2
       do i=1,6
       CS1S3SS(i,2,j)=-((M1*(2*CB*M1*(ZH(i,2)+ IC*ZH(i,5))*ZUC(j,1)+ 
     -  expvS*la*SB2*sqrt2*vv*(ZH(i,3) +IC*ZH(i,6))*ZUC(j,2)))/
     -  (SB2*sqrt2*vv2))
       CS1S3SSC(i,2,j)=DCONJG(CS1S3SS(i,2,j))
       CS1S3SS(i,1,j)=-((M12*sqrt2*(ZH(i,2)+Ic*ZH(i,5))*ZUC(j,1))/(SB*
     - vv2))+(CB*expvS*la*M1*(ZH(i,3) + Ic*ZH(i,6))*ZUC(j,2))/(SB*vv)
       CS1S3SSC(i,1,j)=DCONJG(CS1S3SS(i,1,j))
       enddo   
       CS3S13S(2,j)=-((M12*sqrt2*ZUC(j,1))/(TB*vv)) -(ATC*M1*sqrt2*
     .  ZUC(j,2))/(expETA*TB*vv) - (MUE*sqrt2*M1*ZUC(j,2))/vv

       CS3SSS13(1,j)=(2*M12*ZU(j,2)*ZUC(1,2))/(TB2*vv**2)
       CS3SSS13(2,j)=(2*M12*ZU(j,2)*ZUC(2,2))/(TB2*vv**2)
       CS3S13SC(2,j)=DCONJG(CS3S13S(2,j))
c
       CS3S13S(1,j)=-((expETA*M12*SB*sqrt2*ZUC(j,1)+ATC*M1*SB*sqrt2* 
     - ZUC(j,2) -CB*expETA*MUE*sqrt2*M1*ZUC(j,2))/(expETA*SB*vv))
       CS3S13SC(1,j)=DCONJG(CS3S13S(1,j))

       CS3S13S14(j)=-((M12*sqrt2*ZUC(j,1))/(TB*vv)) -(ATC*M1*sqrt2*
     .  ZUC(j,2))/(expETA*TB*vv) - (MUE*sqrt2*M1*ZUC(j,2))/vv

        CS3S14S13(j)=-((M12*sqrt2*ZU(j,1))/(TB*vv)) -(ATT*expETA*M1* 
     .  sqrt2*ZU(j,2))/(TB*vv) - (MUEC*sqrt2*M1*ZU(j,2))/(vv)

      enddo
      CS3SSS14 = (2*M12)/(TB2*vv**2)

      CS3S3S14S14(2,2) = CS3SSS14
      CS3S3S14S14(2,1) = (2*CB*M12)/(SB*vv**2)
      CS3S3S14S14(1,2) =       CS3S3S14S14(2,1)
      CS3S3S14S14(1,1) = (2*M12)/vv**2

      CTBS3R   =-(sqrt2/(sqrt(expETA)*vv))* M1/TB
      CTBS3(2)= -(sqrt2/(sqrt(expETA)*vv))* M1/TB
      CTBS3C(2)= DCONJG(CTBS3(2))
      CTBS3(1)= -sqrt2*M1/vv/sqrt(expETA)
      CTBS3C(1)=DCONJG(CTBS3(1))

      do i=1,3
       do j=1,2
       CF11FSR(i,j)=(Sqrt(expETA)*M1*sqrt2*ZNeu(i,2)*
     .     ZU(j,1))/(SB*vv)
       CF11FSL(i,j)=(M1*sqrt2*ZNeuC(i,2)*ZU(j,2))
     .     /(Sqrt(expETA)*SB*vv)
       CF11FSRC(i,j)=DCONJG(CF11FSR(i,j))
       CF11FSLC(i,j)=DCONJG(CF11FSL(i,j))
       enddo


      enddo
      do l=1,2
       do k=1,2
         do j=1,2
           do i=1,2
            CS13S13S13S13MT4(i,j,k,l)=(2*M12*(ZU(k,2)*ZU(l,1)*ZUC(i,2)
     -      *ZUC(j,1)+ZU(k,1)*ZU(l,2)*ZUC(i,1)*ZUC(j,2)))/(SB2*vv2)

           enddo
         enddo
            CS14S13S14S13MT4(k,l)=(2*M12*ZU(l,2)*ZUC(k,2))/(SB2*vv2)
       enddo
      enddo

      CF12F3S= -((Sqrt(expETA)*M1*sqrt2)/(SB*vv))
      CF12F3SC= DCONJG(CF12F3S)
      do i=1,2
      CF12F4S(i) =-((M1*sqrt2*ZU(i,2))/(SB*vv))
      CF12F4SC(i)= DCONJG(CF12F4S(i))
      enddo

      do k=1,2
        do i=1,2
         do l=1,2
          do j=1,2
          CS3S3S13S13(k,i,l,j)=(2*M12*ZP(k,2)*ZP(i,2)
     &    *ZU(j,2)*ZUC(l,2))/(SB2*vv2)
          enddo
        enddo
       enddo
      enddo

      end subroutine couplingsEW

c**********************************************************************
c (s)top sector renormalization
c**********************************************************************

      subroutine NHiggs_TBN(M12,M22,M42,MSQ2,Mah2,a0t1,a0t2,a0t4,a0t5
     .     ,A0TNH,aet1,aet2,aet4,aet5,AETNH,Q2)
      implicit none
      integer i
      double precision M12,M22,M42,MSQ2,Mah2(6),a0t1,a0t2,a0t4,a0t5
     .     ,A0TNH(6),aet1,aet2,aet4,aet5,AETNH(6),Q2,M14,mhas
      double precision FD111NH5(6,2),FD111NH55(6),FD111NHM1(6,4),
     .     FD111NHM10(6,4),FD211NHM1(6,4),aK0T(6),aK0T0(6)
      double complex K0T0
      common/MT4NHiggTB/FD111NH5,FD111NH55,FD111NHM1,
     .     FD111NHM10,FD211NHM1,aK0T,aK0T0
      M14=M12**2

      do i=1,6
      mhas=Mah2(i)
      FD111NH5(i, 1) =DBLE( K0T0(M22,mhas,MSQ2,Q2))
      FD111NH5(i, 2) =DBLE( K0T0(M42,mhas,MSQ2,Q2))
      FD111NH55(i)=DBLE( K0T0(MSQ2,mhas,MSQ2,Q2))
      aK0T(i)=K0T0(mhas,M12,M12,Q2)
      aK0T0(i)=K0T0(0D0,mhas,M12,Q2)
      FD211NHM1(i,2)=-((-2*a0t1**2-4*a0t1*M12-4*aet1*M12-4*M14+2*a0t1*M
     & has+2*aet1*Mhas+Mhas**2+2*(a0t1-3*M12+Mhas)*A0TNH(i)+2*M12*AETNH
     &  (i)-6*M12*aK0T(i)+2*Mhas*aK0T(i))/(8*M12-2*Mhas))

      FD211NHM1(i,3)=(-a0t1**2+6*a0t1*M12-2*aet1*M12+2*M14-a0t1*Mhas+ae
     &  t1*Mhas+M12*Mhas+(a0t1+M12)*A0TNH(i)+M12*AETNH(i)+5*M12*aK0T(i
     &  )-Mhas*aK0T(i))/(4*M12-Mhas)

      FD211NHM1(i,4)=(6*a0t1**2*M12+4*a0t1*M14+12*aet1*M14+4*M14*M12-a0t
     &1**2*Mhas-2*a0t1*M12*Mhas-12*aet1*M12*Mhas+2*aet1*Mhas**2-M12*
     &Mhas**2-2*(5*a0t1*M12-3*M14-a0t1*Mhas+M12*Mhas)*A0TNH(i)+2*M12*(-
     &5*M12+Mhas)*AETNH(i)+14*M14*aK0T(i)-8*M12*Mhas*aK0T(i)+Mhas**2*a
     &  K0T(i))/(8*M12-2*Mhas)

      FD111NHM1(i,2)=a0t1**2/2.+aet1*(M12-Mhas)-a0t1*A0TNH(i)-M12*AETNH
     & (i)+(M12-Mhas/2.)*aK0T(i)

      FD111NHM1(i,3)=-(aet1*Mhas)-a0t1*A0TNH(i)-M12*AETNH(i)+M12*aK0T(
     &  i)

      FD111NHM1(i,4)=(M12*(a0t1**2+2*aet1*M12-4*aet1*Mhas-4*a0t1*A0TNH(
     &  i)-4*M12*AETNH(i)+2*M12*aK0T(i)-Mhas*aK0T(i)))/2.

      FD111NHM10(i,2)=(-(aet1*Mhas)-a0t1*A0TNH(i)-M12*AETNH(i)+M12*aK0T
     &  0(i)-Mhas*aK0T0(i))/2.
      FD111NHM10(i,3)=-(aet1*Mhas) - a0t1*A0TNH(i) - M12*AETNH(i)
      FD111NHM10(i,4)=-(M12*(aet1*Mhas + a0t1*A0TNH(i) + M12*AETNH(i)))

      enddo
      end subroutine NHiggs_TBN

c**********************************************************************

      subroutine NHiggs_TB(M12,M22,aet1,aet2,a0t1,a0t2,Q2,coef)
      implicit none
      double precision M12,M22,M14,M24,aet1,aet2,a0t1,a0t2,Q2
     .      ,lambda,lambda2,M16,M26
      double precision I0N,aI0N,coef(5)
      if (M22.gt.M12)then
      aI0N=I0N(M12,M12,M22,Q2)
      else
      aI0N=I0N(M22,M12,M12,Q2)
      endif
      lambda=M22-4*M12; lambda2= lambda**2
      M14=M12**2; M24=M22**2;M16=M12*M14;M26=M22*M24
       coef(1)=(a0t1**2*(2*M12-M22)-a0t1*(-2*a0t2*M12+28*M14-12*M
     &  12*M22+M24)+M12*(2*aI0N*M12+20*M14+a0t2*(6*M12-M22)-6*M12*M22+M
     &  24))/lambda2
       coef(2)=(2*(2*a0t1**2*M12-12*M16-4*M14*M22+aI0N*M12*(-6*M1
     &  2+M22)+2*a0t2*M12*(-5*M12+M22)+aet1*(-4*M12+M22)**2+M12*M24+a0t
     &  1*(a0t2*(-6*M12+M22)-2*(14*M14-7*M12*M22+M24))))/lambda2
       coef(3)=(2*a0t1**2*(7*M12-2*M22)+3*aet1*(-4*M12+M22)**2+2*
     &  a0t2*M12*(-3*M12+M22)+aI0N*M12*(-10*M12+3*M22)+a0t1*(-100*M14+4
     &  2*M12*M22+a0t2*(-10*M12+3*M22)-4*M24)-4*(M16+3*M14*M22-M12*M24)
     &  )/lambda2
       coef(4)=-((40*M16+a0t1**2*(4*M12-2*M22)+4*M14*M22-2*(2*aet
     &  1+aet2)*(-4*M12+M22)**2-6*M12*M24+2*a0t2*(22*M14-9*M12*M22+M24)
     &  +2*aI0N*(18*M14-8*M12*M22+M24)+a0t1*(4*a0t2*M12+72*M14-40*M12*M
     &  22+6*M24)+M26)/lambda2)
       coef(5)=(-12*M16+a0t1**2*(18*M12-5*M22)-14*M14*M22+(6*aet1
     &  +aet2)*(-4*M12+M22)**2+a0t2*(-10*M14+3*M12*M22)+a0t1*(-124*M14+
     &  52*M12*M22+a0t2*(-14*M12+4*M22)-5*M24)+5*M12*M24-aI0N*(30*M14-1
     &  2*M12*M22+M24))/lambda2
      end subroutine NHiggs_TB

c**********************************************************************
      subroutine ChaFD(M32,M22,M42,a0t3,aet3,a0t2,aet2,a0t4,aet4,Q2,coefa)
      implicit none
      double precision M42,M22,M24,M44,aet4,aet2,a0t4,a0t2,Q2
     .      ,M22m42s,M32,a0t3,aet3,M34
      double precision I0N,IT1,IT2,coefa(10)
      common/I0NChaFD/IT1,IT2
      if(M32.gt.M22)then
       IT1= I0N(0D0,M22,M32,Q2)
      else
       IT1= I0N(0D0,M32,M22,Q2)      
      endif
      if(M32.gt.M42)then
       IT2= I0N(0D0,M42,M32,Q2)
      else
       IT2= I0N(0D0,M32,M42,Q2)      
      endif

      M34=M32**2
      M24=M22**2;M44=M42**2;M22m42s=(M22-M42)**2
       coefa(1)=(3*(a0t2*a0t3-a0t3*a0t4-aet2*M22+IT1*M22+2*aet2*M32-
     &  2*aet4*M32-IT1*M32+IT2*M32+aet4*M42-IT2*M42))/16./(M22-M42)
       coefa(2)=(3*(-2*aet2*(M22-M32)+2*a0t2*(a0t3+M22-M32)+M22*(2
     &  *IT1+M22+M32)))/(16.*M22)
       coefa(3)=(3*(2*a0t3*a0t4-2*a0t4*M32+2*aet4*M32+2*a0t4*M42-2
     &  *aet4*M42+2*IT2*M42+M32*M42+M42**2))/(16.*M42)
       coefa(4)=(3*(-(aet2*M24)+IT1*M24+M22*M24+a0t3*(a0t4*M22-2*M
     &  22**2+2*M24)+2*aet4*M22*M32+IT1*M22*M32-IT2*M22*M32+M24*M32+2*a
     &  et2*M22*M42-aet4*M22*M42-2*IT1*M22*M42+IT2*M22*M42-M22**2*M42-2
     &  *aet2*M32*M42-M22*M32*M42+a0t2*(a0t3*(M22-2*M42)+2*(M24-M22*M32
     &  -M22*M42+M32*M42))))/(16.*M22*M22m42s)
       coefa(5)=(3*(-2*aet4*M22*M32-aet2*M22*M42+2*aet4*M22*M42+IT
     &  1*M22*M42-2*IT2*M22*M42+2*aet2*M32*M42-IT1*M32*M42+IT2*M32*M42-
     &  M22*M32*M42-M22*M42**2-aet4*M44+IT2*M44+M32*M44+M42*M44+2*a0t4*
     &  (M22*M32-M22*M42-M32*M42+M44)+a0t3*(a0t2*M42-2*M42**2+a0t4*(-2*
     &  M22+M42)+2*M44)))/(16.*M22m42s*M42)
       coefa(6)=(3*(M22*(-2*aet2*M22+2*IT1*M22+M22**2+2*a0t3*M32+2
     &  *aet2*M32+5*M22*M32-2*M34)+2*a0t2*(a0t3*M22+3*M22**2-3*M22*M32+
     &  M34)))/(16.*M24*(M22-M32))
       coefa(7)=-(3*(M42*(2*a0t3*M32+2*aet4*M32-2*M34-2*aet4*M42+2*
     &  IT2*M42+5*M32*M42+M42**2)+2*a0t4*(M34+M42*(a0t3-3*M32+3*M42))))
     &  /(16.*(M32-M42)*M44)



      end

c**********************************************************************
c this sub calculated the following coeficient
      subroutine CHiggs_TB(M12,M22,aet1,aet2,a0t1,a0t2,Q2,coef)
      implicit none
      double precision M12,M22,M14,M24,aet1,aet2,a0t1,a0t2,Q2
     .      ,lambda,lo,pi2,M16,M26
      double precision I0N,aI0N,coef(5)
      if (M22.gt.M12)then
      aI0N=I0N(0D0,M12,M22,Q2)
      else
      aI0N=I0N(0D0,M22,M12,Q2)
      endif
      lambda=M12-M22
      M14=M12**2; M24=M22**2

       coef(1)=-(M14+3*M12*M22+2*(a0t2*M12+a0t1*(a0t2+3*M12-2*M22
     &  )+aet1*(-M12+M22)+M12*aI0N))/(2.*lambda)

       coef(2)=(-2*M14+2*(2*aet1+aet2)*lambda-2*a0t1*(a0
     &  t2+4*M12-3*M22)-3*M12*M22+2*a0t2*(-2*M12+M22)+M24+2*(-2*M12+M22
     &  )*aI0N)/(2.*lambda)

       pi2= 3.1415926535897932384626433832795029D0
       pi2=pi2**2
       lo=Log(M12/Q2)
       coef(3)=-3*a0t1+aet1+(M12*(12+Pi2+3*(-4+Lo)*Lo))/6.
       coef(4)=-4*a0t1+2*aet1+(M12*(12+Pi2+3*(-4+Lo)*Lo))/3.

      end
c**********************************************************************
      subroutine CHiggsN(M12,MHp2,M22,M42,MSQ2,aet1,AETHp,aet2,
     &   aet4,aet5,a0t1,A0THp,a0t2,a0t4,a0t5,Q2)
      implicit none
      integer i,j
      double precision M12,MHp2,M22,M42,MSQ2,aet1,AETHp,aet2,
     &   aet4,aet5,a0t1,A0THp,a0t2,a0t4,a0t5,Q2,K0T0N,Mhas,aeth,a0th,
     &   LambdaN, alam1, alam2,coef1(10),coef2(10)
      double precision FD111CHMS(2,2,2),FD111CHMSQ(2),FD111CH5(2,2),
     .   FD211CHMSQ(10,2,2),FD211CH0M1(10,2)  

      common/MT4CHiggs/FD111CHMS,FD111CHMSQ,FD111CH5,FD211CHMSQ,FD211CH0M1
      do i=1,2
       if(i.eq.1)then
        mhas=0; a0th=0;aeth=0
       else
        mhas=MHp2; a0th=A0THp; aeth=AETHp
       endif
      FD111CHMS(i,1,1) = K0T0N(M22,M22,mhas,aet2,aet2,aeth,Q2)
      FD111CHMS(i,1,2) = K0T0N(M22,M42,mhas,aet2,aet4,aeth,Q2)
      FD111CHMS(i,2,2) = K0T0N(M42,M42,mhas,aet4,aet4,aeth,Q2)
      FD111CHMSQ(i)    = K0T0N(MSQ2,MSQ2,mhas,aet5,aet5,aeth,Q2)
      FD111CH5(i,1)    = K0T0N(M22,MSQ2,mhas,aet2,aet5,aeth,Q2)
      FD111CH5(i,2)    = K0T0N(M42,MSQ2,mhas,aet4,aet5,aeth,Q2)
       call FD211C(MSQ2,Mhas,M22,a0t5,a0th,a0t2,aet5,aeth,aet2,Q2,
     &      coef1,0)  
       call FD211C(MSQ2,Mhas,M42,a0t5,a0th,a0t4,aet5,aeth,aet4,Q2,
     &      coef2,0) 
       do j=1,10
        FD211CHMSQ(j,i,1)=coef1(j)
        FD211CHMSQ(j,i,2)=coef2(j)
       enddo
      enddo
       

      end
c**********************************************************************
      subroutine NHiggs_TR(M12,M22,M32,aet1,aet2,aet3,a0t1,a0t2,a0t3,
     .    Q2,FDMH)
      implicit none
      double precision M12,M22,M32,aet2,aet3,a0t2,a0t3,Q2,M24,M34,M14,
     .    M1222,M1232,a0t1,aet1,M2232,M2234,M1422,M26,abr1,abr2,abr3,
     .    abr4,M1432,M2432,M36,M1224,M16,M18,M1226,M28,M38,M1424,
     .    M1434,M1622,M1234
      double precision FDMH(4,2,2),lambdaN,alambda,alambda2
      double complex K0T0
c 111
      if (M12.lt.1D-8) M12=0D0
      FDMH(1,1,1) =K0T0(M22,M22,M12,Q2)
      FDMH(1,1,2) =K0T0(M22,M32,M12,Q2) 
      FDMH(1,2,1) =FDMH(1,1,2);   FDMH(1,2,2) =K0T0(M32,M32,M12,Q2) 
c 211
      M14=M12**2;M24=M22**2;M34=M32**2; M1222=M12*M22
      M1232=M12*M32;M1422=M14*M22;M2232=M22*M32;M2234=M22*M34
      M26=M24*M22;M1432=M14*M32;M2432=M24*M32;M36=M34*M32;
      M1224=M12*M24;M16=M14*M12;M18=M14*M14;M1226=M12*M26
      M28=M24**2;M38=M34**2;M1424=M14*M24;M1622=M16*M22
      M1434=M14*M34;M1234=M12*M34
      abr1=M12 + M22 - M32; abr2=M12 + M22 + M32;abr3=M12-M22-M32
      abr4=M12 - M22 + M32
      alambda=LambdaN(M12,M22,M32); alambda2=alambda**2
      FDMH(2,1,1) =(-a0t2**2 + aet2*M12 - a0t2*(M12 - 6*M22) + aet1*M22 
     -  -2*aet2*M22 +M12*M22 + a0t1*(a0t2 + M22) + 2*M24 + 
     -  M22*FDMH(1,1,1))/(-(M12*M22) + 4*M24)

      FDMH(2,2,2) =(-a0t3**2 + aet3*M12 - a0t3*(M12 - 6*M32) + aet1*M32 
     - - 2*aet3*M32 +M12*M32 + a0t1*(a0t3 + M32) + 2*M34 + 
     - M32*FDMH(1,2,2))/(-(M12*M32) + 4*M34)

      FDMH(2,1,2) =(a0t1*(-2*a0t3 - abr4 - (a0t2*abr3)/M22) - 
     -    (a0t3*(M1222 + M2232 - M24))/M22 - 
     -    (aet1*(M1222 + M2232 - M24))/M22 - 
     -    (aet3*(M1222 + M2232 - M24))/M22 - 
     -    (M1422 + M2234 - M26 + 2*M1222*M32)/M22 + 
     -    (aet2*(M1222 + 2*M1232 - M14 + M2232 - M34))/M22 + 
     -    a0t2*((a0t3*abr1)/M22 + 
     -       (-3*M1222 - 2*M1232 + M14 - 3*M2232 + 2*M24 + M34)/M22) - 
     -    ((M1222 + M2232 - M24)*FDMH(1,1,2))/M22)/alambda

      FDMH(2,2,1) =(a0t1*(-2*a0t2 - abr1 - (a0t3*abr3)/M32) + 
     -    a0t2*(-abr1 + (a0t3*abr4)/M32) + 
     -    (aet3*(2*M1222 + M1232 - M14 + M2232 - M24))/M32 - 
     -    ((aet1+aet2+FDMH(1,1,2))*(M1232 + M2232 - M34))/M32  + 
     -    (a0t3*(-2*M1222 - 3*M1232 + M14 - 3*M2232 + M24 + 2*M34))/M32 - 
     -    (M1432 + M2432 + 2*M1222*M32 - M36)/M32)/alambda

c311
      if(M12.eq.0D0)then
      FDMH(3,1,1) =(a0t2 - M22)/M24
      FDMH(3,2,2) =(a0t3 - M32)/M34

      FDMH(4,1,1) =1D0/M22
      FDMH(4,2,2) =1D0/M32
      else
      FDMH(3,1,1) =(2*a0t2**2*(M12 - 2*M22)*M22 + 
     -    a0t2*(24*M1224 - 10*M1422 + M16 - 4*a0t1*M24 - 8*M26) - 
     -    M22*(20*M1224 - 8*M1422 + M16 + a0t1*(6*M1222 - M14 - 4*M24) + 
     -       4*aet1*M24 + 8*aet2*M24 + 8*M26 + 4*M24*FDMH(1,1,1)))/
     -  (2.*M24*M12*(M12 - 4*M22)**2)
      
      FDMH(3,2,2) =(2*a0t3**2*(M12 - 2*M32)*M32 + 
     -    a0t3*(24*M1234 - 10*M1432 + M16 - 4*a0t1*M34 - 8*M32*M34) - 
     -    M32*(20*M1234 - 8*M1432 + M16 + a0t1*(6*M1232 - M14 - 4*M34) + 
     -       4*aet1*M34 + 8*aet3*M34 + 8*M32*M34 + 4*M34*FDMH(1,2,2)))/
     -  (2.*(M12 - 4*M32)**2*M12*M34)

      FDMH(4,1,1) =(a0t2**2*(M12 - 2*M22)**2 + 
     -    2*a0t2*M22*(-(a0t1*M12) + 2*M1222 - M14 + 2*a0t1*M22 + 4*M24) - 
     -    2*M24*(2*aet2*M12 + M14 + aet1*(M12 - 2*M22) + 2*a0t1*M22 - 
     -       4*aet2*M22 - 4*M24 + M12*FDMH(1,1,1) - 2*M22*FDMH(1,1,1)))/
     -  ((M12 - 4*M22)**2*M12*M24)

      FDMH(4,2,2) =(a0t3**2*(M12 - 2*M32)**2 + 
     -    2*a0t3*M32*(-(a0t1*M12) + 2*M1232 - M14 + 2*a0t1*M32 + 4*M34) - 
     -    2*M34*(2*aet3*M12 + M14 + aet1*(M12 - 2*M32) + 2*a0t1*M32 - 
     -       4*aet3*M32 - 4*M34 + M12*FDMH(1,2,2) - 2*M32*FDMH(1,2,2)))/
     -  ((M12 - 4*M32)**2*M12*M34)

      endif
      FDMH(3,1,2) =((-2*(aet1+aet2+aet3+FDMH(1,1,2))*M1222*M22*M32)/M24- 
     -  (a0t3*M22*(M16 + (M22 - M32)**3 - M14*(M22 + 3*M32) + 
     -  M12*(2*M2232 - M24 + 3*M34)))/(2.*M24) + 
     -  a0t1*(-((a0t3*abr4*M22**2)/M24) - (a0t2*abr1*M22*M32)/M24 + 
     -  (M22*(M16 - 2*M12*M2232 + 3*M12*M24 - M22*M24 + M24*M32 - 
     -  3*M14*(M22 + M32) + 3*M12*M34 + M22*M34 - M32*M34))/
     -  (2.*M24)) + a0t2*
     -  ((a0t3*M22*(M14 - M12*(M22 + M32)))/M24 + 
     -  (M18 - 5*M16*M22 + 5*M14*M2232 + 9*M14*M24 + 2*M12*M2432 - 
     -  5*M22*M2432 - 7*M12*M26 + 2*M22*M26 - 4*M16*M32 + 
     -  5*M2432*M32 - 2*M26*M32 + 6*M14*M34 + 5*M12*M22*M34 + 
     -  4*M22**2*M34 - 4*M22*M32*M34 - 4*M12*M36 - M22*M36 + M32*M36
     -  )/(2.*M24)) + 
     -  (M22*(4*M1226 - 6*M1424 - 6*M1434 + 4*M1622 - M18 - M28 - 
     -  8*M1224*M32 - 8*M1422*M32 + 4*M16*M32 + 4*M26*M32 - 
     -  8*M1222*M34 - 6*M24*M34 + 4*M12*M36 + 4*M22*M36 - M38))/
     -  (2.*M24))/alambda2

      FDMH(3,2,1) =(-2*(aet1+aet2+aet3+FDMH(1,1,2))*M1222 + a0t1* 
     -  (-(a0t2*abr1)+(a0t3*(-(M12*M2232) + M22*(M22 - M32)*M32))/M34 + 
     -       (M32*(M16 - 2*M12*M2232 + 3*M12*M24 - M22*M24 + M24*M32 - 
     -            3*M14*(M22 + M32) + 3*M12*M34 + M22*M34 - M32*M34))/
     -        (2.*M34)) + a0t2*
     -     (-(M32*(M16 - (M22 - M32)**3 - M14*(3*M22 + M32) + 
     -             M12*(2*M2232 + 3*M24 - M34)))/(2.*M34) + 
     -       (a0t3*(M14*M32 - M12*(M2232 + M34)))/M34) + 
     -    (a0t3*(M18 - 4*M16*M22 + 5*M14*M2232 + 6*M14*M24 + 5*M12*M2432 - 
     -         4*M22*M2432 - 4*M12*M26 + M22*M26 - 5*M16*M32 + 
     -         2*M12*M2232*M32 + 4*M2432*M32 - M26*M32 + 9*M14*M34 + 
     -         5*M22**2*M34 - 5*M22*M32*M34 - 7*M12*M36 - 2*M22*M36 + 
     -         2*M32*M36))/(2.*M34) - 
     -    (M32*(-4*M1226 + 6*M1424 + 6*M1434 - 4*M1622 + M18 + M28 + 
     -         8*M1224*M32 + 8*M1422*M32 - 4*M16*M32 - 4*M26*M32 + 
     -         8*M1222*M34 + 6*M24*M34 - 4*M12*M36 - 4*M22*M36 + M38))/
     -     (2.*M34))/alambda2

      FDMH(4,1,2) =-(((2*abr2*abr3*M1232)/M32  + 
     -      (2*abr3*(aet2+aet3+aet1+FDMH(1,1,2))*M1232)/M32  + 
     -      (a0t3*(M16 - (M22 - M32)**3 + M14*(-3*M22 + M32) + 
     -           M12*(-4*M2232 + 3*M24 - 3*M34)))/M32 + 
     -      (a0t2*(M16 + M14*(M22 - 3*M32) + (M22 - M32)**3 - 
     -           (a0t3*abr3**2*M12)/M32 - M12*(4*M2232 + 3*M24 - 3*M34)))/
     -       M22 - a0t1*(2*((M22 - M32)**2 - M12*(M22 + M32)) + 
     -         (a0t2*(2*M1232 - M14 + M24 - M34))/M22 + 
     -         (a0t3*(2*M1222 - M14 - M24 + M34))/M32))/alambda2)
      FDMH(4,2,1) = FDMH(4,1,2) 

      end
c======================================================================
      double precision function xF0(x,y)
      implicit none
      double precision x,y

      if (x.eq.y) then
        xf0=0
      elseif (x.eq.0D0 .AND. y.ne.0D0) then
        xf0= y
      elseif (y.eq.0D0 .AND. x.ne.0D0) then
        xf0= x
      else
       xf0=x+y -2*(x*y)/(x-y)* Log(x/y)
      endif 
      end


#define epsi 1D-10

*
***********************************************************************
*
      double precision function A0T(M2, Q2)
      double precision M2, Q2
      if(M2.gt.0D0) then
       A0T=M2* (1- Log(M2/Q2))
      else
       A0T=0D0
      endif

      return
      end

*
***********************************************************************
*
      double precision function AEpT(M2, Q2)
      double precision M2, Q2, Zeta2, Pi
      Pi   = 3.141592653589793D0
      Zeta2= Pi**2/6.
      if (M2 .gt. 0D0) then
        AEpT= M2*(1.+0.5D0* Zeta2 - Log(M2/Q2) + (Log(M2/Q2))**2/2.)
      else
        AEpT=0D0
      endif
      end

*
***********************************************************************
*
      double precision function B0T(M12,M22,Q2)
      double precision M12, M22, Q2
      if(M12.eq.0D0 .AND. M22.eq.0)then
       B0T=0D0
      elseif(M12.eq.0D0 .AND. M22.ne.0)then
       B0T=  1- Log(M22/Q2)
      elseif(M12.gt.0D0 .AND. M22.eq.0)then
       B0T=  1- Log(M12/Q2)
      elseif(abs(M12-M22).eq. 0D0 ) then
       B0T=  - Log(M22/Q2) 
      else
       B0T= 1. - Log(M22/Q2) + M12*Log(M12/M22)/(M22-M12)
      endif
      return
      end
*
***********************************************************************
*
      

      double precision function BEpT(M12,M22,Q2)
      double precision M12, M22, Q2,C,Zeta2, Pi,
     -    AEpT,c1,c2
      c1= M12; c2=M22
      Pi   = 3.141592653589793D0
      Zeta2= Pi**2/6.

      if(M12.gt.M22)then
           C=M22; M22=M12; M12=C
      endif

      if(M12.eq.0D0 .AND. M22.eq.0)then
       BEpT=0D0
      elseif(M12.eq.0D0 .AND. M22.ne.0)then
       BEpT= 1. +0.5D0* Zeta2 - Log(M22/Q2) + (Log(M22/Q2))**2/2.
      elseif(abs(M12-M22).eq. 0D0) then
       BEpT=  0.5D0*Zeta2+ 0.5D0* Log(M22/Q2)**2 
      else
       BEpT= (AEpT(M22,Q2)- AEpT(M12,Q2))/(M22-M12)
      endif
      M12=c1; M22=c2      
      return
      end

*
***********************************************************************
*

      double complex function B0P(P2,M12,M22,Q2)
      double precision P2, M12, M22, Q2, c1, c2,  B0T
      double precision pi, is, ix(2)
      double complex cI,x(2),y(2),a,b,c, CPLOG

      Pi   = 3.1415926535897932384626433832795029D0
      cI=dcmplx(0.,1.)
      c1= M12; c2=M22
      is =-1d0
      if (P2.eq.0D0) then
         B0P=   B0T(c1,c2,Q2)
         return
      else
         if ( c1.eq.0D0 .AND. c2.gt.0D0) then
           if (c2.eq.p2) then
            x(1)=0
            x(2)=0
            B0P = 2 - Log(p2/q2)
           else
            x(1)=0 
            x(2)=1.- c2/p2
            ix(2) =- is/x(2)
        B0P=-Log(p2/q2)-(Log(c2/p2)-x(2)*CPLOG(1d0-1d0/x(2),ix(2))-2d0)
           endif

         elseif ( c1.gt.0D0 .AND. c2.eq.0D0) then
           if (c1.eq.p2) then
            B0P = 2 - Log(p2/q2)
           else
            x(1)=0 
            x(2)=1.- c1/p2
            ix(2) = -is/x(2)
         B0P=-Log(p2/q2)-(Log(c1/p2)- x(2)*CPLOG(1.-1./x(2),ix(2))-2.)
           endif
         elseif(c1.eq.0d0 .AND. c2.eq.0d0) then
            x(1)=0; x(2) =1; ix(2)=- is
            B0P = -(Log(p2/q2) - 2 )+ cI*pi
         else
            a= p2
            b=(-p2 +c2-c1)
            c=c1
            call CYPRoot2(a,b,c, is, x, ix)

            B0P=-log(p2/q2)-(CPLOG(1-x(1),-ix(1))-x(1)*
     -          CPLOG(1.-1/x(1),ix(1))-1.)- ( CPLOG(1.- x(2),-ix(2)) - 
     -          x(2)*CPLOG(1.-1./x(2),ix(2))- 1.)
         endif
         
      endif
           
      end

*
***********************************************************************
*

      double complex function B1P(P2,M12,M22,Q2)
      double precision P2, M12, M22, Q2, c1, c2,  B0T, A0T
      double precision pi, is, ix(2)
      double complex cI,x(2),y(2),a,b,c, CPLOG,B0P

      Pi   = 3.1415926535897932384626433832795029D0
      cI=dcmplx(0.,1.)
      c1= M12; c2=M22
      is =-1d0
      if (P2.eq.0D0) then
         if (c1.eq.0D0.AND. c2.gt.0D0) then
          x(2) =0; ix(2) = -is/(c2)
          B1P=-0.5D0* ( Log(q2/c2)  ) -0.25D0
         elseif (c2.eq.0D0.AND. c1.gt.0D0) then
          B1P=-0.5D0* ( Log(q2/c1)  ) -0.25D0         
         elseif (c2.eq.0D0.AND. c1.eq.0D0) then
           B1P=0D0
         elseif (abs(c1-c2).le.1D-10) then
           B1P=-0.5D0* ( Log(q2/c1))
         else
          x(2) =-c1/(c2-c1);ix(2) = -is/(c2-c1)
          B1P=-0.5D0* ( Log(q2/c1)  -CPLOG(1.-1/x(2), ix(2)) +
     -          1/3D0/x(2) + 1/4D0/(x(2)**2) +1/5D0/(x(2)**3)+
     -          1/6D0/(x(2)**4)+ 1/7D0/(x(2)**5)  )
         endif        
         return
      else
         if ( c1.eq.0D0 .AND. c2.gt.0D0) then
           if (c2.eq.p2) then
            B1P =-0.5D0*(1- Log(p2/q2))
           else
            B1P=((c2-p2)* B0P(p2,0D0,c2,q2) - A0T(c2,q2) )/(2D0*p2)
           endif

         elseif ( c1.gt.0D0 .AND. c2.eq.0D0) then
           if (c1.eq.p2) then
            B1P = -0.5D0*(1- Log(p2/q2))-1D0
           else

            B1P=(-(c1+p2)* B0P(p2,0D0,c1,q2) + A0T(c1,q2) )/(2D0*p2)
           endif
         elseif(c1.eq.0d0 .AND. c2.eq.0d0) then
            x(1)=0; x(2) =1; ix(2)=- is
            B1P =-0.5D0*( -(Log(p2/q2) - 2 )+ cI*pi)
         else
            B1P =-0.5d0*(( p2+c1-c2)*B0P(p2,c1,c2,q2)-A0T(c1,q2)+
     -           A0T(c2,q2))/(p2)
         endif
         
      endif
           
      end
*
***********************************************************************
*

      double complex function B1EpP(P2,M12,M22,Q2)
      implicit none
      double precision P2, M12, M22, Q2, c1, c2, AEpT
      double complex BEpP

      c1= M12; c2=M22
      if (P2.eq.0D0) then
         B1EpP=0D0 ! this situation have not been evaluated
         return
      else
         if ( c1.eq.0D0 .AND. c2.gt.0D0) then
            B1EpP=((c2-p2)* BEpP(p2,0D0,c2,q2)- AEpT(c2,q2) )/(2D0*p2)
         elseif ( c1.gt.0D0 .AND. c2.eq.0D0) then
            B1EpP=(-(c1+p2)* BEpP(p2,0D0,c1,q2)+ AEpT(c1,q2) )/(2D0*p2)
         elseif(c1.eq.0d0 .AND. c2.eq.0d0) then
            B1EpP= (- BEpP(p2,0D0,0D0,q2)  )/(2D0)
         else
            B1EpP=-0.5d0*((p2+c1-c2)*BEpP(p2,c1,c2,q2)-AEpT(c1,q2)
     &            +AEpT(c2,q2))/(p2)

         endif
         
      endif
           
      end

*
***********************************************************************
*
* e.q 1.6 in Z. Phys. C57, 605-613 U. Nierste 1992
* this has been check against mathematica within agreement
* keep in mind that in mathematica one needs to introduce a small negative imaginary part
* in the Log

      double complex function BEpP(P2,M12,M22,Q2)
      implicit none
      double precision P2, M12, M22, Q2, c1, c2,  BEpT
      double precision Pi, Zeta2, is,ix(2),dx(2)
      double complex cI,x(2),y(2),a,b,c, B0P,CPLOG,CPSp

      Pi   = 3.1415926535897932384626433832795029D0
      Zeta2= Pi**2/6.
      c1= M12; c2=M22
      cI=dcmplx(0.,1.)
      is =-1d0
      if (P2.eq.0D0) then
         BEpP=   BEpT(c1,c2,Q2)
         return
      else
         if ( c1.eq.0D0 .AND. c2.gt.0D0) then
           if (c2.eq.p2) then
            x(1)=0
            x(2)=0

            BEpP= 0.5D0*Zeta2 + 0.5D0* Log(p2/q2)*Log(p2/q2)
     -       - (Log(p2/q2)-2.)*( 2 )

           else
            x(1)=0
            x(2)=1-c2/p2
            ix(2) = -is/x(2)

            BEpP= 0.5D0*Zeta2 + 0.5D0* Log(p2/q2)*Log(p2/q2)
     -       - (Log(p2/q2)-2.)*(B0P(p2,c1,c2,q2) + Log(p2/q2) )
     -       +0.5D0*(1- x(2))*CPLOG(1-x(2),-ix(2))*CPLOG(1-x(2),-ix(2))
     -       +0.5D0*x(2)*CPLOG(-x(2), -ix(2))*CPLOG(-x(2), -ix(2))
     -       +(- x(2))*( CPLOG(x(2), ix(2))*CPLOG((x(2)-1.)/x(2),ix(2)) 
     -       +Zeta2- CPSp((x(2)-1)/(x(2)) , ix(2))  ) 
           endif

         elseif ( c1.gt.0D0 .AND. c2.eq.0D0) then
           if (c1.eq.p2) then
            x(1) =0
            x(2) =0
            BEpP= 0.5D0*Zeta2 + 0.5D0* Log(p2/q2)*Log(p2/q2)
     -       - (Log(p2/q2)-2.)*( 2 )
           else
            x(1)=0
            x(2)=1-c1/p2
            ix(2) =- is/x(2)
            BEpP= 0.5D0*Zeta2 + 0.5D0* Log(p2/q2)*Log(p2/q2)
     -       - (Log(p2/q2)-2.)*(B0P(p2,c1,c2,q2) + Log(p2/q2) )+0.5D0*
     -       (1.-x(2))*CPLOG(1.-x(2),-ix(2))*CPLOG(1.-x(2),-ix(2))
     -       +0.5D0*x(2)*CPLOG(-x(2), -ix(2))*CPLOG(-x(2), -ix(2))+(
     -       - x(2))*( CPLOG(x(2), ix(2))* CPLOG((x(2)-1.)/x(2),ix(2)) 
     -       +Zeta2- CPSp((x(2)-1.)/(x(2)) , ix(2))  ) 
           endif
         elseif(c1.eq.0d0 .AND. c2.eq.0d0) then
            x(1)=0;ix(1)=is; x(2)=1D0; ix(2) =-is
            dx(1)=DBLE(x(1))
            dx(2)=DBLE(x(2))
            BEpP= 0.5D0*Zeta2 + 0.5D0* Log(p2/q2)*Log(p2/q2)
     -       -(Log(p2/q2)-2.)*(  2. + cI*pi  )+0.5D0*x(2)*
     -       CPLOG(-x(2), -ix(2))*CPLOG(-x(2), -ix(2))+(- x(2))*
     -       (CPSp(dcmplx(1D0,0D0), dx(2)*ix(1)-dx(1)*ix(2))-
     -       CPSp((x(2)-1.)/(x(2)),dx(2)*ix(1)-dx(1)*ix(2)-ix(1)+ix(2)))
         else
            a= p2
            b=(-p2 +c2-c1)
            c=c1
            call CYPRoot2(a,b,c, is, x, ix)
            y(1)= x(2)/(x(2)-x(1))
            y(2)= (x(2)-1.)/(x(2)-x(1)) 
            dx(1)=DBLE(x(1))
            dx(2)=DBLE(x(2))

            BEpP = 0.5D0*Zeta2 + 0.5D0* Log(p2/q2)*Log(p2/q2)
     -       - (Log(p2/q2)-2.)*(B0P(p2,c1,c2,q2) + Log(p2/q2) )+0.5D0*
     -       (1.- x(1))*CPLOG((1.-x(1)),-ix(1))*CPLOG((1.-x(1)),-ix(1))
     -       +0.5D0*x(1)*CPLOG(-x(1),-ix(1))*CPLOG(-x(1), -ix(1))+0.5D0
     -       *(1.- x(2))*CPLOG(1.-x(2),-ix(2))*CPLOG(1.-x(2),-ix(2))
     -       +0.5D0*x(2)*CPLOG(-x(2), -ix(2))*CPLOG(-x(2), -ix(2))
     -       + (1.- x(1))*CPLOG(1.-x(1),-ix(1))*CPLOG(1.-x(2),-ix(2))
     -       + x(1)*CPLOG(-x(1), -ix(1))*CPLOG(-x(2),-ix(2))
     -       + (x(1)- x(2))*(CPLOG(x(2)-x(1), ix(2))* 
     -       CPLOG((x(2)-1.)/x(2), ix(2)) 
     -       + CPSp(y(1),dx(2)*ix(1)-dx(1)*ix(2))
     -       - CPSp(y(2),dx(2)*ix(1)-dx(1)*ix(2)-ix(1)+ix(2))  )
         endif       
      endif
           
      end
************************************************************************
	subroutine CYPRoot2(a, b, c, is, x, ix)
************************************************************************
* a part of Ninh's code
* Find the 2 roots of the equation: a*x^2 + b*x + c + is*I = 0
* where "is" is infinitesimal, Ref0. Eq. (A.2).
* Input: a, b, c are complex; "is" is real.
* Output: x(2) and ix(2).
************************************************************************
	implicit none
	double complex x(2),a, b, c, d
	double precision  is, ix(2)

	if ( abs(a) .le. 0 ) then
	 print*,'Error in CYPRoot2: a vanishes, STOP'
	 stop
	endif

	ix(1) = is

	if ( abs(c) .le. 0) then
	x(1) = 0
	x(2) = -b/a
	d = sqrt(b**2)
	if ( DBLE(d) .lt. 0 ) ix(1) = -ix(1) 

	else
	d = b**2 - 4*a*c
	d = sqrt(d)
	x(1) = (-b - d)/(2*a)
	x(2) = (-b + d)/(2*a)
	if ( DBLE(d) .lt. 0 ) ix(1) = -ix(1) 
	
	if ( abs(x(1)) .lt. abs(x(2)) ) then
	 x(1) = c/a/x(2)
	else
	 x(2) = c/a/x(1)
	endif 
	 
	endif

	ix(2) = -ix(1)


	end

	double complex function CPLOG(x, is)
************************************************************************
* Def. Log(x + i*is) where "is" is infinitesimal.
* Author: Le Duc Ninh (2011)
************************************************************************
	implicit none
	double precision is,pi
	double complex x, cipi
        Pi   = 3.1415926535897932384626433832795029D0
        cipi = dcmplx(0D0, pi)

	CPLOG = 0

***[singular cases
	if (abs(x) .le. 0) then
	 print*,'Error in CPLOG(): log(0) encountered, return 0'
         return
	endif
***]singular cases

	if (DIMAG(x) .eq. 0) then
	 if (DBLE(x) .gt. 0) then
	  CPLOG = log(x)
	 else
	  CPLOG = log(-x) + cipi*is/abs(is)
	 endif 	
	else
	CPLOG = log(x)	
	endif
	end

************************************************************************
	double complex function CPSp(x, is)
************************************************************************
* Computes the dilogarithm (Li2 or Sp) for any (complex) x 	
* to a precision prec.
* Author: Le Duc Ninh (2011)
************************************************************************
	implicit none
	double complex x, CPSp1, CPLOG
	double precision is,pi
	external CPSp1, CPLOG
        Pi   = 3.1415926535897932384626433832795029D0
	CPSp = 0

***[special cases
	if ( abs(x-1) .le. 0 ) then 
	 CPSp = pi**2/6D0 
	 return
	endif  

	if ( abs(x-1) .le. epsi ) then 
	 CPSp = -(1-x) + pi**2/6D0 - CPLOG(x, is)*CPLOG(1-x, -is) 
	 return
	endif  
***]special cases

	if ( abs(x) .le. 1 .and. DBLE(x) .le. 0.5D0 ) then 
	 CPSp = CPSp1(x, is)
	elseif ( abs(x) .gt. 1 .and. DBLE(x) .le. 0.5D0 ) then
	 CPSp = -CPSp1(1/x, -is) - pi**2/6D0 - CPLOG(-x, -is)**2/2
	
	elseif ( abs(x) .le. 1 .and. DBLE(x) .gt. 0.5D0 ) then
	 CPSp = -CPSp1(1-x, -is) + pi**2/6D0 
     &	  - CPLOG(x, is)*CPLOG(1-x, -is) 

	else
* here we have |x| > 1 AND Re(x) > 0.5	
	 if ( DBLE(1/x) .le. 0.5D0 ) then
	  CPSp = -CPSp1(1/x, -is) - pi**2/6D0 - CPLOG(-x, -is)**2/2
	 else
	  CPSp = -CPSp1(1-x, -is) + pi**2/6D0 
     &	  - CPLOG(x, is)*CPLOG(1-x, -is) 
         endif
	 
	endif

	end

************************************************************************
	double complex function CPSp1(x, is)
************************************************************************
* Computes the dilogarithm (Li2 or Sp) for complex x with 
* |x| <= 1 and Re(x) <= 0.5 to a precision epsi.
* Ref0. 't Hooft and Veltman, Nucl. Phys. B153 (1979) 365.
* Author: Le Duc Ninh (2011)
************************************************************************
	implicit none
	integer i, init, nn
	save init
	data init /0/
        parameter(nn = 60)
	double complex x, z, CPLOG, ctemp
	double precision is, bern(nn), temp,pi
	external CPLOG
        Pi   = 3.1415926535897932384626433832795029D0
	CPSp1 = 0
	if ( abs(x) .gt. 1 .or. DBLE(x) .gt. 0.5D0 ) then 
	 print*,'Error in CPSp11(): invalid input, STOP'
	 stop
	endif 

***[special cases
	if ( abs(x) .le. epsi ) then 
	 CPSp1 = x
	 return
	endif  

	if ( abs(x-0.5D0) .le. 0 ) then 
	 CPSp1 = pi**2/12D0 - log(2D0)**2/2D0 
	 return
	endif
***]special cases

***[Bernouilli numbers
c        if (init .eq. 0) then
c        init = 1

        do i = 3, nn-1, 2
          bern(i) = 0
        enddo

        bern( 1) = -1D0/2D0
        bern( 2) =  1D0/6D0
        bern( 4) = -1D0/30D0
        bern( 6) =  1D0/42D0
        bern( 8) = -1D0/30D0
        bern(10) =  5D0/66D0
        bern(12) = -691D0/2730D0
        bern(14) =  7D0/6D0
        bern(16) = -3617D0/510D0
        bern(18) =  43867D0/798D0
        bern(20) = -174611D0/330D0
        bern(22) =  854513D0/138D0
        bern(24) = -236364091D0/2730D0
        bern(26) =  8553103D0/6D0
        bern(28) = -23749461029D0/870D0
        bern(30) =  8615841276005D0/14322D0
        bern(32) = -7709321041217D0/510D0
        bern(34) =  2577687858367D0/6D0
        bern(36) = -26315271553053477373D0/1919190D0
        bern(38) =  2929993913841559D0/6D0
        bern(40) = -261082718496449122051D0/13530D0

	bern(42) = 1520097643918070802691D0/1806D0
	bern(44) = -27833269579301024235023D0/690D0
	bern(46) = 596451111593912163277961D0/282D0
	bern(48) = -5609403368997817686249127547D0/46410D0
	bern(50) = 495057205241079648212477525D0/66D0
	bern(52) = -801165718135489957347924991853D0/1590D0
	bern(54) = 29149963634884862421418123812691D0/798D0
	bern(56) = -2479392929313226753685415739663229D0/870D0
	bern(58) = 84483613348880041862046775994036021D0/354D0
	bern(60) = -1215233140483755572040304994079820246041491D0/
     &   56786730D0
c        endif
***]Bernouilli numbers

	z = -CPLOG(1 - x, -is)
	ctemp = bern(1)*z**2/2D0
	CPSp1 = z + ctemp 
c        print*, CPSp1, x, ctemp, bern(1),z**2,1/2D0
      
        i = 0
	temp = 1

	do while ( abs(ctemp) .gt. epsi )
	 i = i + 2

c	print*,'From CPSp1(): i, z, ctemp =', i, z, ctemp

	 if ( i .gt. nn) then 
	  print*,'Error in CPSp1(): 
     &	  Bernouilli series is too short, return at', nn
	  ctemp = 0
	 
	 else
	  temp = temp*i*(i+1)
	  ctemp = bern(i)*z**(i + 1)/temp	  
	 endif  

	CPSp1 = CPSp1 + ctemp
	enddo

	end

*
***********************************************************************
*
      
      double precision function C0T(Mass12,Mass22,Mass32,Q2)
      double precision Mass12,Mass22,Mass32,x, y, z, Q2,C
      double precision B0T
        x=Mass12
        y=Mass22
        z=Mass32
   
       if (z.lt.y) then
          c=z; z=y; y=c
       endif  
       if(z.lt.x) then
        c=z;        z=x;        x=c
       endif
       if(y.lt.x) then
        c=y; y=x; x=c
       endif
      if (abs(z-x) .gt. 0D0) then
       C0T =(B0T(x,y,Q2)-B0T(y,z,Q2))/(x-z)
      else
       C0T=-0.5D0/z
      endif
      return
      end
*
***********************************************************************
*
      
      double precision function CEpT(Mass12,Mass22,Mass32,Q2)
      double precision Mass12,Mass22,Mass32, x, y, z, Q2,C
      double precision BEpT
        x=Mass12
        y=Mass22
        z=Mass32

       if (z.lt.y) then
          c=z; z=y; y=c
       endif  
       if(z.lt.x) then
        c=z;        z=x;        x=c
       endif
       if(y.lt.x) then
        c=y; y=x; x=c
       endif
      if (abs(z-x) .gt. 0D0) then
       CEpT =(BEpT(x,y,Q2)-BEpT(y,z,Q2))/(x-z)
      else
       CEpT=0.5D0* Log(z/Q2)/z
      endif
      return
      end
*
***********************************************************************
*
** note that this formula are of 2.34 of hep-ph/0501132 but difference
* in sign for AepT and A0T 

      double complex function K0T0(Mass12,Mass22,Mass32,Q2)
      double precision Mass12,Mass22,Mass32,x, y, z, Q2
      double precision res1,c
      double precision AEpT,I0N
      double complex EtaN,res2
        x=Mass12
        y=Mass22
        z=Mass32
       res1= -AEpT(x,Q2)- AEpT(y,Q2)- AEpT(z,Q2)

       if (z.lt.y) then
          c=z; z=y; y=c
       endif  

       if(z.lt.x) then
        c=z; z=x; x=c
       endif

       if(y.lt.x) then
        c=y; y=x; x=c
       endif
       res2 = I0N(x,y,z,Q2)
       K0T0= (res1+res2)
      return
      end

*
***********************************************************************
*
      double complex function SKK(s,Mass12,Mass22,Mass32,Q2)
      double precision Mass12,Mass22,Mass32,s, x, y,z,c,Q2,ix1,ix2
      double precision I0N,Zeta2,pi
      double complex CPSp,CPLOG
       ix1=1D0; ix2=-1D0
       pi= 3.1415926535897932384626433832795029D0
       Zeta2= pi**2/6D0
        x=Mass12
        y=Mass22
        z=Mass32
       if (z.lt.y) then
          c=z; z=y; y=c
       endif  
       if(z.lt.x) then
        c=z; z=x; x=c
       endif
       if(y.lt.x) then
        c=y; y=x; x=c
       endif

c[special case s=0
       if(s.le.epsi) then
        SKK= I0N(x,y,z,Q2)
       endif
c]
c[special case x=y=z=0
       if(x.eq.0D0 .AND. y.eq.0D0 .AND. z.eq.0D0) then
          SKK= -13*s/8D0 +s/2D0*Log(s/Q2)
          goto 99
       endif
c]
 
c[special case x=y=z=0
       if(x.eq.0D0 .AND. y.eq.0D0 .AND. z.gt.0D0) then
          SKK=(13*s)/8. - z*(2 + Zeta2) + ((-s**2 + z**2)
     -        *CPLog(dcmplx(1 - s/z,0D0),-ix1))/(2.*s)+ (-s/2. + 2*z)*
     -        Log(z/Q2)-z*(Log(z/Q2)**2/2.+CPSp(dcmplx(s/z,0D0),ix1))
          goto 99
       endif
c]      

99      return
      end

*
***********************************************************************
*
      double complex function TKK(s,Mass12,Mass22,Mass32,Q2)
      double precision Mass12,Mass22,Mass32,s, x, y,z,Q2,ix1,ix2
      double precision Zeta2,pi
      double complex CPSp,CPLOG
       ix1=1D0; ix2=-1D0
       pi= 3.1415926535897932384626433832795029D0
       Zeta2= pi**2/6D0
        x=Mass12
        y=Mass22
        z=Mass32

c[special case s=0
       if(s.le.epsi) then
        if (z.eq.0) then
          if(abs(x-y).eq.0D0) then
          TKK= -(1/2D0)*(Log(x/Q2)-1)**2
          else
          TKK=-0.5 - Zeta2 + 2*Log(x/Q2) + Log(x/Q2)**2/2.
     -    -CPLog(dcmplx((x - y)/Q2,0D0),ix2)*Log(x/y)- Log(y/Q2) 
     -    -Log(x/Q2)*Log(y/Q2) + (x - y)*(Log(x/Q2)/x - 
     -    CPLog(dcmplx((x - y)/Q2,0D0), ix2)/x- Log(x/y)/(x-y)+
     -    CPLog(dcmplx(1-y/x,0D0),ix2)/x) 
     -    + CPSp(dcmplx(y/x,0D0), ix1)
          endif
        endif
        goto 99
       endif
c]
99      return
      end
*
***********************************************************************
*
      double precision function K0T0N(Mass12,Mass22,Mass32,aet1,aet2,
     &              aet3,Q2)
      double precision Mass12,Mass22,Mass32,x, y, z,Q2,aet1,aet2,aet3
      double precision res1,c,I0N,res2
        x=Mass12
        y=Mass22
        z=Mass32
       res1= -aet1-aet2-aet3
       if (z.lt.y) then
          c=z; z=y; y=c
       endif  
       if(z.lt.x) then
        c=z; z=x; x=c
       endif
       if(y.lt.x) then
        c=y; y=x; x=c
       endif
       res2 = I0N(x,y,z,Q2)
       K0T0N= (res1+res2)
      return
      end


*
***********************************************************************
*
      double precision function K0T1(M12,M22,M32,Q2)
      double precision M12, M22, M32, Q2
      double precision A0T
       K0T1=( -A0T(M12,Q2)- A0T(M22,Q2)-A0T(M32,Q2)-0.5D0*(M12+M22+M32))
      end
*
***********************************************************************
*
      double precision function K0T2(M12,M22,M32,Q2)
      double precision M12, M22, M32, Q2
       K0T2= -0.5D0*(M12+M22+M32)
      end

*
***********************************************************************
*
      double precision function LambdaN(x,y,z)
      double precision x,y,z
       LambdaN=x**2+y**2+z**2-2*x*y-2*x*z-2*y*z

      end
*
***********************************************************************
*
      double complex function EtaN(Mass12,Mass22,Mass32)
      double precision Mass12,Mass22,Mass32,x,y,z,c,pi,pi2,LambdaN
     .  ,alambda,is
      double complex CPSp,R, CPLOG
       x= Mass12
       y= Mass22
       z= Mass32
       alambda=LambdaN(x,y,z)
       if (alambda .ge. 0D0) then
        R=sqrt(alambda)
       else
        R=dcmplx(0.,Sqrt(-alambda))
       endif
        is=-1
       pi= 3.1415926535897932384626433832795029D0
       pi2=pi**2
       if (z.lt.y) then
          c=z; z=y; y=c
       endif  

       if(z.lt.x) then
        c=z; z=x; x=c
       endif
       EtaN= R*( 2.* Log((z+x-y-R)/(2.*z))*Log((z+y-x-R)/(2.*z)) -
     -     Log(x/z)*Log(y/z) - 2.*CPSp((z+x-y-R)/(2.*z),is) 
     -     -2.* CPSp((z+y-x-R)/(2.*z), is) + Pi2/3.)

      end
*
***********************************************************************
*
      double precision function I0N(x,y,z,Q2)
      double precision x,y,z,c,pi,pi2,LambdaN
     .  ,alambda,is,EtaN,Q2
      double complex CPSp,R, CPLOG

       is=-1
       pi= 3.1415926535897932384626433832795029D0
       pi2=pi**2
      if(x.gt.0)then
       alambda=LambdaN(x,y,z)
       if (alambda .ge. 0D0) then
        R=sqrt(alambda)
       else
        R=dcmplx(0.,Sqrt(-alambda))
       endif

       EtaN= R*( 2.* Log((z+x-y-R)/(2.*z))*Log((z+y-x-R)/(2.*z)) -
     -     Log(x/z)*Log(y/z) - 2.*CPSp((z+x-y-R)/(2.*z),is) 
     -     -2.* CPSp((z+y-x-R)/(2.*z), is) + Pi2/3.)
       I0N= 0.5D0*(x-y-z)* Log(y/Q2)*Log(z/Q2) +
     -         0.5D0*(y-x-z)* Log(x/Q2)*Log(z/Q2) +
     -         0.5D0*(z-x-y)* Log(x/Q2)*Log(y/Q2) +
     -         2*x*Log(x/Q2) + 2*y*Log(y/Q2)+2*z*Log(z/Q2)
     -         -2.5D0*(x+y+z)-0.5D0*EtaN
      else
       if ( (z-y)/z .gt. 0D0 ) then
         if (y .gt. 0D0) then
         I0N= (z-y)*( CPsp(dcmplx(y/z,0d0),1D0)-Log(z/y)*Log((z-y)/q2) 
     -    +0.5D0*Log(z/q2)**2- pi2/6D0) - 2.5D0* (z+y) + 2*z*Log(z/q2)
     -    + 2*y*Log(y/q2) -z*Log(z/q2)*Log(y/q2)

         else
         I0N= -0.5D0 *z*Log(z/q2)**2+ 2*z*Log(z/q2)-2.5D0*z-pi2/6D0*z
         endif
       else
         if (z.gt.0D0) then
         I0N = -z*Log(z/q2)**2 + 4.*z* Log(z/q2) - 5*z 
         else
         I0N=0
         endif
       endif

      endif
      end
*
***********************************************************************
*
      double precision function I0A(Mass12,Mass22,Mass32,Q2)
      double precision x,y,z,c,pi,pi2,LambdaN
     .  ,alambda,is,EtaN,Q2,Mass12,Mass22,Mass32
      double complex CPSp,R, CPLOG

        x=Mass12
        y=Mass22
        z=Mass32

       if (z.lt.y) then
          c=z; z=y; y=c
       endif  
       if(z.lt.x) then
        c=z; z=x; x=c
       endif
       if(y.lt.x) then
        c=y; y=x; x=c
       endif

       is=-1
       pi= 3.1415926535897932384626433832795029D0
       pi2=pi**2

      if(x.gt.0)then
       alambda=LambdaN(x,y,z)
       if (alambda .ge. 0D0) then
        R=sqrt(alambda)
       else
        R=dcmplx(0.,Sqrt(-alambda))
       endif

       EtaN= R*( 2.* Log((z+x-y-R)/(2.*z))*Log((z+y-x-R)/(2.*z)) -
     -     Log(x/z)*Log(y/z) - 2.*CPSp((z+x-y-R)/(2.*z),is) 
     -     -2.* CPSp((z+y-x-R)/(2.*z), is) + Pi2/3.)
       I0A= 0.5D0*(x-y-z)* Log(y/Q2)*Log(z/Q2) +
     -         0.5D0*(y-x-z)* Log(x/Q2)*Log(z/Q2) +
     -         0.5D0*(z-x-y)* Log(x/Q2)*Log(y/Q2) +
     -         2*x*Log(x/Q2) + 2*y*Log(y/Q2)+2*z*Log(z/Q2)
     -         -2.5D0*(x+y+z)-0.5D0*EtaN
      else
       if ( (z-y)/z .gt. 0D0 ) then
         if (y .gt. 0D0) then
         I0A= (z-y)*( CPsp(dcmplx(y/z,0d0),1D0)-Log(z/y)*Log((z-y)/q2) 
     -    +0.5D0*Log(z/q2)**2- pi2/6D0) - 2.5D0* (z+y) + 2*z*Log(z/q2)
     -    + 2*y*Log(y/q2) -z*Log(z/q2)*Log(y/q2)

         else
         I0A= -0.5D0 *z*Log(z/q2)**2+ 2*z*Log(z/q2)-2.5D0*z-pi2/6D0*z
         endif
       else
         if (z.gt.0D0) then
         I0A = -z*Log(z/q2)**2 + 4.*z* Log(z/q2) - 5*z 
         else
         I0A=0
         endif
       endif

      endif
      end

*
***********************************************************************
*
*(assume x>=y I(x,y)=I(y,x))

      double precision function I0(x,y,q2)
      double precision x, y, q2,c, pi, pi2
      double complex CPsp
       pi= 3.1415926535897932384626433832795029D0
       pi2=pi**2/6D0
       if ( (x-y)/x .gt. 0D0 ) then
         if (y .gt. 0D0) then
         I0= (x-y)*( CPsp(dcmplx(y/x,0D0),1D0) - Log(x/y)*Log((x-y)/q2) 
     -    +0.5D0*Log(x/q2)**2- pi2) - 2.5D0* (x+y) + 2*x*Log(x/q2)
     -    + 2*y*Log(y/q2) -x*Log(x/q2)*Log(y/q2)
         else
         I0= -0.5D0 * x * Log(x/q2)**2 + 2* x* Log(x/q2)- 2.5D0*x - pi2*x
         endif
       else
         if (x.gt.0D0) then
         I0 = -x*Log(x/q2)**2 + 4.*x* Log(x/q2) - 5*x 
         else
         I0=0
         endif
       endif
      end

*
***********************************************************************
*
c Input: M1**2, M2**2, M3**3, Q2.
c output: coef(5) coresponding to 
c     F^D00000_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(1),
c     F^D00001_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(2),  
c     F^D01000_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(3),  
c     F^D10000_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(4),
c     F^D10001_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(5),
c     F^D11000_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(6),
c     F^D00002_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(7),
c     F^D01001_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(8),
c     F^D02000_({1,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(9),

      subroutine FD111C(Mass22,Mass12,Mass32,a0t2,a0t1,a0t3,aet2,aet1,
     .                  aet3,Q2,coef,id)
      implicit none
      integer id
      double precision Mass22,Mass12,Mass32 
      double precision M12,M22,M32, Q2
      double precision M14,M24,M34,M16,M26,M36,M18,M28,M38
      double precision abr1,abr2,abr3,abr4,
     .   a0t1,a0t2,a0t3,aet1,aet2,aet3
      double precision lambdaN, A0T, AEpT
      double precision K0T0N,coef(10),ak0t0

      M12= Mass12; M22=Mass22; M32=Mass32
      M14=M12**2;  M24=M22**2 ; M34=M32**2
      M16=M12*M14; M26=M22*M24; M36=M32*M34
      M18=M14**2;  M28=M24**2;  M38=M34**2

      abr1=(M12 + M22 - M32);abr2=(M12 + M22 + M32)
      abr3=(M12 - M22 - M32);abr4=(M12 - M22 + M32)

      if(id.eq.2) then
       coef(1)=(-M12 - M22 - M32)/2.
       coef(2)=(-M14 - M24 - 2*M12*M32 - 2*M22*M32 + M34)/4.

       coef(3)=-(abr2*M12)/2. - M22*M32
       coef(4)=-(abr2*M22)/2. - M12*M32
       coef(5)=(-M26 - 2*M12*M22*M32 - 2*M24*M32 - M14*(M22 + 4*M32) 
     -  + M22*M34)/4. 
       coef(6)=(-2*M24*M32 - M14*(M22 + 2*M32)-M12*(M24 + M22*M32))/2.  
       coef(7)=(-M16-M26-5*M24*M32-M14*(M22+5*M32)-M12*(M24+4*M22*M32-
     &  3*M34)+3*M22*M34-M36)/8.
       coef(8)=(-M16-2*(M14+2*M24)*M32+M12*(-M24-2*M22*M32+M34))/4.
       coef(9)=-(abr2*M14)/2. - M12*M22*M32 - M24*M32 - M22*M34
      else
       if(id.eq.1) then 
       coef(1)=(-2*a0t1 - 2*a0t2 - 2*a0t3 - M12 - M22 - M32)/2.
       coef(2)=(-2*a0t1*M12 - M14 - 2*a0t2*M22 - 2*M12*M22 - M24 + 
     -  a0t3*(-4*M12 - 4*M22 + 2*M32) + M34)/4.

       coef(3)=-(a0t1*M12) - a0t3*M12 - (abr2*M12)/2. - a0t3*M22 -
     -  a0t2*(M12 + M32)
       coef(4)=-((2*a0t2 + abr2)*M22)/2. - a0t3*(M12 + M22) - 
     -  a0t1*(M22 + M32)
       coef(5)=(-2*a0t1*M12*(M22 + 2*M32) - 2*a0t3*(2*M14 + 2*M12*M22 
     -    + 2*M24 - M22*M32) - M22*(M14 + 2*a0t2*M22 + 2*M12*M22 + M24
     -    - M34))/4.
       coef(6)=a0t3*(-M14 - M12*M22 - M24) - a0t2*M22*(M12 + M32) - 
     -  a0t1*M12*(M22 + M32) + (-(M14*M22) - M12*(M24 + M22*M32))/2.
       coef(7)=(-(abr1**2*abr2)-2*(a0t1*abr2*M12+a0t2*abr2*M22+a0t3*(4
     &  *M14+4*M12*M22+4*M24-3*M12*M32-3*M22*M32+M34)))/8.
       coef(8)=(-2*a0t1*M14-2*a0t2*M22*(M12+2*M32)-2*a0t3*(2*M14+2*M12
     &  *M22+2*M24-M12*M32)-M12*(M14+2*M12*M22+M24-M34))/4.
       coef(9)=-(a0t1*M14)-(abr2*M14)/2.-a0t3*(M14+M12*M22+M24+M22*M32
     &  )-a0t2*(M14+M12*M32+M22*M32+M34)
       elseif(id.eq.0) then
       ak0t0= K0T0N(M12,M22,M32,aet1,aet2,aet3,Q2) 
       coef(1)=ak0t0
       coef(2)=(a0t1*a0t2)/2. - (a0t1*a0t3)/2. - (a0t2*a0t3)/2. + 
     -  (abr1*ak0t0)/2. - (aet3*(M12 + M22))/2. + 
     -  (aet2*(M12 - M32))/2. + (aet1*(M22 - M32))/2.

       coef(3)=-(a0t2*a0t3) + ak0t0*M12 - aet3*M22 - aet2*M32
       coef(4)=-(a0t1*a0t3) - aet3*M12 + ak0t0*M22 - aet1*M32
       coef(5)=(-(a0t2*a0t3*M22) + a0t1*(a0t3*(-2*M12 - M22) + a0t2*M22) 
     -  + aet3*(-2*M14 - M12*M22 - M24) + aet2*(M12*M22 - M22*M32) + 
     -  ak0t0*(M12*M22 + M24 - M22*M32) + 
     -  aet1*(M24 - 2*M12*M32 - M22*M32))/2.
       coef(6)=-(a0t1*a0t3*M12) - a0t2*a0t3*M22 + ak0t0*M12*M22  
     -  + aet3*(-M14 - M24) - aet1*M12*M32 - aet2*M22*M32
       coef(7)=(abr1**2*ak0t0+aet1*(M12*(M22-3*M32)+(M22-M32)**2)+a0t2
     &  *a0t3*(-M12-3*M22+M32)+aet3*(-3*M14-3*M24+M22*M32+M12*(-2*M22+M
     &  32))+a0t1*(a0t2*abr1+a0t3*(-3*M12-M22+M32))+aet2*(M14+M12*(M22-
     &  2*M32)-3*M22*M32+M34))/4.
       coef(8)=(a0t1*(a0t2-a0t3)*M12+aet2*M14-aet3*M14+ak0t0*M14+aet1*
     &  M12*M22-aet3*M12*M22+ak0t0*M12*M22-a0t2*a0t3*(M12+2*M22)-2*aet3
     &  *M24-aet1*M12*M32-aet2*M12*M32-ak0t0*M12*M32-2*aet2*M22*M32)/2.
       coef(9)=-(a0t2*a0t3*abr2)+ak0t0*M14-abr2*(aet3*M22+aet2*M32)
       endif
      endif
      end subroutine FD111C




*
***********************************************************************
*
c Input: M1**2, M2**2, M3**3, Q2.
c output: coef(5) coresponding to 
c     F^D00000_({2,M1}{2,M2}{0,0}{0,0}{1,M3}) coef(1),
c     F^D00001_({2,M1}{2,M2}{0,0}{0,0}{1,M3}) coef(2),  
c     F^D10000_({2,M1}{2,M2}{0,0}{0,0}{1,M3}) coef(3),  
c     F^D00002_({2,M1}{2,M2}{0,0}{0,0}{1,M3}) coef(4),  
c     F^D01000_({2,M1}{2,M2}{0,0}{0,0}{1,M3}) coef(5),
c     F^D10001_({2,M1}{2,M2}{0,0}{0,0}{1,M3}) coef(6),
c     F^D11000_({2,M1}{2,M2}{0,0}{0,0}{1,M3}) coef(7),
c     F^D00001_({2,M1}{1,M3}{0,0}{0,0}{2,M2}) coef(8),
  
      subroutine FD221C(Mass12,Mass22,Mass32,a0t1,a0t2,a0t3,aet1,aet2,
     .                  aet3,Q2,coef,id)
      implicit none
      integer id
      double precision Mass12,Mass32,Mass22
      double precision M12,M22,M32, Q2
      double precision M14,M24,M34,M16,M26,M36,M18,M28,M38,
     .   M1422, M1222,M1224,M1424,M1226,M1434,M1432,M1622,
     .   M2432, M1232,M2232,M1234,M2234
      double precision abr1,abr2,abr3,abr4, alambda, alambda2,
     .   a0t1,a0t2,a0t3,aet1,aet2,aet3
      double precision lambdaN, A0T, AEpT,coef(10)
      double precision K0T0N,ak0t0

      M12=Mass12; M22=Mass22;M32=Mass32
      if(id.eq.2)then
       coef(1) =0D0
       coef(2) =0D0
       coef(3) =-0.5D0
       coef(4) =(-M12 - M22 - 2*M32)/4.
       coef(5) =-0.5D0
       coef(6) =(-M22 - M32)/2.
       coef(7) =-M12 - M22 - M32/2.
       coef(8) =-1/2d0   
      else 

      abr1=(M12 + M22 - M32);abr2=(M12 + M22 + M32)
      abr3=(M12 - M22 - M32);abr4=(M12 - M22 + M32)
      if(id.eq.1)then
       coef(1) =0D0
       coef(2) =-0.5D0
       coef(3) =0.5 - a0t2/M22
       coef(5) =0.5 - a0t1/M12
       coef(4) =(-2*a0t1-2*a0t2-4*a0t3+abr1-3*M12-3*M22+2*M32)/4.
       coef(6)=-a0t2 - a0t3 - M12
       coef(7)=-2*a0t1 - 2*a0t2 - a0t3 - M32/2.
       coef(8)=((-a0t2 + M22))/(M22)

      elseif(id.eq.0)then
      M14=M12**2;  M24=M22**2 ; M34=M32**2
      M16=M12*M14; M26=M22*M24; M36=M32*M34
      M18=M14**2;  M28=M24**2;  M38=M34**2
      M1422=M14*M22; M1222=M12*M22;M1224=M12*M24
      M1424=M14*M24;M1226=M12*M26;M1434=M14*M34
      M1432=M14*M32; M1622=M16*M22;M2432=M24*M32
      M1232=M12*M32; M2232=M22*M32; M1234=M12*M34
      M2234=M22*M34
      alambda=LambdaN(M12,M22,M32)
      alambda2=alambda**2
      ak0t0=K0T0N(M12,M22,M32,aet1,aet2,aet3,Q2)
       coef(1) =(2*abr1*aet1*M32+2*abr1*aet2*M32+2*abr1*aet3*M32+2*abr1
     &  *(abr2+ak0t0)*M32+2*a0t3*(M14-M2232+M24-M12*(2*M22+M32))+(a0t2*
     &  (M18-3*M16*M22+4*M14*M2232+2*M12*M22*M2232+3*M14*M24-M12*M22*M2
     &  4-3*M16*M32-2*M12*M2232*M32+M12*M24*M32+a0t3*M12*(2*M1232-M14+M
     &  24-M34)+3*M14*M34+M12*M22*M34-M12*M32*M34))/(M12*M22)+(a0t1*(-M
     &  16+a0t3*(M14-(M22-M32)**2)+(M22-M32)**3+3*M14*(M22+M32)+(a0t2*(
     &  (-(M22)+M32)**2*M32+M14*M32+2*M12*(M22-M32)*M32))/M22-M12*(-4*
     &  M2232+3*M24+M34)))/M12)/alambda2

       coef(2) =(1+aet1/M12-(2*a0t2)/M22+aet2/M22+(a0t1*(a0t2-2*M22))/(
     &  M12*M22)+(8*aet3*M1222*M32+8*(abr2+ak0t0)*M1222*M32+(2*a0t2*(M1
     &  8-3*M16*M22+3*M14*M2232+7*M12*M22*M2232+3*M14*M24-M12*M22*M24-4
     &  *M16*M32-M22**3*M32+7*M12*M2232*M32-M12*M24*M32-3*M22*M32**3+M3
     &  2**4+2*a0t3*M22*(-M14+M12*(M22+M32))+6*M14*M34-4*M12*M22*M34+3*
     &  M22**2*M34-4*M12*M32*M34))/M22+2*a0t3*(M16+(M22-M32)**3-M14*(M2
     &  2+3*M32)+M12*(2*M2232-M24+3*M34))+aet1*(-(M18/M12)-(M22-M32)**4
     &  /M12+(4*M16*(M22+M32))/M12+4*(M22-M32)**2*(M22+M32)+(2*M14*(2*M
     &  2232-3*(M24+M34)))/M12)-(aet2*(M18+(M22-M32)**4-4*M16*(M22+M32)
     &  +2*M14*(2*M2232+3*(M24+M34))-4*M12*(-M2234+M2432+M26+M36)))/M22
     &  +(a0t1*(4*a0t3*abr4*M1222+2*(-(M16*(M22+M32))+3*M14*(M22+M32)**
     &  2+(M22-M32)**2*((M22-M32)**2-3*M12*(M22+M32)))-(a0t2*(M18+(M22-
     &  M32)**4-4*M16*(M22+M32)+6*M14*(M24+M34)-4*M12*(-2*M2234+M26+M36
     &  )))/M22))/M12)/alambda2)/2.

       coef(3) =((a0t2*(2*M18-8*M16*M22+2*M22**4+9*M14*M2232+M12*M22*M2
     &  232+12*M14*M24-8*M12*M22*M24-7*M16*M32-7*M22**3*M32-M12*M2232*M
     &  32+8*M12*M24*M32-5*M22*M32**3+M32**4-a0t3*(M14*(M22+M32)-(M22-M
     &  32)*(M22**2-2*M12*M32-2*M22*M32+M32**2))+9*M14*M34+5*M12*M22*M3
     &  4+9*M22**2*M34-5*M12*M32*M34))/M22+a0t3*(M16+(M22-M32)**3-M14*(
     &  M22+M32)+M12*(-M24+M34))+aet1*(-M16+(M22-M32)**3+3*M14*(M22+M32
     &  )-M12*(-4*M2232+3*M24+M34))+aet3*(-M16+(M22-M32)**3+3*M14*(M22+
     &  M32)-M12*(-4*M2232+3*M24+M34))+(-abr2-ak0t0)*(M16-(M22-M32)**3-
     &  3*M14*(M22+M32)+M12*(-4*M2232+3*M24+M34))+a0t1*(a0t3*(-M14-3*(M
     &  22-M32)**2+4*M12*(M22+M32))-2*(M16+3*M12*(-M2232+M24)-(M22-M32)
     &  **3-M14*(3*M22+2*M32))+(a0t2*(-M16+3*M14*M22-3*M2234-3*M12*M24+
     &  M26+4*M14*M32-5*M12*M34+2*M36))/M22)+(aet2*(-M18+(M22-M32)**3*M
     &  32+M16*(3*M22+4*M32)-M14*(M2232+3*M24+6*M34)+M12*(-5*M2234+M26+
     &  4*M36)))/M22)/alambda2

       coef(4)=(abr1-(a0t2*(3*M12+3*M22-2*M32))/M22+(aet1*(2*M12+2*M22
     &  -M32))/M12+(aet2*(2*M12+2*M22-M32))/M22+(a0t1*(a0t2*(2*M12+2*M2
     &  2-M32)+M22*(-3*M12-3*M22+2*M32)))/(M12*M22)+(2*a0t3*(2*M18+(2*M
     &  22-3*M32)*(M22-M32)**3-M16*(4*M22+9*M32)+M14*(5*M2232+4*M24+15*
     &  M34)+M12*(10*M2234+5*M2432-4*M26-11*M36))+a0t1*(M18+((3*M22-2*M
     &  32)*(M22-M32)**4)/M12-(M18*(5*M22+8*M32))/M12-(M22-M32)**2*(12*
     &  M2232+7*M24-11*M34)+(2*M16*(7*M2232+3*M24+10*M34))/M12+2*a0t3*(
     &  -M16-(M22-M32)**3+M14*(5*M22+3*M32)-M12*(2*M2232+3*(M24+M34)))-
     &  (a0t2*(2*M12*M18+(M22-M32)**4*(2*M22-M32)-3*M18*(2*M22+3*M32)-2
     &  *M12*(M22-M32)**2*(4*M2232+3*M24-3*M34)+4*M16*(M2232+M24+4*M34)
     &  +2*M14*(8*M2234-3*M2432+2*M26-7*M36)))/(M12*M22)+(2*M14*(7*M223
     &  4+11*M2432+M26-11*M36))/M12)+aet1*(-2*M18-((M22-M32)**4*(2*M22-
     &  M32))/M12+(M18*(6*M22+7*M32))/M12+(2*M16*(3*M2232-2*M24-5*M34))
     &  /M12+2*(M22-M32)**2*(3*M2232+3*M24-2*M34)-(4*M14*(5*M2234+M26-2
     &  *M36))/M12)+(a0t2*(-7*M18*M22+2*M16*M2232-5*M22**3*M2232+28*M14
     &  *M2234+2*M16*M24+M22**3*M24+22*M14*M2432+6*M14*M26-14*M18*M32+1
     &  5*M22**2*M2232*M32-3*M22**2*M24*M32+5*M2232*M32**3-M24*M32**3-2
     &  *a0t3*(M16*M22+M22**4-3*M14*M2232+3*M14*M24-3*M22**3*M32+3*M22*
     &  *2*M32**2-M22*M32**3)+26*M16*M34+2*M22**3*M34-15*M22*M2232*M34+
     &  3*M22*M24*M34-6*M22**2*M32*M34-2*M32**3*M34+6*M22*M34**2+M12*(3
     &  *M18+(M22-M32)*(6*a0t3*M2232+23*M2234+10*a0t3*M24+9*M2432-5*M26
     &  -11*M36))-24*M14*M36))/M22+2*aet3*(-M18-M22*(M22-M32)**3+M16*(4
     &  *M22+3*M32)+M14*(M2232-3*(2*M24+M34))+M12*(-6*M2234+M2432+4*M26
     &  +M36))+2*(ak0t0*(M18+(M22-2*M32)*(M22-M32)**3-M16*(4*M22+5*M32)
     &  +M14*(9*M2232+6*M24+9*M34)+M12*(2*M2234+9*M2432-4*M26-7*M36))+M
     &  32*(-M18+2*M14*(3*M2232+5*M24)-(M22-M32)**3*(M22+M32)+2*M16*(2*
     &  M22+M32)-2*M12*(4*M2234-3*M2432-2*M26+M36)))+(aet2*(-2*M12*M18+
     &  M18*(6*M22+9*M32)+(M22-M32)**3*(M2232-2*M24-M34)-2*M16*(3*M2232
     &  +2*M24+8*M34)-2*M14*(5*M2234+2*M26-7*M36)+2*M12*(3*M28+3*M26*M3
     &  2-10*M24*M34+7*M22*M36-3*M38)))/M22)/alambda2)/4.

       coef(5)=(a0t3*(M16+M2234-M12*M24-M2432+M26-M14*(M22+3*M32)+3*M1
     &  2*M34-M36)+aet2*(M16-M2234+3*M2432-M26-3*M14*(M22+M32)+M12*(4*M
     &  2232+3*(M24+M34))-M36)+aet3*(M16-M2234+3*M2432-M26-3*M14*(M22+M
     &  32)+M12*(4*M2232+3*(M24+M34))-M36)+(abr2+ak0t0)*(M16-M2234+3*M2
     &  432-M26-3*M14*(M22+M32)+M12*(4*M2232+3*(M24+M34))-M36)+a0t2*(-(
     &  a0t3*(3*M14-4*M12*M22-4*M2232+M24-6*M12*M32+3*M34))-2*(-M16-2*M
     &  2432+M26+3*M14*(M22+M32)-3*M12*(M2232+M24+M34)+M36))-(aet1*((M2
     &  2-M32)**4-M16*(M22+M32)+3*M14*(M24+M34)+M12*(5*M2234+M2432-3*(M
     &  26+M36))))/M12+(a0t1*(-8*M1226+12*M1424+9*M1434-8*M1622+2*M18+2
     &  *M28+9*M1224*M32+9*M1422*M32-7*M16*M32-7*M26*M32-a0t3*(3*M1432-
     &  M16+(-(M22)+M32)**2*M32+M12*(2*M2232+M24-3*M34))+a0t2*(-3*M142
     &  2+M16-(M22-2*M32)*(M22-M32)**2+3*M12*(M24-M34))+4*M1222*M34+9*M
     &  24*M34-5*M12*M36-5*M22*M36+M38))/M12)/alambda2

        coef(6)=aet1+M12/2.+a0t1*(-1.5+a0t2/M22)-(3*a0t2*M12)/(2.*M22)+ 
     -  (aet2*M12)/M22+(aet1*(M1226-3*M1424-3*M1434+3*M1622-M18 - 
     -  M1224*M32 + 2*M1422*M32 + 3*M16*M32 - M1222*M34 + M12*M36) + 
     -  aet3*(M1226 - 3*M1424 - 3*M1434 + 3*M1622 - M18 - M1224*M32 + 
     -  2*M1422*M32 + 3*M16*M32 - M1222*M34 + M12*M36) + 
     -  a0t3*(-2*M1226 + 2*M1424 + 6*M1434 - 2*M1622 + M18 + M28 + 
     -  4*M1422*M32 - 4*M16*M32 - 4*M26*M32 + 6*M1222*M34 + 6*M24*M34- 
     -  4*M12*M36 - 4*M22*M36 + M38) + 
     -  ak0t0*(-3*M1226 + 3*M1424 + 3*M1434-M1622+M28+3*M1224*M32 + 
     -  6*M1422*M32 - M16*M32 - 4*M26*M32 + 3*M1222*M34 + 6*M24*M34 - 
     -  3*M12*M36 - 4*M22*M36 + M38) - 
     -  (aet2*(M12*M18 - M1424*M22 + 7*M1434*M22 + 3*M1622*M22 - 
     -  3*M18*M22 - 4*M18*M32 - 3*M1224*M22*M32 + 2*M1422*M22*M32 + 
     -  M16*M22*M32 + 6*M16*M34 + 7*M1222*M22*M34 - 4*M14*M36 - 
     -  5*M12*M22*M36 + M12*M38))/M22 + 
     -  a0t2*((-5*M1226)/2. + 8*M1434 + 5*M1622 - 5*M18 + 
     -  (3*M12*M18)/(2.*M22) + M28 + 10*M1422*M32 + 4*M16*M32 - 
     -  (6*M18*M32)/M22 - 4*M26*M32 - 
     -  2*a0t3*(-M1224 - M1234 + M1422 + M1432 + 2*M1222*M32) + 
     -  9*M1222*M34 + (9*M16*M34)/M22 + 6*M24*M34 - 8*M12*M36 - 
     -  (6*M14*M36)/M22 - 4*M22*M36 + M38 + (3*M12*M38)/(2.*M22)) + 
     -  (-(M12*M18) + M18*M22 + 2*M16*M24 - 2*M14*M26 - M12*M28 + 
     -  M22*M28 - 4*M1226*M32 + 10*M1424*M32 + 12*M1622*M32 + M18*M32- 
     -  3*M28*M32 + 10*M1224*M34 + 10*M1422*M34 + 2*M16*M34 + 
     -  2*M26*M34 - 4*M1222*M36 - 2*M14*M36 + 2*M24*M36 - M12*M38 - 
     -  3*M22*M38 + M32*M38)/2. + 
     -  a0t1*(-4*M1226 + 3*M1424 + 3*M1434 - M18/2. + (3*M28)/2. + 
     -  4*M1224*M32 + 6*M1422*M32 - 6*M26*M32 + 4*M1222*M34 + 
     -  9*M24*M34 - 4*M12*M36 - 6*M22*M36 + 
     -  a0t3*(-M1224 - 3*M1234 + 3*M1422 + 3*M1432 - M16 - 3*M2234 + 
     -  3*M2432 - M26 + 4*M1222*M32 + M36) + (3*M38)/2. - 
     -  (a0t2*(6*M1434 + M18 - M1224*M22 + 5*M1234*M22 + 3*M1422*M22+ 
     -  M1432*M22 - 3*M16*M22 + 3*M22*M2234 - M22*M2432 - 
     -  4*M16*M32 - 4*M12*M36 - 3*M22*M36 + M38))/M22))/alambda2

        coef(7)=(M12*M18-3*M18*M22+2*M16*M24+2*M14*M26-3*M12*M28+M22* 
     -  M28-2*M1226*M32+16*M1424*M32-2*M1622*M32-2*M18*M32-2*M28*M32+ 
     -  8*M1224*M34+8*M1422*M34-2*M1222*M36+2*M14*M36 + 2*M24*M36 + 
     -    aet3*(-4*M1226 + 6*M1424 + 3*M1434 - 4*M1622 + M18 + M28 + 
     -  5*M1224*M32+5*M1422*M32 - 3*M16*M32 - 3*M26*M32+3*M24*M34 - 
     -  M12*M36 - M22*M36) + 
     -  a0t3*(-2*M1226 + 2*M1424 + 3*M1434 - 2*M1622 + M18 + M28 + 
     -  M1224*M32 + M1422*M32 - 3*M16*M32 - 3*M26*M32 + 2*M1222*M34 + 
     -  3*M24*M34 - M12*M36 - M22*M36) + 
     -  aet1*(-3*M1434 + M1224*M32 + M1422*M32 + M16*M32 + M26*M32 - 
     -  4*M1222*M34 - 3*M24*M34 + 3*M12*M36 + 3*M22*M36 - M38) + 
     -  aet2*(-3*M1434 + M1224*M32 + M1422*M32 + M16*M32 + M26*M32 - 
     -  4*M1222*M34 - 3*M24*M34 + 3*M12*M36 + 3*M22*M36 - M38) - 
     -  M12*M38 - M22*M38 + a0t2*
     -  (-9*M1226 + 15*M1424 + 12*M1434 - 11*M1622 + 3*M18 + 2*M28 + 
     -  10*M1224*M32 + 11*M1422*M32 - 10*M16*M32 - 7*M26*M32 + 
     -  5*M1222*M34 + 9*M24*M34 + 
     -  a0t3*(-2*M1224 + 3*M1422 + 3*M1432 - 2*M16 + 3*M2234 - 3*M2432 + 
     -  M26 + 2*M1222*M32 - M36) - 6*M12*M36 - 5*M22*M36 + M38) + 
     -  ak0t0*(-8*M1226 + 12*M1424 + 9*M1434 - 8*M1622 + 2*M18 + 2*M28 + 
     -  9*M1224*M32 + 9*M1422*M32 - 7*M16*M32 - 7*M26*M32 + 
     -  4*M1222*M34 + 9*M24*M34 - 5*M12*M36 - 5*M22*M36 + M38) + 
     -  a0t1*(-11*M1226 + 15*M1424 + 9*M1434 - 9*M1622 + 2*M18 + 3*M28 + 
     -  11*M1224*M32 + 10*M1422*M32 - 7*M16*M32 - 10*M26*M32 + 
     -  5*M1222*M34 + 12*M24*M34 + 
     -  a0t3*(3*M1224 + 3*M1234 - 2*M1422 - 3*M1432 + M16 + 3*M2432 - 
     -  2*M26 + 2*M1222*M32 - M36) - 5*M12*M36 - 6*M22*M36 + 
     -  a0t2*(-6*M1234 + 3*M1432 - 6*M2234 + 3*M2432 - 2*M1222*M32 + 
     -  3*M36) + M38))/alambda2

        coef(8)=(4*a0t3*abr3*M1232-3*M18-16*M1224*M32+8*M16*(M22+M32)+ 
     -  (M22 - M32)**3*(M22 + 3*M32) - 2*M14*(-4*M2232 + 3*M24 + M34)+ 
     -  2*(aet1+aet3+ak0t0)*(-M16+(M22-M32)**3+3*M14*(M22+M32)-M12* 
     -  (3*M24+M34))+(2*a0t2*(2*M18 +(M22-M32)**3*(a0t3+3*M22 - M32) - 
     -  M16*(9*M22 + 7*M32)+M14*(10*M2232+15*M24+a0t3*(M22-M32)+9*M34)+ 
     -  M12*(5*M2234 + 7*M2432 - 11*M26 - 2*a0t3*(2*M2232 + M24 - M34)- 
     -  5*M36)))/M22 - (2*a0t1*(M22*(M16 - (M22 - 3*M32)*(M22 - M32)**2
     -  - M14*(3*M22 + M32) +M12*(-4*M2232 + 3*M24 - 3*M34) + 
     -  a0t3*(M14 - 4*M2232 + M24 - 2*M12*(M22 + 2*M32) + 3*M34)) + 
     -  a0t2*(M16 + M2234 + 2*M2432 - M26 - M14*(3*M22 + 4*M32) + 
     -  M12*(2*M2232 + 3*M24 + 5*M34) - 2*M36)))/M22 + 
     -  (2*aet2*(-M18 + (M22 - M32)**3*M32 + M16*(3*M22 + 4*M32) - 
     -  M14*(M2232+3*M24+6*M34)+M12*(-5*M2234-4*M2432+M26+4*M36)
     -  ))/M22)/(2.*alambda2)
        endif
      endif

      end subroutine FD221C

*
***********************************************************************
*
c Input: M1**2, M2**2, M3**2, Q2.
c output: coef(5) coresponding to 
c     F^D_({3,M1}{1,M3}{0,0}{0,0}{1,M2}) coef(1),
c     F^D00001_({3,M1}{1,M3}{0,0}{0,0}{1,M2}) coef(2),  
c     F^D10000_({3,M1}{1,M3}{0,0}{0,0}{1,M2}) coef(3),  
c     F^D10001_({3,M1}{1,M3}{0,0}{0,0}{1,M2}) coef(4),
c     F^D01000_({3,M1}{1,M3}{0,0}{0,0}{1,M2}) coef(5), 
c     F^D11000_({3,M1}{1,M3}{0,0}{0,0}{1,M2}) coef(6), 
c     F^D20000_({3,M1}{1,M3}{0,0}{0,0}{1,M2}) coef(7), 
  
      subroutine FD311C(Mass12,Mass32,Mass22,a0t1,a0t3,a0t2,aet1,aet3,
     .    aet2,Q2,coef,id)
      implicit none
      integer id
      double precision Mass12,Mass32,Mass22
      double precision M12,M22,M32, Q2
      double precision M14,M24,M34,M16,M26,M36,M18,M28,M38,
     .   M1422, M1222,M1224,M1424,M1226,M1434,M1432,M1622,
     .   M2432, M1232,M2232,M1234,M2234
      double precision  alambda, alambda2,
     .   a0t1,a0t2,a0t3,aet1,aet2,aet3,abr1,abr2,abr3,abr4
      double precision lambdaN, A0T, AEpT
      double precision K0T0N, coef(10),ak0t0

      M12=Mass12; M22=Mass22; M32=Mass32
      if (id.eq.2) then
       coef(1)=0D0
       coef(2)=-0.25D0
       coef(3) =-0.5D0
       coef(4)=(-3*M12 - 2*M22)/4.
       coef(5)=0
       coef(6)=-M22 - M32/2.
       coef(7)=(-3*M12-M22-M32)/2.
      else
      M14=M12**2;  M24=M22**2 ; M34=M32**2
      M16=M12*M14; M26=M22*M24; M36=M32*M34
      M18=M14**2;  M28=M24**2;  M38=M34**2
      M1422=M14*M22; M1222=M12*M22;M1224=M12*M24
      M1424=M14*M24;M1226=M12*M26;M1434=M14*M34
      M1432=M14*M32; M1622=M16*M22;M2432=M24*M32
      M1232=M12*M32; M2232=M22*M32; M1234=M12*M34
      M2234=M22*M34
      alambda=LambdaN(M12,M22,M32)
      alambda2=alambda**2
       if (id.eq.1) then
       coef(1)=1/(2.*M12)
       coef(2)=(-a0t1 + M12)/(2.*M12)
       coef(3)= 1 - a0t1/M12
       coef(4)=(-3*a0t1 - 2*a0t2 + M12 - M32)/2.
       coef(5)=(M22 + M32)/(2.*M12)
       coef(6)=-a0t2 + (3*M22)/2. + M32 - (a0t1*(M22 + M32))/M12
       coef(7)=-3*a0t1 - a0t2 - a0t3 + M12 - M22/2. - M32/2.
       elseif (id.eq.0) then
      abr1=M12 + M22 - M32; abr2=M12 + M22 + M32
      abr3=M12 - M22 - M32; abr4=M12 - M22 + M32
      ak0t0= K0T0N(M12,M22,M32,aet1,aet2,aet3,Q2) 
        coef(1)=-(4*aet1*M12**2*M2232+4*aet2*M12**2*M2232+4*aet3*M12**2
     &  *M2232-a0t1*(2*M18+2*M14*M2232+9*M14*M24-5*M12*M24*(M22-M32)+(M
     &  22-M32)**4-2*a0t2*abr1*M12*M32-7*M16*(M22+M32)-2*a0t3*M22*(M14+
     &  M12*(-M22+M32))+9*M14*M34+5*M12*(M22-M32)*M34)-a0t2*M12*(2*a0t3
     &  *abr3*M12-M16+3*M14*M22-2*M12*M2232-2*M22*M2232-3*M12*M24+M22*M
     &  24+M14*M32+2*M2232*M32-M24*M32+M12*M34+M22*M34-M32*M34)+a0t3*M1
     &  2*(M16+(M22-M32)**3-M14*(M22+3*M32)+M12*(2*M2232-M24+3*M34))+M1
     &  2*(M18+4*ak0t0*M12*M2232+8*M12*M2432-4*M12*M26+(M22-M32)**4+8*M
     &  12*M2232*M32-4*M16*(M22+M32)+M14*(8*M2232+6*(M24+M34))-4*M12*M3
     &  6))/(2.*alambda2*M14)

       coef(2)=(-(a0t3/M12)+M32/M12-(a0t1*M32)/M14+(2*aet2*(M16-M2234+
     &  3*M2432-M26-3*M14*(M22+M32)+3*M12*(M24+M34)-M36)+2*aet3*(M16-M2
     &  234+3*M2432-M26-3*M14*(M22+M32)+3*M12*(M24+M34)-M36)-2*a0t2*(-M
     &  16+3*M14*M22-4*M12*M2232-3*M2234-3*M12*M24-M2432+M26+5*M14*M32-
     &  7*M12*M34+a0t3*(M14-4*M2232+M24-2*M12*(M22+2*M32)+3*M34)+3*M36)
     &  +a0t3*(M18/M12+(M22-M32)**4/M12-(4*M16*(M22+M32))/M12+(6*M14*(M
     &  24+M34))/M12-4*(-2*M2432+M26+M36))-(2*aet1*((M22-M32)**4-M16*(M
     &  22+M32)+M14*(4*M2232+3*(M24+M34))+M12*(5*M2234+M2432-3*(M26+M36
     &  ))))/M12+a0t1*((6*M12*M18)/M14+((M22-M32)**4*M32)/M14-(M18*(22*
     &  M22+19*M32))/M14-(2*M12*(M22-M32)**2*(5*M2232-2*M24+M34))/M14+(
     &  10*M16*(M2232+3*M24+2*M34))/M14+(2*a0t2*M12*(M16-(M22-2*M32)*(M
     &  22-M32)**2-M14*(3*M22+2*M32)-M12*(2*M2232-3*M24+M34)))/M14+(2*a
     &  0t3*M12*(M16-(-(M22)+M32)**2*M32-M14*(2*M22+3*M32)+M12*(-4*M22
     &  32+M24+3*M34)))/M14+2*(7*M2234+13*M2432-3*(3*M26+M36)))-(6*M16*
     &  M24-8*M14*M26-12*M1226*M32-2*M1424*M32+12*M1622*M32+M18*M32+M28
     &  *M32+6*M1224*M34+4*M1422*M34+2*M16*M34-4*M26*M34+4*M1222*M36-2*
     &  M14*M36+6*M24*M36+2*ak0t0*M12*(-M16+M2234-3*M12*M24-3*M2432+M26
     &  +3*M14*(M22+M32)-3*M12*M34+M36)-4*M22*M38+M32*M38-M12*(M18-3*M2
     &  8+M38))/M12)/alambda2)/4.

       coef(3)=(aet2*(M16-3*M14*(M22+M32)-(M22-M32)**2*(M22+M32)+3*M12
     &  *(M24+M34))+aet3*(M16-3*M14*(M22+M32)-(M22-M32)**2*(M22+M32)+3*
     &  M12*(M24+M34))+(a0t3*(M16-(M22-M32)**2*(3*M22+M32)-M14*(5*M22+3
     &  *M32)+M12*(2*M2232+7*M24+3*M34)))/2.+(aet1*(-(M22-M32)**4+M16*(
     &  M22+M32)+3*M12*(M22-M32)**2*(M22+M32)-M14*(4*M2232+3*(M24+M34))
     &  ))/M12+a0t2*(a0t3*(-M14-2*(M22-M32)**2+3*M12*(M22+M32))+(M16-(M
     &  22-M32)**2*(M22+3*M32)-M14*(3*M22+5*M32)+M12*(2*M2232+3*M24+7*M
     &  34))/2.)+a0t1*((a0t2*(M16+3*M12*(-M2232+M24)-(M22-M32)**3-M14*(
     &  3*M22+2*M32)))/M12+(a0t3*(M16+(M22-M32)**3-M14*(2*M22+3*M32)-3*
     &  M12*(M2232-M34)))/M12+((6*M18)/M12+(3*(M22-M32)**4)/M12-(21*M16
     &  *(M22+M32))/M12-15*(M22-M32)**2*(M22+M32)+(M14*(14*M2232+27*(M2
     &  4+M34)))/M12)/2.)+(M18-(M22-M32)**2*(2*M2232+3*(M24+M34))-2*M14
     &  *(8*M2232+3*(M24+M34))+2*ak0t0*(M16-3*M14*(M22+M32)-(M22-M32)**
     &  2*(M22+M32)+3*M12*(M24+M34))+4*M12*(M2234+M2432+2*(M26+M36)))/2
     &  .)/alambda2

        coef(4)=(-3*a0t3+2*aet3+(a0t1*(2*a0t3-3*M32))/M12+M32+(2*aet1*M
     &  32)/M12+(3*M12*M18-6*M18*M22-2*M16*M24+12*M14*M26-9*M12*M28+2*M
     &  22*M28+4*M1226*M32+22*M1424*M32-20*M1622*M32-3*M18*M32-3*M28*M3
     &  2-2*M1224*M34-6*M16*M34+a0t3*(3*M18+12*M12*(M2432-M26)+(M22-M32
     &  )**3*(3*M22+M32)-4*M16*(3*M22+2*M32)+2*M14*(2*M2232+9*M24+3*M34
     &  ))+2*a0t2*(3*M18+2*M22*(M22-M32)**3-11*M16*(M22+M32)+M14*(10*M2
     &  232+15*M24+13*M34)+a0t3*(-3*M16+2*(M22-M32)**3+M14*(8*M22+6*M32
     &  )-M12*(-8*M2232+7*M24+M34))+M12*(7*M2234+7*M2432-9*M26-5*M36))+
     &  2*ak0t0*(3*M18+2*M22*(M22-M32)**3-M16*(11*M22+9*M32)+3*M14*(2*M
     &  2232+5*M24+3*M34)+3*M12*(M2234+3*M2432-3*M26-M36))+4*M1222*M36+
     &  6*M14*M36-2*M24*M36+2*aet3*(2*M18+(M22-M32)**3*(M22+M32)-M16*(7
     &  *M22+5*M32)+M14*(2*M2232+9*M24+3*M34)+M12*(-M2234+5*M2432-5*M26
     &  +M36))+a0t1*(10*M18+(3*(M22-M32)**4*M32)/M12-(M18*(38*M22+29*M3
     &  2))/M12+2*a0t2*(M16-(M22-6*M32)*(M22-M32)**2+M14*(-3*M22+2*M32)
     &  +M12*(-10*M2232+3*M24-9*M34))-2*(M22-M32)**2*(11*M2232-4*M24+5*
     &  M34)+(2*M16*(7*M2232+27*M24+12*M34))/M12+(2*M14*(15*M2234+25*M2
     &  432-17*M26+M36))/M12+2*a0t3*((2*M18)/M12-3*M2432+2*M26-(M22-M32
     &  )**4/M12-(M16*(4*M22+5*M32))/M12+(M14*(-4*M2232+M24+3*M34))/M12
     &  +M36))+2*aet2*(M18+2*(M22-M32)**3*M32-M16*(3*M22+M32)+M14*(-2*M
     &  2232+3*M24-3*M34)+M12*(-5*M2234+M2432-M26+5*M36))+aet1*((-2*(M2
     &  2-M32)**4*M32)/M12+(2*M18*(M22+2*M32))/M12+2*(M22-M32)**2*(8*M2
     &  232-M24+M34)-(2*M16*(2*M2232+3*M24+5*M34))/M12-(2*M14*(13*M2234
     &  +9*M2432-3*(M26+M36)))/M12)+3*M12*M38+6*M22*M38-3*M32*M38)/alam
     &  bda2)/4.

        coef(5)=(-4*aet1*M12**2*M2234-4*aet2*M12**2*M2234-4*aet3*M12**2
     &  *M2234+a0t2*M12*(M18-4*M16*M22-2*a0t3*M12*M2232+7*M14*M2232+M22
     &  *M2234+6*M14*M24+M12*M2432-2*M22*M2432-4*M12*M26+M22*M26+2*a0t3
     &  *M14*M32-5*M16*M32+2*M12*M2232*M32-M2234*M32+2*M2432*M32-M26*M3
     &  2-2*a0t3*M12*M34+7*M14*M34-3*M12*M32*M34)+a0t3*M12*(-2*M12*M223
     &  4-M16*M32+M14*M22*M32+M12*M24*M32-(M22-M32)**3*M32+3*M14*M34-3*
     &  M12*M36)+a0t1*(13*M14*M2432+6*M14*M26-5*M12*M2432*(M22-M32)-4*M
     &  12*M26*(M22-M32)+4*M12*M2232*(M22-M32)*M32+(M22-M32)**4*(M22+M3
     &  2)+M18*(M22+2*M32)+2*a0t3*(M12*M2232*(M22-M32)-M14*M22*M32)-2*a
     &  0t2*abr1*M12*M34+8*M14*M22*M34-M16*(11*M2232+4*M24+7*M34)+9*M14
     &  *M36+5*M12*(M22-M32)*M36)+M12*(8*M16*M2232-4*ak0t0*M12*M2234+4*
     &  M16*M24-10*M14*M2432-6*M14*M26+4*M12*M28-M24*(M22-M32)**3-4*M12
     &  *M2234*M32-M18*(M22+M32)+4*M16*M34-14*M14*M22*M34-12*M12*M24*M3
     &  4+(M22-M32)**3*M34-6*M14*M36+4*M12*M38))/(2.*alambda2*M14)

        coef(6)=(M18*(-M22+M32)+2*M16*(2*M2232+2*M24+ak0t0*M32)-(M22-M3
     &  2)**2*(M22+M32)*(M24+2*ak0t0*M32+3*M34)+2*aet3*M32*(M16-3*M14*(
     &  M22+M32)-(M22-M32)**2*(M22+M32)+3*M12*(M24+M34))+a0t3*M32*(M16-
     &  (M22-M32)**2*(3*M22+M32)-M14*(5*M22+3*M32)+M12*(2*M2232+7*M24+3
     &  *M34))-2*M14*(5*M2432+3*M26+3*(ak0t0+M32)*M34+M22*(3*ak0t0*M32+
     &  11*M34))+2*aet2*(-M18+M16*(4*M22+5*M32)+(M22-M32)**2*(M2232-M24
     &  -2*M34)-M14*(7*M2232+6*M24+9*M34)-M12*(4*M2234+M2432-4*M26-7*M3
     &  6))+a0t2*(3*M18+9*M14*M2232+18*M14*M24+15*M12*M2432-12*M12*M26-
     &  7*M2232*(M22-M32)**2+3*M24*(M22-M32)**2-M16*(12*M22+11*M32)+13*
     &  M14*M34+14*M12*M22*M34+a0t3*(-2*M14*M32-4*(M22-M32)**2*M32+6*M1
     &  2*(M22*M32+M34))-5*M12*M36)-(2*aet1*(M18*M22+(M22-M32)**4*(M22+
     &  M32)-M16*(5*M2232+4*M24+M34)-M12*(M22-M32)**2*(7*M2232+4*M24+3*
     &  M34)+M14*(10*M2234+7*M2432+6*M26+3*M36)))/M12+2*a0t1*((a0t3*(M1
     &  6*M32-M14*(2*M2232+3*M34)+(M22-M32)*(M22**2*M32+M32**3-3*M12*M3
     &  4-2*M22*M34)))/M12+(3*(M22-M32)*(4*M2234-5*M2432-4*M26+5*M36)+(
     &  3*M22**3*M24+39*M14*M2432+18*M14*M26+32*M14*M2232*M32-9*M22**2*
     &  M24*M32-3*M24*M32**3+3*M18*(M22+2*M32)-3*M22**3*M34+9*M22*M24*M
     &  34+9*M22**2*M32*M34+3*M32**3*M34-9*M22*M34**2-3*M16*(11*M2232+4
     &  *M24+7*M34)+27*M14*M36)/M12)/2.-(a0t2*(M18+M22*(M22-M32)**3-M16
     &  *(4*M22+5*M32)+M14*(7*M2232+6*M24+8*M34)+M12*(7*M2234+M2432-4*(
     &  M26+M36))))/M12)+M12*(6*ak0t0*(M2432+M36)+4*(M28+M26*M32+2*M22*
     &  M36+2*M38)))/(2.*alambda2)

       coef(7)=(4*aet1*(-(M22-M32)**4+M16*(M22+M32)+3*M12*(M22-M32)**2
     &  *(M22+M32)-3*M14*(M2232+M24+M34))+a0t3*M12*(3*M16-(M22-M32)**2*
     &  (5*M22+3*M32)-M14*(11*M22+9*M32)+M12*(6*M2232+13*M24+9*M34))+a0
     &  t2*M12*(3*M16-(M22-M32)**2*(3*M22+5*M32)-M14*(9*M22+11*M32)-2*a
     &  0t3*(3*M14+4*(M22-M32)**2-7*M12*(M22+M32))+M12*(6*M2232+9*M24+1
     &  3*M34))+2*ak0t0*(3*M18+(M22-M32)**4-10*M16*(M22+M32)-6*M12*(M22
     &  -M32)**2*(M22+M32)+6*M14*(M2232+2*(M24+M34)))+4*aet2*M12*(M16-3
     &  *M14*(M22+M32)-(M22-M32)**2*(M22+M32)+M12*(M2232+3*(M24+M34)))+
     &  4*aet3*M12*(M16-3*M14*(M22+M32)-(M22-M32)**2*(M22+M32)+M12*(M22
     &  32+3*(M24+M34)))+2*a0t1*(a0t3*(2*M16+2*(M22-M32)**3-3*M14*(M22+
     &  2*M32)-M12*(5*M2232+M24-6*M34))+a0t2*(2*M16-2*(M22-M32)**3-3*M1
     &  4*(2*M22+M32)-M12*(5*M2232-6*M24+M34))+(10*M18+5*(M22-M32)**4-3
     &  5*M16*(M22+M32)-25*M12*(M22-M32)**2*(M22+M32)+M14*(26*M2232+45*
     &  (M24+M34)))/2.)+M12*(3*M18-4*M16*(M22+M32)-6*M14*(4*M2232+M24+M
     &  34)-(M22-M32)**2*(6*M2232+5*(M24+M34))+4*M12*(4*M2234+4*M2432+3
     &  *(M26+M36))))/(2.*alambda2)

        endif
      endif
      end subroutine FD311C

*
***********************************************************************
*
c Input: M1**2, M2**2, M3**3, Q2.
c output: coef(5) coresponding to 
c1     F^D_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(1),
c2     F^D00001_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(2),  
c3     F^D01000_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(3),  
c4     F^D10000_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(4),  
c5     F^D10001_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(5), 
c6     F^D11000_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(6), 
c7     F^D00002_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(7),
c8     F^D20000_({2,M2}{1,M1}{0,0}{0,0}{1,M3}) coef(8),
  
      subroutine FD211C(Mass22,Mass12,Mass32,a0t2,a0t1,a0t3,aet2,aet1,
     .    aet3,Q2,coef,id)
      implicit none
      integer id
      double precision Mass12,Mass32,Mass22
      double precision M12,M22,M32, Q2,pi
      double precision M14,M24,M34,M16,M26,M36,M18,M28,M38,
     .   M1422, M1222,M1224,M1424,M1226,M1434,M1432,M1622,
     .   M2432, M1232,M2232,M1234,M2234
      double precision abr11,abr12,abr13,abr14, alambda,
     .   a0t1,a0t2,a0t3,aet1,aet2,aet3
      double precision lambdaN
      double precision K0T0N, coef(10),ak0t0

      M12=Mass12;M22=Mass22;M32=Mass32
      M14=M12**2;  M24=M22**2 ; M34=M32**2
      M16=M12*M14; M26=M22*M24; M36=M32*M34
      M18=M14**2;  M28=M24**2;  M38=M34**2
      M1422=M14*M22; M1222=M12*M22;M1224=M12*M24
      M1424=M14*M24;M1226=M12*M26;M1434=M14*M34
      M1432=M14*M32; M1622=M16*M22;M2432=M24*M32
      M1232=M12*M32; M2232=M22*M32; M1234=M12*M34
      M2234=M22*M34
       pi= 3.1415926535897932384626433832795029D0
      if (id.eq.2) then
       coef(1)=-0.5D0
       coef(2)= (-M22 - M32)/2.
       coef(3)=-M12/2. - M32
       coef(4)=(-M12 - 2*M22 - M32)/2.
       coef(5) =(-2*M1232 - M14 - 4*M2232 - 3*M24 + M34)/4.
       coef(6)= (-M14 - 4*M2232 - M12*(2*M22 + M32))/2.
       coef(7)=(-M14 - 10*M2232 - 3*M24-2*M12*(M22+2*M32)+3*M34)/8.
       coef(8)=(-3*M24)/2. - M22*M32 - M12*(M22 + M32)
      elseif (id.eq.1)then
       coef(1)= 0.5D0 - a0t2/M22
       coef(2)= -a0t2 - a0t3 - M12/2.
       coef(3)=-a0t3 + M12/2. + M32 - (a0t2*(M12 + M32))/M22
       coef(4)=-a0t1 - 2*a0t2 - a0t3 + (-M12 - M32)/2.
       coef(5) =-(a0t1*M12)/2. - (3*a0t2*M22)/2. + 
     -  (a0t3*(-2*M12 - 4*M22+M32))/2.+(-4*M1222-M14-M24+M34)/4.
       coef(6)= -(a0t1*M12) - M1232/2. - M14/2. + a0t3*(-M12 - 2*M22)
     -   + M2232 - 2*a0t2*(M12 + M32)
       coef(7)=-(a0t1*M12)/4. + (a0t2*(-2*M12 - 3*M22 - 2*M32))/4. + 
     -  a0t3*(-M12 - 2*M22 + (3*M32)/4.) + 
     -  (-4*M1222 + 2*M1232 - 3*M14 + 4*M2232 - M24 + M34)/8.
       coef(8)=-(a0t3*(M12 + 2*M22)) - a0t1*(2*M22 + M32) - 
     -  (M22*(6*a0t2 + 2*M12 + M22 + 2*M32))/2.

      elseif(id.eq.0)then
      alambda=LambdaN(M12,M22,M32)
      abr11=(M12 + M22 - M32);abr12=(M12 + M22 + M32)
      abr13=(M12 - M22 - M32);abr14=(M12 - M22 + M32)
      ak0t0= K0T0N(M12,M22,M32,aet1,aet2,aet3,Q2) 
      if(M32.eq.0 .AND. M22.eq.M12) then
      coef(1)=-(a0t2**2 - 2*a0t2*M22 + 2*aet2*M22)/(2.*M24)
      coef(2)=-a0t2 - aet2 - a0t2**2/(2.*M22) - M22
      coef(3)=a0t2 - aet2 - a0t2**2/(2.*M22)
      coef(4)=-a0t2 - 3*aet2 - (3*a0t2**2)/(2.*M22) - 2*M22
      elseif(M32.eq.0 .AND. M12.eq.0) then
       coef(2)=a0t2 - aet2 - (M22*(12+Pi**2-12*Log(M22/Q2)+
     &     3*Log(M22/Q2)**2))/6.
      elseif(M22.eq.0) then
      
      else


      coef(1)=(-((a0t1*(2*a0t3+abr14)+abr14*(a0t3+abr12+aet1+aet
     &  3+ak0t0))*M22)+aet2*(-M14+M12*M22+2*M12*M32+M22*M32-M34)+a0t2*(
     &  a0t3*abr11-a0t1*abr13+M14-3*M12*M22+2*M24-2*M12*M32-3*M22*M32+M
     &  34))/(alambda*M22)

      coef(2)=(aet1-(a0t2*M12)/M22+(aet2*M12)/M22+(a0t1*(a0t2-M2
     &  2))/M22+(-2*abr13*aet3*M12-M16-(a0t1*abr11*(a0t2*abr13+(2*a0t3+
     &  abr14)*M22))/M22+aet1*(-M14+(M22-M32)**2)-M14*(M22+M32)+(M22-M3
     &  2)**2*(2*ak0t0+M22+M32)+(a0t2*(-M1224+M1234-2*M1422-2*M1432+M16
     &  +4*a0t3*M12*M22+2*M2234-4*M2432+2*M26-4*M1222*M32))/M22+M12*((M
     &  22-M32)**2-2*ak0t0*(M22+M32))+2*a0t3*((M22-M32)**2-M12*(M22+M32
     &  ))+(aet2*M12*(2*M1232-M14+4*M2232+M24-M34))/M22)/alambda)/2.


       coef(3)=(-(abr14*aet1*M12)-abr14*(abr12+ak0t0)*M12-(a0t1*M
     &  12*(a0t2*abr13+(2*a0t3+abr14)*M22))/M22+aet3*(-2*M14-(M22-M32)*
     &  *2+M12*(3*M22+M32))+a0t3*((M22-M32)**2-M12*(M22+3*M32))+(a0t2*(
     &  2*M1224-M1234+M16-2*M2234+M2432-5*M1222*M32+M32**3-M14*(3*M22+M
     &  32)+a0t3*(-(M22-M32)**2+M12*(3*M22+M32))))/M22+(aet2*(-M16-(-(M
     &  22)+M32)**2*M32+M14*(M22+M32)+M12*(3*M2232+M34)))/M22)/alambda

       coef(4)=-((a0t3*abr14*M22+abr14*aet1*M22+abr14*aet3*M22+a0
     &  t1*(a0t2*abr13+(2*a0t3+abr14)*M22)+aet2*(M14+M32*(-M22+M32)-M12
     &  *(M22+2*M32))+M22*(2*M1232+M14-M24+M34)-ak0t0*(-3*M1222-2*M1232
     &  +M14-3*M2232+2*M24+M34)-a0t2*(a0t3*abr11-3*M1222-2*M1232+M14-3*
     &  M2232+2*M24+M34))/alambda)

       coef(5)=-(a0t2*M12)/2.+aet2*M12+a0t1*(a0t2-M22/2.)+aet1*M2
     &  2+(-(ak0t0*(3*M1224-3*M1234-M16+M14*M22-3*M22**3-5*M2234+7*M243
     &  2+4*M1222*M32+3*M14*M32+M36))+2*a0t3*M22*((M22-M32)**2-M12*(
     &  M22+M32))+M22*(-M16+abr12*(M22-M32)**2-M14*(M22+M32))+aet1*((M2
     &  2-M32)**3+2*M1232*(M22+M32)-M14*(M22+M32))+a0t1*(M22*(-M14+(M22
     &  -M32)**2)+a0t3*(2*M1232-M14+4*M2232-3*M24-M34)+a0t2*(2*M1232-M1
     &  4+M24-M34))-aet3*(M16+M26+M14*M22-6*M12*M2232-3*M12*M24-2*M14
     &  *M32-2*M24*M32+M22*M34+M12*M34)+a0t2*(M16-2*M14*(M22+M32
     &  )+2*(M22-M32)**2*M22+a0t3*(-M14-(M22-M32)**2+2*M12*(3*M22+M32)
     &  )+M12*(-4*M2232-M24+M34))+aet2*(M1432-M16-(-(M22)+M32)**2*M32+
     &  M12*(6*M2232+M24+M34)))/(2.*alambda)

       coef(6)=(-(abr14*aet1*M1222)-a0t1*(a0t2*abr13*M12+(2*a0t3+
     &  abr14)*M1222)+a0t3*M22*((M22-M32)**2-M12*(M22+3*M32))+aet3*M22*
     &  (-3*M14-2*(M22-M32)**2+M12*(5*M22+3*M32))+a0t2*(M16+(-(M22)+
     &  M32)**2*M32-M14*(3*M22+M32)+a0t3*(-M14-2*(M22-M32)**2+M12*(5*M22+
     &  3*M32))-M12*(5*M2232-2*M24+M34))+M12*(-(M22*(2*M1232+M14-M24+M3
     &  4))+ak0t0*(-3*M1222-2*M1232+M14-3*M2232+2*M24+M34))+aet2*(M1422
     &  -M16-2*(-(M22)+M32)**2*M32+M12*(5*M2232+3*M34)))/alambda

       coef(7)=((a0t1*(-(abr11*M22)+a0t2*(M12+2*M22-M32)))/M22+ae
     &  t1*(M12+2*M22-M32)+a0t3*(M22+M32)-aet3*(2*M22+M32)-(a0t2*(M14+M
     &  12*M22-M12*M32-M22*M32-M32**2+a0t3*(2*M22+M32)))/M22+(aet2*(M14
     &  +2*M12*M22-2*M2232-M12*M32-M34))/M22+(-((a0t1*(a0t2*abr11**2*ab
     &  r13+M22*(abr11**2*abr14+a0t3*(3*M14+2*M12*(M22-3*M32)+3*(M22-M3
     &  2)**2))))/M22)+abr11*(-M16+M12*(M22-M32)**2-M14*(M22+M32)+(M22-
     &  M32)**2*(M22+M32)+ak0t0*(M14+3*(M22-M32)**2-4*M12*(M22+M32)))+a
     &  et3*(-3*M16-M14*(M22-7*M32)-(M22-M32)**2*(3*M22-M32)+M12*(6*M22
     &  32+7*M24-5*M34))+a0t3*(3*(M22-M32)**3-M14*(M22+3*M32)-2*M12*(2*
     &  M2232+M24-3*M34))+aet1*(-M1422-M16+(M22-2*M32)*(M22-M32)**2+M12
     &  *(M24+3*M34))+(aet2*(-M18-M16*(M22-3*M32)+(-(M22)+M32)**2*M32*
     &  (-3*M22+M32)+M14*(3*M2232+M24-2*M34)-M12*(M2234-9*M2432-M26+M36
     &  )))/M22+(a0t2*(M1226-3*M1424+2*M1434+M18-M16*M22+2*M24**2-4*M22
     &  *M2432-9*M1224*M32-3*M1422*M32-3*M16*M32-M22**3*M32-M22*M32**3+
     &  a0t3*(10*M1224-2*M1234+M14*M22-3*M22**3-5*M2234+7*M2432+M14*M32
     &  +M32**3)+7*M1222*M34+2*M2232*M34+3*M24*M34+M12*M36-M38))/M22)/a
     &  lambda)/4.

        coef(8)=(-(a0t3*abr14*M24)+aet2*M22*(-M14+(M22-M32)*M32+M12*(M22
     &  +2*M32))+a0t2*M22*(a0t3*abr11+M14-3*M12*M22+2*M24-2*M12*M32-3*M2
     &  2*M32+M34)+aet3*(-M16+M26-M24*M32+2*M14*(M22+M32)-M12*(2*M24-2*
     &  M22*M32+M34))+a0t1*(-(a0t2*abr13*M22)-abr14*M22**2-a0t3*(M14+3*M2
     &  4-2*M22*M32-2*M12*(M22+M32)+M34))+M22*(-(M22*(M14-M24+2*M12*M32
     &  +M34))+ak0t0*(2*M14-5*M12*M22+3*M24-4*M12*M32-5*M22*M32+2*M34))
     &  +aet1*(M26-M14*M32-2*M24*M32+2*M22*M34+M12*(-M24+2*(M22*M32+M34
     &  ))-M36))/alambda
        endif
      endif
      end subroutine FD211C






